using Parameters, Chain
using Distributions
using CSV, DataFrames, DataFramesMeta
using Interpolations, Plots
using Statistics, StatsBase
using NLsolve, Roots

function find_percentile_columns(df::AbstractDataFrame, suffix::String="")
  pctiles = Float64[]
  for name in names(df)
    p = @chain name begin
      replace(suffix => "")
      replace("_" => ".")
      replace("p" => "")
    end
    try push!(pctiles, parse(Float64, p))
    catch
    end
  end
  sort(pctiles)
end

function discretize_sumstats(; infile::String, varname::String, by=[:year, :male], nvar::String, kwargs...)

  # load input data (percentiles)
  indata = DataFrame(CSV.File(infile))
  rename!(indata, names(indata) .=> [replace(n, varname => "") for n in names(indata)]);

  # find percentile variables
  percentiles = find_percentile_columns(indata, varname)
  
  # discretize distribution
  outdata = @chain indata begin
    groupby(by)
    combine(df -> construct_pmf_x(df, 1; percentiles=percentiles, kwargs...))
  end

  # drop never-filled bins
  dropdata = combine(groupby(outdata, [:lb]), :pmf => sum => :sum_pmf)
  outdata = leftjoin(outdata, dropdata; on=:lb)
  @subset!(outdata, :sum_pmf .> 0)
  select!(outdata, Not(:sum_pmf))

  # add N in each bin
  if nvar in names(indata)
    dfN = select(indata, [by; nvar])
    rename!(dfN, nvar => "Ntotal")
    outdata = leftjoin(outdata, dfN; on = by)
    outdata.N = outdata.pmf .* outdata.Ntotal
    select!(outdata, Not("Ntotal"))
  end

  outdata

end


addbounds(p) = [0;p;100]
perc_col(p, suffix::String="") = isinteger(p) ? "p$(Int(p))$suffix" : replace("p$(p)$suffix", "." => "_")


function impute_fullycensored_rows!(df::AbstractDataFrame; censorvar::String, impcols)
  df.imputed = falses(nrow(df))
  for row in 2:nrow(df)
    if df.fullycensored[row]
      if !df.fullycensored[row-1]
        df[row,impcols] = copy(df[row-1, impcols])
        df.fullycensored[row] = false
        df.imputed[row] = true
      end
    end
  end
  for row in reverse(1:nrow(df)-1)
    if df.fullycensored[row]
      if !df.fullycensored[row+1]
        df[row,impcols] = copy(df[row+1, impcols])
        df.fullycensored[row] = false
        df.imputed[row] = true
      end
    end    
  end
end

function get_cdf_knots(df::AbstractDataFrame, row::Int; percentiles, suffix="")
  _nonmissing = [!ismissing(df[row,perc_col(p,suffix)]) for p in percentiles]
  p_nonmissing = percentiles[_nonmissing]
  q = [df[row,perc_col(p,suffix)] for p in p_nonmissing]
  cdf_q = p_nonmissing ./ 100
  q, cdf_q
end

function fit_cdf_mean(df::AbstractDataFrame, row::Int; lb, percentiles, kwargs...)
  q, cdf_q = get_cdf_knots(df, row; percentiles=percentiles)
  @assert lb <= minimum(q) "lb=$lb, minimum(q)=$(minimum(q))"
  _fit_cdf_mean([lb; q], [0.0; cdf_q]; bracket_ub=_bracket_ub(q), target_mean=df[row,"mean"], kwargs...)
end
function fit_cdf_std_skew(df::AbstractDataFrame, row::Int; percentiles, kwargs...)
  q, cdf_q = get_cdf_knots(df, row; percentiles=percentiles)
  _fit_cdf_std_skew(q, cdf_q; target_std=df[row,"sd"], target_skew=df[row,"skew"], kwargs...)
end
function fit_cdf_max(df::AbstractDataFrame, row::Int; lb, percentiles, target_max, kwargs...)
  q, cdf_q = get_cdf_knots(df, row; percentiles=percentiles)
  target_max = max(q[end] + diff(q)[end], target_max)
  _fit_cdf_max([lb; q], [0.0; cdf_q]; target_max=target_max, kwargs...)
end

function _fit_cdf(q, cdf_q; kwargs...)
  @assert issorted(q)
  @assert issorted(cdf_q)
  @assert all(cdf_q[2:end-1] .< 1)
  @assert cdf_q[1] ≈ 0
  @assert cdf_q[end] ≈ 1
  Fopt = interpolate_cdf(q, cdf_q; kwargs...)  
end

function _fit_cdf_max(q, cdf_q; n::Int=100, loggrid::Bool=false, target_max, kwargs...)
  @assert issorted(q)
  @assert issorted(cdf_q)
  @assert all(cdf_q .< 1)
  Fopt = interpolate_cdf([q; target_max], [cdf_q; 1.0]; kwargs...)  
end

function _fit_cdf_mean(q, cdf_q; n::Int=100, loggrid::Bool=false, bracket_ub=_bracket_ub(q), target_mean, kwargs...)
  @assert issorted(q)
  @assert issorted(cdf_q)
  @assert all(cdf_q .< 1)
  function f(ub)
    ub = max(ub, q[end])
    F = interpolate_cdf([q; ub], [cdf_q; 1.0]; kwargs...)
    D = discretize_distribution(F, n; loggrid=loggrid)
    mean(D) - target_mean
  end
  if prod(f.(bracket_ub)) >= 0 
    Fopt = interpolate_cdf([q; q[end]], [cdf_q; 1.0]; kwargs...)  
  else
    ubopt = max(find_zero(f, bracket_ub), q[end])
    Fopt = interpolate_cdf([q; ubopt], [cdf_q; 1.0]; kwargs...)
  end
end
_bracket_ub(q) = (q[end], 20*q[end])

function _fit_cdf_std_skew(q, cdf_q; n::Int=100, loggrid=false, guess=_guess_lbub(q), target_std, target_skew, kwargs...)
  @assert issorted(q)
  @assert issorted(cdf_q)
  @assert all(cdf_q .< 1)
  function f!(res, bounds)
    lb = min(bounds[1], q[1])
    ub = max(bounds[2], q[end])
    F = interpolate_cdf([lb; q; ub], [0.0; cdf_q; 1.0]; kwargs...)
    D = discretize_distribution(F, n; loggrid=loggrid)
    res[1] = std(D) - target_std
    res[2] = skewness(D) - target_skew
    res
  end
  # nlout = nlsolve(f!, guess)
  nlout = mcpsolve(f!, [-Inf, q[end]], [q[1], Inf], guess, xtol=1e-6, ftol=1e-8, reformulation = :smooth, autodiff = :forward)
  lb = min(nlout.zero[1], q[1])
  ub = max(nlout.zero[2], q[end])
  Fopt = interpolate_cdf([lb; q; ub], [0.0; cdf_q; 1.0]; kwargs...)
end
_guess_lbub(q) = 1.2*[q[1], q[end]]

# Discretize an interpolated CDF and return a DiscreteNonParametric distribution
function discretize_distribution(cdf::AbstractInterpolation, nbins::Int; loggrid::Bool=false)
  lb = myknots(cdf)[1]
  ub = myknots(cdf)[end]
  if loggrid
    grid = 10 .^ range(log10(lb-1e-10), stop=log(ub+1e-10), length=nbins+1)
  else
    grid = range(lb-1e-10, stop=ub+1e-10, length=nbins+1)
  end
  pmf = diff(mycdf.(Ref(cdf), grid))
  @assert isprobvec(pmf) "$(sum(pmf))"
  mid = (grid[1:end-1] + grid[2:end]) ./ 2
  DiscreteNonParametric(mid, pmf)
end
function discretize_distribution(cdf::AbstractInterpolation, grid::AbstractVector)
  pmf = diff(mycdf.(Ref(cdf), grid))
  @assert isprobvec(pmf) "$(sum(pmf))"
  mid = (grid[1:end-1] + grid[2:end]) ./ 2
  DiscreteNonParametric(mid, pmf)
end

function interpolate_cdf(quantile::AbstractVector, cdf::AbstractVector; itptype=FritschCarlsonMonotonicInterpolation())
  @assert length(quantile) == length(cdf)
  df = combine(groupby(DataFrame(grid=quantile, value=cdf), :grid), :value => maximum => :value)
  interpolate(myknots(df.grid, itptype), df.value, itptype)
end

myknots(x, itp::Interpolations.Gridded) = (x,)
myknots(x, itp::Interpolations.MonotonicInterpolationType) = x
myknots(itp::Interpolations.GriddedInterpolation) = itp.knots[1]
myknots(itp::Interpolations.MonotonicInterpolation) = itp.knots



function mycdf(itp::Interpolations.AbstractInterpolation, x::Real)
  if x < myknots(itp)[1] 
    return 0.0
  elseif x > myknots(itp)[end]
    return 1.0
  else  
    return itp(x)
  end
end

function myinvcdf(itp::Interpolations.GriddedInterpolation, p::Real)
  if p < itp.coefs[1] return myknots(itp)[1]
  elseif p > itp.coefs[end] return myknots(itp)[end] 
  else
    i = findfirst(x -> x >= p, itp.coefs)
    f0, f1 = itp.coefs[i-1], itp.coefs[i]
    x0, x1 = myknots(itp)[i-1], myknots(itp)[i]
    return invcdf = x0 + (p - f0) * (x1-x0) / (f1-f0)
  end
end

function construct_pmf_x(df::AbstractDataFrame, row::Int; lbs::AbstractVector, kwargs...)
  ubs_fallback = lbs[end] + diff(lbs)[end]
  F = in("mean", names(df)) ? fit_cdf_mean(df, row; kwargs...) : fit_cdf_max(df, row; target_max=ubs_fallback, kwargs...)
  D = discretize_distribution(F, [lbs; Inf])
  max_supF = myknots(F)[end]
  ubs = max_supF > lbs[end] ? [lbs[2:end]; max_supF] : [lbs[2:end]; lbs[end] + lbs[end]-lbs[end-1]]
  @assert issorted(ubs)
  DataFrame(lb=lbs; ub=ubs, pmf=probs(D))
end

function compute_standard_moments(df::AbstractDataFrame)
  Ey = mean(disallowmissing(df.mean), weights(disallowmissing(df.pmf_x)))
  Ey2 = mean(disallowmissing(df.m2), weights(disallowmissing(df.pmf_x)))
  Ey3 = mean(disallowmissing(df.m3), weights(disallowmissing(df.pmf_x)))
  Ey4 = mean(disallowmissing(df.m4), weights(disallowmissing(df.pmf_x)))
  V = Ey2 - Ey^2
  skew = (Ey3 - 3*Ey*V - Ey^3) / V^(3/2)
  kurt = Ey4 / V^2
  DataFrame(sd=sqrt(V), skew=skew, kurt=kurt)
end


function construct_joint_distribution(df::AbstractDataFrame;  nbins::Int=1000, 
                                                  margpmf=df.N / sum(df.N),
                                                  kwargs...)

  out = DataFrame(bin_x=repeat(df.bin_x', nbins, 1)[:],
                  lb_x=repeat(df.lb_x', nbins, 1)[:],
                  ub_x=repeat(df.ub_x', nbins, 1)[:],
                  pmf_x=repeat(margpmf', nbins, 1)[:],
                  bin_y = repeat(1:nbins, nrow(df), 1)[:]
                  )

  Fvec = [fit_cdf_std_skew(df, row; kwargs...) for row in 1:nrow(df)]

  # Gloabl grid
  mingrid = minimum([get_gridmin(F) for F in Fvec]) - 1e-10
  maxgrid = maximum([get_gridmax(F) for F in Fvec]) + 1e-10
  grid = range(mingrid, stop=maxgrid, length=nbins+1)
  out[!,:lb_y] = repeat(grid[1:end-1], nrow(df), 1)[:]
  out[!,:ub_y] = repeat(grid[2:end], nrow(df), 1)[:]

  # conditional pmf of y given x
  out.pmf_y_x = zeros(nrow(df)*nbins)
  for i in 1:nrow(df)
    _rows = (i-1)*nbins+1:i*nbins
    D = discretize_distribution(Fvec[i], grid)
    out[_rows, :pmf_y_x] .= probs(D) 
  end

  # joint pmf of y and x
  out.pmf = out.pmf_x .* out.pmf_y_x

  out
end

# Recover marginal distribution: P(y) = ∑ₓ P(y,x)
function create_marginal_distribution(df::AbstractDataFrame)
  out = combine(groupby(df, [:bin_y, :lb_y, :ub_y]), :pmf => sum => :pmf_y)
  @assert abs(sum(out.pmf_y) - 1) < 1e-4 # TODO why not always = 1
  out.pdf_y = out.pmf_y ./ (out.ub_y - out.lb_y)
  out
end  

get_gridmin(itp::Interpolations.AbstractInterpolation) = myknots(itp)[1]
get_gridmax(itp::Interpolations.AbstractInterpolation) = myknots(itp)[end]

function summary_stats(y, w; N=missing, percentiles)
  # @assert false
  @assert isprobvec(w[:]) "sum of weights: $(sum(isnan.(w)) / length(w))"
  D = DiscreteNonParametric(y[:], w[:])
  @assert issorted(y[:])
  @assert all(w .>= 0)
  wclean = w / sum(w)
  F = interpolate((y,), cumsum(w[:])/sum(w), Gridded(Linear()))
  N = ismissing(N) ? missing : Int(round(N))
  base = (N = N,
          mean = mean(D),
          sd   = std(D),
          max  = maximum(D),
          min  = minimum(D),
          skew = skewness(D),
          kurt = kurtosis(D, false),
          gini = gini(y[:], wclean[:])
          )
  pnames = [Symbol(perc_col(p)) for p in percentiles]
  # pctiles = [quantile(D, p/100) for p in percentiles]
  pctiles = [myinvcdf(F, p/100) for p in percentiles] # TODO 
  quants = NamedTuple{Tuple(pnames)}(Tuple(pctiles))
  merge(base, quants)
end

function gini(y, w)
  @assert length(y) == length(w)
  S = cumsum(y .* w)
  G = S[1]*w[1] +
              sum(w[2:end] .* 
                      (S[2:end]+S[1:end-1]))
  return 1 - G/S[end]
end



plot_itpcdf(args...; kwargs...) = plot_itpcdf!(Plots.plot(), args...; kwargs...)
function plot_itpcdf!(plt::Plots.Plot, itp::Interpolations.AbstractInterpolation; kwargs...)
  x = range(minimum(myknots(itp)); stop=maximum(myknots(itp)), length=100)
  y = itp.(x)
  scatter!(plt, myknots(itp), itp.(myknots(itp)); label="Knots", kwargs...)
  plot!(plt, x, y; label="Interpolation", kwargs...)
end


function check_plot(d1::DataFrame, d2::DataFrame, var::Vector{String})
  plt = plot(xticks=unique(d1.year), legend=:topright, xlabel="", ylabel="")
  for (i,v) in enumerate(var)
    j = (i-1)*2
    cmen = j+1
    cwomen = j+2
    plot!(d1.year[d1.male .== 1], d1[d1.male .== 1, v], c=cmen, w=2, label="$v Men")    
    plot!(d1.year[d1.male .== 0], d1[d1.male .== 0, v], c=cwomen, w=2, label="$v Women")
    plot!(d2.year[d2.male .== 1], d2[d2.male .== 1, v], style=:dash, c=cmen, w=2, label="")
    plot!(d2.year[d2.male .== 0], d2[d2.male .== 0, v], style=:dash, c=cwomen, w=2, label="")
  end
  plot(plt)
end


function optimal_bandwidth(y::AbstractVector, args...)
  s = std(y, args...)
  p25 = quantile(y, args..., 0.25)
  p75 = quantile(y, args..., 0.75)
  m = min(s, (p75-p25)/1.349)
  h = 0.9*m / length(y)^0.2
end