function sumstats_volatility(;  varname::String,
                                by::Vector{String},
                                path_cond_growth_dist::AbstractString,
                                path_income_dist::AbstractString,
                                percentiles_stats::AbstractVector
                                )


  #' # Data Preparation
  # ================================================================

  #' ## Load conditional earnings growth data
  data = DataFrame(CSV.File(path_cond_growth_dist));

  #' find percentile variables
  percentiles_growth = find_percentile_columns(data, varname);

  #' Change variable names
  rename!(data, names(data) .=> [replace(n, "$varname" => "") for n in names(data)])
  rename!(data, "bin" => "bin_x")
  rename!(data, "lb" => "lb_x")
  rename!(data, "ub" => "ub_x")
  data.mid_x = (data.ub_x + data.lb_x) / 2
  select!(data, Not(:ub_x)); 
  
  #' Rename N variable (we will use N from earnings distribution data (no missings))
  rename!(data, "N" => "N_growthdata")

  #' ## Marginal Distribution of Earnings
  lbs_x = sort(unique(data.lb_x));
  pmf_x = discretize_sumstats(; infile=path_income_dist, by=by, varname="labor", nvar="N", lb=minimum(lbs_x), lbs=lbs_x, loggrid=true)
  dfN = combine(groupby(pmf_x, by), :N => sum => :N)


  #' Change variable names
  rename!(pmf_x, :lb => :lb_x)
  rename!(pmf_x, :ub => :ub_x)
  rename!(pmf_x, :pmf => :pmf_x)

  #' ## Merge earnings distribution to growth data
  data = leftjoin(pmf_x, data, on = [by; "lb_x"])
  replace!(data.pmf_x, missing => 0)
  disallowmissing!(data, :pmf_x)
  

  #' ## Fully Censored Data                
  #' We do not have any data for some of the very top earnings bins because N < 30 (especially for women)
  data.fullycensored = ismissing.(data.mean);

  #' ## Imputation of Fully Censored Data
  #' Impute information from the last available earnings bin with non-missing data. In other cases, only some percentiles are censored and we will impute those later.
  impcols = ["mean"; "sd"; "skew"; "kurt"; "m2"; "m3"; "m4"; perc_col.(percentiles_growth)]
  impute_fullycensored_rows!(data; censorvar="fullycensored", impcols=impcols)

  #' # Distribution of earnings growth
  # ================================================================

  #' 1. interpolate conditional distributions (each row separately) using monotonic cubic spline interpolation (Fritsch-Carlson algorithm). Thereby the minimum and maximum of the distribution is chosen such that the standard deviation and skewness of the interpolated distribution match the values in the data
  #' 2. find the global minimum and maximum and construct a fine grid
  #' 3. discretize the conditional (continuous) distribution on a fine grid
  #' 4. compute the joint distribution by weighting the conditional pmf with the distribution of earnings bins

  #' ## Joint and conditional distribution
  # -----------------------------------------------------------------
  PMF = @chain data begin
    groupby(by)
    combine(df -> construct_joint_distribution(df; margpmf=df.pmf_x, nbins=1000, percentiles=percentiles_growth))
  end

  #' ## Unconditional distribution
  # -----------------------------------------------------------------

  #' Compute from joint distribution: P(y) = ∑ₓP(y,x)
  pmf_y = @chain PMF begin
    groupby(by)
    combine(df -> create_marginal_distribution(df))
  end
  pmf_y = leftjoin(pmf_y, dfN, on=by)


  #' #### Export summary statistics
  sumstats = @chain pmf_y begin
    groupby(by)
    combine(df -> summary_stats((df.lb_y + df.ub_y) ./ 2, df.pmf_y; N=df.N[1], percentiles=percentiles_stats))
  end

  return (sumstats=sumstats, discr_dist=pmf_y, imputed_data=data)

end


