*! v20220530 hw
/*---------------------------------------------------------*/
*! Researchproject: Inequality and Inceom Dynamics in Germany (GRID project)
*! Author: Johannes F. Schmieder / Stefanie Wolter / Moritz Drechsel-Grau / Kai D. Schmid / Hannes Walz
/*---------------------------------------------------------*/


// =============== Preamble ================ //

{
//--- Settings ---//

* clearing
clear all
program drop _all
macro drop _all
cap log close _all
clear matrix

* set
version 16.1
set more off, perm
set varabbrev off, perm
set matsize 10000, perm
set maxiter 1000, perm
set iterlog on, perm
set linesize 255
set pagesize 100
set rmsg off, perm
set trace on
set tracedepth 2
set traceexpand off, perm
set tracesep off, perm
set traceindent off, perm
set tracenumber off, perm    

* date
global mydate: di %td_CY-N-D date("$S_DATE", "DMY")
global mydate=strltrim("$mydate")

* seed
set seed 190424


//--- paths ---//

* checks
pwd 
sysdir

* paths
global root     = "...\" 	 				// !!! change main dir path here !!!
	global do     = "${root}`c(dirsep)'do`c(dirsep)'"
	global data     = "${root}`c(dirsep)'dta`c(dirsep)'"
	global log      = "${root}`c(dirsep)'log`c(dirsep)'"
	global orig     = "${root}`c(dirsep)'orig`c(dirsep)'"
	global figs     = "${root}`c(dirsep)'figs`c(dirsep)'"
	global out      = "${root}`c(dirsep)'out`c(dirsep)'"	
foreach dir in "log" "data" "figs" "out" {
	cap mkdir "$`dir'"
}
cd "${root}"

* main log path
global mainlog "${log}${mydate}"
cap mkdir "${log}${mydate}"

* set project ado dir
adopath ++ "${do}/"

*check ado paths
adopath
}

/*===========================================================================================*/
/*                                      Main Program                                         */
/*===========================================================================================*/
capture program drop main
program define main
	set matsize 4000
	set li 120 // 200
	set trace on
	set tracesep off
	set tracedepth 1 //2
  
	local time = subinstr("`=c(current_time)'",":","",.)
	scalar d = date("`=c(current_date)'","DMY") 	// create log file including time and date in file name
	local b  _`=subinstr("`c(current_date)'"," ","",.)'_`time'
	global logfile ${mainlog}/${mydate}_GRID_Germany_IAB_input.log
	log using $logfile , text replace name(IAB_input_joint_analyses_`b')
	
	
	// =====================================
    // IAB Core Analysis
    // =====================================
	
	*** adjusted core code
	run_core_master_code, vsample(short) wominijob(0) dfopt(data)
	run_core_master_code, vsample(long) wominijob(1) dfopt(data)
	
	*** Distributions as IAB input for combined analyses
	distributions_iab
	
	
	// =====================================
    // Descriptive Statistics & Tables
    // =====================================

	*** descriptive tables
	core_tables, folderdate($mydate) vsample(short)
	desc_table_content_rest

	*** job transition probabilities
	jobtype_trans_prob1	
	jobtype_trans_prob2
	
	
	// =====================================
	// Different dependent variables 
    // =====================================	
	
	*** gen data for diff. dep var cohort plots
	data_diff_dep_var_cohort	
	
	
	cap log IAB_input_joint_analyses_`b'

end // main program

/*===========================================================================================*/
/*                                   Main Programs                                           */
/*===========================================================================================*/

/*---------------------------------------------*/
/* Run Core Master Code */
/*---------------------------------------------*/
cap program drop run_core_master_code
program define run_core_master_code
	syntax , vsample(str) dfopt(str) [wominijob(int 0)]

	//--- long or short sample ---//

	global vsample="`vsample'"			
	if "${vsample}"=="short" global yrsample=2001
	if "${vsample}"=="long" global yrsample=1993
	if "`vsample'"=="" {
	    di as error "Sample not defined!"
		exit 198
	}
	global rvl=""

	* real minimum income threshold
	global wo_mini_inc_threshold=`wominijob'  			// !!! set to 1 for using REAL MINIMUM INCOME THRESHOLD w/o mini jobs, to 0 /w mini jobs !!!
	
	//--- run it ---//	
	
	* 0
	global DATAFILEOPT="`dfopt'"
	do "${do}0_Initialize.do"

	* 1
	do "${do}1_Gen_Base_Sample.do"

	* 2
	do "${do}2_DescriptiveStats.do"

	* 3
	do "${do}3_Inequality.do"

	* 4
	do "${do}4_Volatility.do"

	* 5
	do "${do}5_Mobility"
	
	* 7
	do "${do}7_Paper_Figs_`vsample'.do"
	
	
end // run master code	

/*---------------------------------------------*/
/* Distributions */
/*---------------------------------------------*/
cap program drop distributions_iab
program define distributions_iab

	** inequality
	distributions_do, dfopt(data) privacy(1)

	** volatility & mobility
	distributions_vol_mob

end // distributions_iab

/*---------------------------------------------*/
/* Core Tables */
/*---------------------------------------------*/
cap program drop core_tables
program define core_tables
	syntax, folderdate(str) vsample(str)

	// Date of (the start of ) the analysis
	global date_analysis = "`folderdate'"
	
	global vsample="`vsample'"			
	if "${vsample}"=="short" global yrsample=2001
	if "${vsample}"=="long" global yrsample=1993	
	global yrfirst = ${yrsample} 		// First year in the dataset 	
	global yrlast =  2018 		// Last year in the dataset 	

	// Where the firgures are going to be saved
	global folderfile = "${out}/${date_analysis} Descriptives - `vsample'"
	cap mkdir "${folderfile}"	
	
	
	//--- sample descriptives table ---//		
	
	desc_table, vsample(`vsample')
	
	
	//--- percentiles table ---//	
	
	// All
	clear
	import delimited using "${out}/${date_analysis} Descriptive_Stat - `vsample'/l_cs_labor_yr_sum_stats", clear
	perc_earn_table using "${folderfile}/percentiles_earnings_`vsample'.tex", years($yrfirst/$yrlast) varstub(labor)

	// Male
	clear
	import delimited using "${out}/${date_analysis} Descriptive_Stat - `vsample'/l_cs_labor_yrgender_sum_stats", clear
	keep if male == 1
	perc_earn_table using "${folderfile}/percentiles_earnings_male_`vsample'.tex", years($yrfirst/$yrlast) varstub(labor)

	// Female
	clear
	import delimited using "${out}/${date_analysis} Descriptive_Stat - `vsample'/l_cs_labor_yrgender_sum_stats", clear
	keep if male == 0
	perc_earn_table using "${folderfile}/percentiles_earnings_female_`vsample'.tex", years($yrfirst/$yrlast) varstub(labor)
	clear
	
end // core_tables	
	
/*---------------------------------------------*/
/* Descriptive table stats of missing entries */
/*---------------------------------------------*/
cap program drop desc_table_content_rest
program desc_table_content_rest

//--- Prep ---//

	use "${data}/clean_yearly_data", clear

	replace age=year-byear+1
	recode educ4 est_size (.=9)
	recode akm_FE_pctl (.=99)
	replace days_work_c=365-days_work_c
	
	//preserve
	keep if age>=25 & age <=55
	drop if mi(earn_all_imputed)

	gen below_med=.
	forval yy=2001/2018 {
	    sum rearn_all_imputed if year==`yy', de
		replace below_med=0 if year==`yy' & rearn_all_imputed>=r(p50)
		replace below_med=1 if year==`yy' & rearn_all_imputed<r(p50)
	}	
	
//--- Values for in-text ---//	
	
	tab year if below_med==1 & male==1, sum(days_work_c)
	tab year if below_med==0 & male==1, sum(days_work_c)

	tab year if below_med==1 & male==0, sum(days_work_c)
	tab year if below_med==0 & male==0, sum(days_work_c)

	tab year if male==1, sum(days_work_c)
	tab year if male==0, sum(days_work_c)
	
	gen jt1=jobtype2==1
	gen jt2=jobtype2==2
	gen jt3=jobtype2==3
	
	tab year if male==1, sum(jt1)
	tab year if male==1, sum(jt2)
	tab year if male==1, sum(jt3)
	
	tab year if male==0, sum(jt1)
	tab year if male==0, sum(jt2)
	tab year if male==0, sum(jt3)		
	
	gen cs=educ2==1 //college share
	tab year if male==1, sum(cs)
	tab year if male==0, sum(cs)	

	tab year if male==1, sum(age)
	tab year if male==0, sum(age)		
	
	tab year if male==1, sum(nonger)
	tab year if male==0, sum(nonger)	
	
//--- Desc table ---//

	
	keep if year==2008

	gen ft=jobtype2==1
	gen pt=jobtype2==2
	gen mj=jobtype2==3

	gen ed1=educ4==1
	gen ed2=educ4==2
	gen ed3=educ4==3
	gen ed4=educ4==4
	gen ed9=educ4==9

	collapse ft pt mj nonger ed1 ed2 ed3 ed4 ed9 days_work_c, by(male)

	tabstat ft pt mj nonger ed1 ed2 ed3 ed4 ed9 days_work_c, by(male) s(mean)

	restore
	
end //desc_table_content_rest

/*---------------------------------------------*/
/* Jobtype transition probabilities (imp) */
/*---------------------------------------------*/
cap program drop jobtype_trans_prob2
program jobtype_trans_prob2	
	
//--- init ---//

	* load and set folder
	cap mkdir "${out}/${mydate} Transition probs/"
	use "${data}/clean_yearly_data", clear

	* settings and init for plots
	global vsample="short"
	global yrsample=2001
	global rvl=""
	global wo_mini_inc_threshold=0
	* do files needed
	run "${do}/0_Initialize.do"
	run "${do}/myplotsettings"
	* locals settings
	local colors blue red
	local fcolors = ""
	foreach co of local colors {
		local fcolors = "`fcolors'" + " " + "`co'*0.25"
	}		
	*Recessions bars 
	gen rece = inlist(year,${receyears})	
	
	
//--- adjustments ---//	
	
	replace age=year-byear+1	
	
	keep if age>=25 & age <=55
	drop if mi(earn_all_imputed)
	drop if year<2001 
	drop if year>2017
	drop if rearn_all_imputed<2300 
	
	xtset persid year
	
	tsfill, full
	
	bysort persid: egen temp1=min(byear)
	replace byear=temp1 if byear==.
	replace age=year-byear+1	
	bysort persid: egen temp2=min(male)
	replace male=temp2 if male==.
	
	recode jobtype2 (.=9)
	
	gen F_jobtype2=F.jobtype2
	
	gen F_FT=F_jobtype2==1
	gen F_PT=F_jobtype2==2
	gen F_MJ=F_jobtype2==3
	gen F_No=F_jobtype2==9
	
	drop if year==2017
	keep if age>=25 & age <=55
	
	collapse (mean) F_FT F_PT F_MJ F_No (count) N=persid , by(year male jobtype2)
	
	outsheet using "${out}/${mydate} Transition probs/transition_probabilities_jobtype2.csv", replace comma nol

//--- grpahs ---//			
	
	*Recessions bars 
	gen rece = inlist(year,${receyears})		
	* locals settings
	local colors blue red
	local fcolors = ""
	foreach co of local colors {
		local fcolors = "`fcolors'" + " " + "`co'*0.25"
	}		
	
	** plots
	forval m=0/1 {
	    foreach jt of numlist 1 2 3 9 {
			preserve
			keep if `m'==male
			keep if `jt'==jobtype2
			
			twoway (bar rece year, c(l) color(gray*0.5) yscale(off)) ///
    	       (connected F_FT F_PT F_MJ F_No  year, ///
        			lcolor(`colors')  /// Line colors
        			lpattern(solid longdash) ///
        			msymbol(O S) /// Markers
        			mlcolor(`colors')  /// Marker  line color
                    mfcolor(`fcolors')  ///	Fill color
        			yaxis(2)  yscale(alt axis(2)) ///
                    ytitle("Shares of working time", axis(2)) ///
                    ylabel(0(0.2)1, axis(2)) ///
                ),  ///
        	xtitle("") ///
            xlabel(2000(2)2014, grid) ///		xaxis options
        	legend(position(9) order(2 "Full-time" 3 "Part-time" 4 "Minijob" 5 "Out of Workforce"))
			cap noisily: graph export "${out}/${mydate} Transition probs/fig1_jt2_m`m'_jt`jt'_v1.pdf", replace 	
	
			restore
		}
	}

	
end //jobtype_trans_prob2		
	
/*---------------------------------------------*/
/* Jobtype transition probabilities (un-imp) */
/*---------------------------------------------*/
cap program drop jobtype_trans_prob1
program jobtype_trans_prob1	
	
//--- init ---//

	* load and set folder
	cap mkdir "${out}/${mydate} Transition probs/"
	use "${data}/clean_yearly_data", clear

	* settings and init for plots
	global vsample="short"
	global yrsample=2001
	global rvl=""
	global wo_mini_inc_threshold=0
	* do files needed
	run "${do}/0_Initialize.do"
	run "${do}/myplotsettings"
	* locals settings
	local colors blue red
	local fcolors = ""
	foreach co of local colors {
		local fcolors = "`fcolors'" + " " + "`co'*0.25"
	}		
	*Recessions bars 
	gen rece = inlist(year,${receyears})	
	
	
//--- adjustments ---//	
	
	replace age=year-byear+1	
	
	keep if age>=25 & age <=55
	drop if mi(earn_all_imputed)
	drop if year<2001 
	drop if year>2017
	drop if rearn_all_imputed<2300 
	
	xtset persid year
	
	tsfill, full
	
	bysort persid: egen temp1=min(byear)
	replace byear=temp1 if byear==.
	replace age=year-byear+1	
	bysort persid: egen temp2=min(male)
	replace male=temp2 if male==.
	
	recode jobtype1 (.=9)
	
	gen F_jobtype1=F.jobtype1
	
	gen F_FT=F_jobtype1==1
	gen F_PT=F_jobtype1==2
	gen F_MJ=F_jobtype1==3
	gen F_No=F_jobtype1==9
	
	drop if year==2017
	keep if age>=25 & age <=55
	
	collapse (mean) F_FT F_PT F_MJ F_No (count) N=persid , by(year male jobtype1)
	
	outsheet using "${out}/${mydate} Transition probs/transition_probabilities_jobtype1.csv", replace comma nol

//--- grpahs ---//			
	
	*Recessions bars 
	gen rece = inlist(year,${receyears})		
	* locals settings
	local colors blue red
	local fcolors = ""
	foreach co of local colors {
		local fcolors = "`fcolors'" + " " + "`co'*0.25"
	}		
	
	** plots
	forval m=0/1 {
	    foreach jt of numlist 1 2 3 9 {
			preserve
			keep if `m'==male
			keep if `jt'==jobtype1
			
			twoway (bar rece year, c(l) color(gray*0.5) yscale(off)) ///
    	       (connected F_FT F_PT F_MJ F_No  year, ///
        			lcolor(`colors')  /// Line colors
        			lpattern(solid longdash) ///
        			msymbol(O S) /// Markers
        			mlcolor(`colors')  /// Marker  line color
                    mfcolor(`fcolors')  ///	Fill color
        			yaxis(2)  yscale(alt axis(2)) ///
                    ytitle("Shares of working time", axis(2)) ///
                    ylabel(0(0.2)1, axis(2)) ///
                ),  ///
        	xtitle("") ///
            xlabel(2000(2)2014, grid) ///		xaxis options
        	legend(position(9) order(2 "Full-time" 3 "Part-time" 4 "Minijob" 5 "Out of Workforce"))
			cap noisily: graph export "${out}/${mydate} Transition probs/fig1_jt1_m`m'_jt`jt'_v1.pdf", replace 	
	
			restore
		}
	}

end //jobtype_trans_prob1	
	
/*---------------------------------------------*/
/* Data for alternative dep var cohort plots */
/*---------------------------------------------*/
cap program drop data_diff_dep_var_cohort
program define data_diff_dep_var_cohort

	global folderfile="${out}/${mydate} Data ddv cohort plot"
	cap mkdir "${folderfile}"
	
	use "${data}/clean_yearly_data", clear

	gen age=year-byear+1
	
	keep if age>=25 & age<=55
	
	replace lrearn_all_imputed=. if rearn_all_imputed<2300
	replace rearn_all_imputed=. if rearn_all_imputed<2300

	recode educ4 (.=9 )
	lab def educ4lbl 9 "missing" , add 
	lab val educ4 educ4lbl
	gen educ2=educ4
	recode educ2 (1 2 3=1) (4=2) (.=9)
	lab def educ2lbl 1 "lower education" 2 "higher education (at least college)" 9 "missing"
	lab val educ2 educ2lbl
	
	preserve
	collapse (count) N=lrearn_all_imputed (mean) mean_rearn_all_imputed=rearn_all_imputed, by(year male age educ4 jobtype2)
	
	export delimited "${folderfile}/data_diff_dep_var_cohort_plot_v1_uncencored.csv", nolab replace
	
	replace mean_rearn_all_imputed=. if N<30
	replace N=. if N<30
	
	export delimited "${folderfile}/data_diff_dep_var_cohort_plot_v1.csv", nolab replace
	restore
 
 
	collapse (count) N=lrearn_all_imputed (mean) mean_rearn_all_imputed=rearn_all_imputed, by(year male age educ2 jobtype2)
	
	export delimited "${folderfile}/data_diff_dep_var_cohort_plot_v2_uncencored.csv", nolab replace
	
	preserve
	replace mean_rearn_all_imputed=. if N<30
	replace N=. if N<30
	
	export delimited "${folderfile}/data_diff_dep_var_cohort_plot_v2.csv", nolab replace
	restore
	
	clear
 
end //data_diff_dep_var_cohort	


/*===========================================================================================*/
/*                                    Sub Programs                                           */
/*===========================================================================================*/

/*---------------------------------------------*/
/* Distributions do file (from TPP) */
/*---------------------------------------------*/
cap program drop distributions_do	
program define distributions_do	// dfopt= data set to use: data / data_01perc; privacy(int 1) /w 1== data protecction for cells N<20/30	
	syntax , dfopt(str) [privacy(int 1)]
	
/*
 This program generates the descriptive statistics
 by Moritz Drechsel-Grau & Kai Schmid & Hannes Walz
*/

//------//

clear
global protect = `privacy'

// Create folder for output and log-file
global outfolder="${mydate} 206_Distributions"
capture noisily mkdir "${out}/$outfolder"
cap noi mkdir "${out}/$outfolder/inequality"
capture log close
capture noisily log using "${log}/$outfolder.log", replace

// Initialize Settings
global yrsample=2001
global vsample="short"
global wo_mini_inc_threshold=0
global DATAFILEOPT="`dfopt'"
do "${do}/0_Initialize.do"
do "${do}/1_Gen_Base_Sample_Dist.do"

// cd to the output file, create the program for moments, and load base sample.
cd "${out}/$outfolder"
qui do "${do}/myprogs.do"

timer clear 1
timer on 1

qui {

//============//
//--- Data ---//
//============//

foreach yr of numlist $yrfirst/$yrlast { 

noi disp as text _n "Working in year `yr'"


local yr5F = `yr'+5   // t+5
local yrp  = `yr'-1		// t-1

// Variables to load
local vars = "male yob labor`yr' logearn`yr' researn`yr'"
local flist = ""

if inlist(`yr', $d1yrlistcomma) {
	local vars "`vars' researn1F`yr'"
	local flist "`flist' 1"
}

if inlist(`yr',${d5yrlist}) {
	local vars "`vars' researn5F`yr'"
	local flist "`flist' 5"
}

if inlist(`yrp',${perm3yrlist}) {
	local vars "`vars' permearn`yrp' avg_pastearn`yrp' level_permearn`yrp'"
}

if inlist(`yr', ${perm3yrlist}) & inlist(`yr', ${d5yrlist}) {
	local vars "`vars' permearnalt`yr' respermearnalt`yr' permearnalt`yr5F' respermearnalt`yr5F'"
}


// Load base data
noi disp as text "... load and prepare data"
use `vars' using "${data}/master_sample.dta" if labor`yr'~=. , clear

*** set weight to 1 for IAb samples
gen nonweight=1
global weights = "[aweight=nonweight]" 
global weight_var = "nonweight" 

// Rename variables	
rename labor`yr' labor
rename logearn`yr' logearn
rename researn`yr' researn
cap rename researn1F`yr' researn1F	
cap rename researn5F`yr' researn5F	
cap rename permearn`yrp' permearn
cap rename avg_pastearn`yrp' avg_pastearn
cap rename permearnalt`yr' permearnalt
cap rename permearnalt`yr5F' permearnalt5F
cap rename respermearnalt`yr' respermearnalt
cap rename respermearnalt`yr5F' respermearnalt5F


// Age group
gen age = `yr'-yob+1
gen agegp = .
replace agegp = 1 if age <= 34 & agegp == .
replace agegp = 2 if age <= 44 & agegp == .
replace agegp = 3 if age <= 55 & agegp == .

// Age restriction
drop if $yrfirst - yob + 1 > $end_age | $yrlast - yob + 1 < $begin_age
drop yob

// Generate year
gen year = `yr'

// CS sample indicator
generate CSsample = 1 if logearn != . 


//==================//
//--- Inequality ---//
//==================//


// Labor Income Percentiles
//------------------------------------------------------------------------------------------

noi disp as text "... labor income summary statistics"

foreach var in labor logearn researn {

  // loop over (sub)populations
  foreach het in "" $hetgrouplabor_ineq {
    local suf=subinstr("`het'"," ","",.)
    bymydist_detail `var' if CSsample == 1, saving("inequality/sumstats_`var'_`suf'`yr'") by(year `het') ///
      percentiles(0.1(0.1)0.9 1(1)99 99.1(0.1)99.9 99.95 99.99 99.995 99.999)
  }
} // var


// Labor Income Shares
//------------------------------------------------------------------------------------------

noi disp as text "... labor income shares"

// loop over (sub)populations
foreach het in "" $hetgrouplabor_ineq {
	local suf=subinstr("`het'"," ","",.)

	// create fractiles
	cap drop ranklabor lblabor ublabor
	bysort year `het': create_pctclass labor $weights if CSsample == 1, ///
          generate(ranklabor) lbvar(lblabor) ubvar(ublabor) pctiles(1/90 90.5(0.5)99 99.1(0.1)99.9 99.95 99.99 99.995 99.999)

	// average income within fractiles
  bymymeans labor if CSsample == 1, by(year `het' ranklabor lblabor ublabor) ///
    saving("inequality/mean_labor_by_ranklabor_`suf'`yr'")
}

/*
//==================//
//--- Volatility ---//
//==================//


// Growth-Rate Percentiles
//------------------------------------------------------------------------------------------

if inlist(`yr', $d1yrlistcomma) {

  noi disp as text "... summary stats for growth rate (LX-sample)"

  // loop over (sub)populations
  foreach het in "" $hetgrouplabor_vola {
    local suf=subinstr("`het'"," ","",.)

    foreach f in `flist' {
      bymydist_detail researn`f'F, saving("volatility/sumstats_researn`f'F_`suf'`yr'") by(year `het') ///
        percentiles(0.001 0.01 0.05 0.1(0.1)0.9 1/99 99.1(0.1)99.9 99.95 99.99 99.999)
    } 

  } // f

} // 


// Growth-Rate Percentiles by Labor Income Bins
//------------------------------------------------------------------------------------------

noi disp as text "... summary stats for growth rate by labor income bins"

if inlist(`yr', $d1yrlistcomma) {

// loop over (sub)populations
foreach het in "" $hetgrouplabor_vola {
	local suf=subinstr("`het'"," ","",.)

	// Labor Income Bins
	if "`het'" == "male agegp" local lbs $lbfine5
	else local lbs $lbfine4
	cap drop bin lb ub
	create_bins labor, binvar(bin) lbvar(lb) ubvar(ub) lbs(`lbs')

	// Analysis
	foreach f in `flist' {
		// Conditional distribution of growth rate
		bymydist_detail researn`f'F if CSsample == 1, saving("volatility/sumstats_researn`f'F_by_labor_`suf'`yr'") by(year `het' bin lb ub) ///
				percentiles(0.01 0.1 0.5 1/10 15(5)90 91/99 99.5 99.9) meanvars(labor)   
		// Distribution of labor income in analysis sample
		bymydist_detail labor if CSsample == 1 & researn`f'F != ., saving("volatility/sumstats_`f'Fsample_labor_`suf'`yr'") by(year `het') percentiles(0.5(0.5)90 90.1(0.1)99.9 99.95 99.99 99.995 99.999)
	}

}

} // 



// Growth-Rate Percentiles by Permanent Income Rank
//------------------------------------------------------------------------------------------

if inlist(`yr', $d5yrlist) & inlist(`yrp', $perm3yrlist) {

noi disp as text "... summary stats for growth rate by permanent income rank (H-sample)"

cap drop Hsample
generate Hsample = 1 if researn1F != . & researn5F != .	& permearn != .

// loop over (sub)populations
foreach het in "" $hetgrouplabor_vola {
	local suf=subinstr("`het'"," ","",.)

	// Rankings of permanent income
	//-----------------------------------
	cap drop rank_permearn lb_permearn ub_permearn
	bysort year `het': create_pctclass permearn $weights if Hsample == 1,   ///
				generate(rank_permearn) lbvar(lb_permearn) ubvar(ub_permearn) pctiles(5(5)90 91/99 99.9) 

	// Summary statistics by het group and permanent income
	//------------------------------------------------------
	foreach f in `flist' {
		bymydist_detail researn`f'F if Hsample == 1, by(year `het' rank_permearn) ///
				saving("volatility/sumstats_researn`f'F_by_permearn_`suf'`yr'") ///
				percentiles($pctiles_selected) meanvars(permearn avg_pastearn lb_permearn ub_permearn)
	} // f

} // het

} // 


//================//
//--- Mobility ---//
//================//

/* 
  1. compute distribution of permearn by age which gives a mapping from permearn to age-specific TPP-ranks
  2. combine distribution of permearn from TPP and IAB which gives a mapping from permearn to true age-specific ranks
  3. construct bins of permearn
  4. in each bin and age, compute percentiles of permearn5F
  5. map these percentiles into age-specific ranks of permearn5F
  6. compute the average of these ranks

  Alternative: 
    - regress permearn and permearn5F on age dummies to take out age-specific averages
    - repeat the above steps using age groups instead of age
*/

// Future Permanent Income by Permanent Income
//------------------------------------------------------------------------------------------

noi disp as text "... summary stats for future permanent income by current permanent income"

if inlist(`yr', $d5yrlist) & inlist(`yr', $perm3yrlist) {

  // rank permanent earnings within gender and age
  foreach v in "" "5F" {
    bysort male age: create_pctclass permearnalt`v' $weights if !missing(permearnalt, permearnalt5F), generate(rankpermearnalt`v') ///
                          lbvar(lbpermearnalt`v') ubvar(ubpermearnalt`v') ///
                          pctiles(10(10)90 95 99)
  }
  
  // rank residualized permanent earnings by gender and age group
  foreach v in "" "5F" {
    bysort male agegp: create_pctclass respermearnalt`v' $weights if !missing(respermearnalt, respermearnalt5F), generate(rankrespermearnalt`v') ///
                          lbvar(lbrespermearnalt`v') ubvar(ubrespermearnalt`v') ///
                          pctiles(2.5(2.5)97.5 99 99.9)
  }
  
 
  // permearnalt and permearnalt5F by age
	bymydist_detail permearnalt if !missing(permearnalt, permearnalt5F), ///
      saving("mobility/sumstats_permearnalt_maleage`yr'") by(year male age) ///
			percentiles(1/99 99.9)
	bymydist_detail permearnalt5F if !missing(permearnalt, permearnalt5F), ///
      saving("mobility/sumstats_permearnalt5F_maleage`yr'") by(year male age) ///
			percentiles(1/99 99.9)
      
 	// permearnalt5F by rankpermearnalt and age
	bymydist_detail permearnalt5F if !missing(permearnalt, permearnalt5F), ///
      saving("mobility/sumstats_permearnalt5F_by_rankpermearnalt_maleage`yr'") ///
      by(year male age rankpermearnalt) ///
			percentiles(1/10 15(5)90 91/99) meanvars(permearnalt respermearnalt rankrespermearnalt rankpermearnalt5F rankrespermearnalt5F)
     
  // respermearnalt and respermearnalt5F by age group
	bymydist_detail respermearnalt if !missing(respermearnalt, respermearnalt5F), ///
      saving("mobility/sumstats_respermearnalt_maleagegp`yr'") by(year male agegp) ///
			percentiles(1/99 99.9)
	bymydist_detail respermearnalt5F if !missing(respermearnalt, respermearnalt5F), ///
      saving("mobility/sumstats_respermearnalt5F_maleagegp`yr'") by(year male agegp) ///
			percentiles(1/99 99.9)     
      
	// respermearnalt5F by rankrespermearnalt and age group
	bymydist_detail respermearnalt5F if !missing(respermearnalt, respermearnalt5F), ///
      saving("mobility/sumstats_respermearnalt5F_by_rankrespermearnalt_maleagegp`yr'") ///
      by(year male agegp rankrespermearnalt) ///
			percentiles(1/10 15(5)90 91/99) meanvars(respermearnalt permearnalt rankpermearnalt rankrespermearnalt5F rankpermearnalt5F)
      


} // inlist(`yr',${d5yrlist})
*/

} // Loop over years

//=================================//
//--- Collect data across years ---//
//=================================//

noi disp as text _n "Collecting data across years"


// Inequality Data
//====================================
noi disp as text "... inequality data"

noi disp as text "... inequality data"

// summary statistics
foreach var in labor logearn researn {
  foreach het in "" $hetgrouplabor_ineq {
    local suf=subinstr("`het'"," ","",.)
    append_and_export , filestub("inequality/sumstats_`var'_`suf'") years($yrfirst/$yrlast) var(`var')
  }
}

// mean income by income fractiles
foreach het in "" $hetgrouplabor_ineq {
	local suf=subinstr("`het'"," ","",.)
	append_and_export , filestub("inequality/mean_labor_by_ranklabor_`suf'") years($yrfirst/$yrlast) var(labor) dataprivacy(data_privacy_mean)
}

/*
// Volatility Data: 
//====================================
noi disp as text "... volatility data"

// Growth-Rate Percentiles
//-------------------------------------------------
foreach f in 1 5 {
  if `f' == 1 local years $d1yrlist 
  else local years = subinstr("$d5yrlist", ",", " ", .)
  foreach het in "" $hetgrouplabor_vola {
    local suf=subinstr("`het'"," ","",.)
    append_and_export , filestub("volatility/sumstats_researn`f'F_`suf'") years(`years') var(researn`f'F)
  }
}

// Growth-Rate Percentiles by Permanent Income Bins
//-------------------------------------------------
foreach f in 1 5 {
  foreach het in "" $hetgrouplabor_vola {
    local suf=subinstr("`het'"," ","",.)
    append_and_export , filestub("volatility/sumstats_researn`f'F_by_permearn_`suf'") years($years_Hsample) var(researn`f'F)
  }
}

// Growth-Rate Percentiles by Current Income Bins
//-------------------------------------------------
foreach f in 1 5 {
  if `f' == 1 local years $d1yrlist 
  else local years = subinstr("$d5yrlist", ",", " ", .)
  foreach het in "" $hetgrouplabor_vola {
    local suf=subinstr("`het'"," ","",.)
    append_and_export , filestub("volatility/sumstats_researn`f'F_by_labor_`suf'") years(`years') var(researn`f'F)
		append_and_export , filestub("volatility/sumstats_`f'Fsample_labor_`suf'") years(`years') var(labor)

  }
}


// Mobility Data
//==================================

noi disp as text "... mobility data"

foreach var in respermearnalt permearnalt {

  if "`var'" == "respermearnalt" local suf = "maleagegp"
  else local suf = "maleage"

	append_and_export , filestub("mobility/sumstats_`var'_`suf'") years(${years_Hsample}) var(`var')
	append_and_export , filestub("mobility/sumstats_`var'5F_`suf'") years(${years_Hsample}) var(`var'5F)
	append_and_export , filestub("mobility/sumstats_`var'5F_by_rank`var'_`suf'") years(${years_Hsample}) var(`var'5F)

} // var
*/  
  
} // quietly


//--- END ---//
//===========//

timer off 1
timer list 1


end //distributions_do	

/*---------------------------------------------*/
/* Distribution - volatility & mobility */
/*---------------------------------------------*/
cap program drop distributions_vol_mob
program define distributions_vol_mob

	* init
	do "${do}/201_Initialize"

	* Dist do file
	do "${do}/206_Distributions_iab_hw"


end // distributions_vol_mob

/*-------------------------------------------------------*/
/* Percentiles Earnings (table)  */
/*-------------------------------------------------------*/
cap program drop perc_earn_table
program perc_earn_table
	syntax using , years(numlist ascending) varstub(str)

	// Initiate tex-file
	file open table `using', write replace

	// Begin tabular
	file write table "\begin{tabular}{lrrrrrrrrrrr}" _n
	file write table "\toprule \toprule" _n
	
	local coltitles = "Year"
	foreach p in 1 5 10 25 50 75 90 95 99 99.9 99.99 {
		local coltitles = "`coltitles' & \multicolumn{1}{c}{P`p'}"
	}
	file write table "`coltitles' \\" _n
	file write table "\midrule" _n

	// Add content
	qui foreach year of numlist `years' {
			
		local thisline = "`year'"
		foreach var in p1`varstub' p5`varstub' p10`varstub' p25`varstub' p50`varstub' p75`varstub' p90`varstub' p95`varstub' p99`varstub' p99_9`varstub' p99_99`varstub' {
			sum `var' if year == `year', meanonly
			local p = strofreal(`r(mean)', "%9,0fc")
			if "`varstub'"=="_logmean" local p = strofreal(`r(mean)', "%9,2fc")
			if "`varstub'"=="_logmid" local p = strofreal(`r(mean)', "%9,2fc")
			local thisline = "`thisline' & `p'"
		}

		file write table "`thisline' \\" _n

	}

	// End tabular
	file write table "\bottomrule" _n
	file write table "\end{tabular}" _n

	// Close tex-file
	file close table
		
end // perc_earn_table

/*-------------------------------------------------------*/
/* Descriptive Table   */
/*-------------------------------------------------------*/
cap program drop desc_table
program desc_table
	syntax, vsample(str)
	
	file open table using "${folderfile}/descriptives_`vsample'.tex", write replace

	file write table "\begin{tabular}{ccccccccccc}" _n
	file write table "\toprule \toprule" _n
	file write table "Year & Obs & \multicolumn{2}{c}{Mean Earnings} & Women & \multicolumn{3}{c}{Age Group Shares} & \multicolumn{6}{c}{Education Shares} \\" _n
	file write table "\cmidrule(lr){3-4}\cmidrule(lr){6-8}\cmidrule(lr){9-14}" _n
	file write table " &  & Men & Women & Share (\%) & $[25,35]$ & $[36,45]$ & $[46,55]$ & 1 & 2 & 3 \\" _n
	file write table "\midrule" _n


	qui foreach year of numlist $yrfirst/$yrlast {

		// Headcount Data
		import delimited using "${out}/${date_analysis} Descriptive_Stat - `vsample'/cs_cross_tabulation", clear
		keep if year == `year'
		replace educ=7 if mi(educ)
		recode educ (1=1) (2 3 4 =2) (5 6 =3) (mis=7)
		
		qui sum numobs
		local ntotal = strofreal(`r(N)' * `r(mean)', "%9.0f")
		local ntotalnice = strofreal(`ntotal', "%9.0fc")
		disp `ntotalnice'
		
		qui sum numobs if male == 0
		local nfemale = strofreal(`r(N)' * `r(mean)', "%9.0f")
		local sharefemale = strofreal(`nfemale' / `ntotal' * 100, "%9.1f")
		disp "`sharefemale'"
			
		qui sum numobs if age >= 25 & age <= 35
		local n25to35 = strofreal(`r(N)' * `r(mean)', "%9.0f")
		local share25to35 = strofreal(`n25to35' / `ntotal' * 100, "%9.1f")
		disp "`share25to35'"
		
		qui sum numobs if age >= 36 & age <= 45
		local n36to45 = strofreal(`r(N)' * `r(mean)', "%9.0f")
		local share36to45 = strofreal(`n36to45' / `ntotal' * 100, "%9.1f")
		disp "`share36to45'"

		qui sum numobs if age >= 46 & age <= 55
		local n46to55 = strofreal(`r(N)' * `r(mean)', "%9.0f")
		local share46to55 = strofreal(`n46to55' / `ntotal' * 100, "%9.1f")
		disp "`share46to55'"	
		
		qui sum numobs if educ==1
		local neduc1 = strofreal(`r(N)' * `r(mean)', "%9.0f")
		local shareeduc1 = strofreal(`neduc1' / `ntotal' * 100, "%9.1f")
		disp "`shareeduc1'" 
		
		qui sum numobs if educ==2
		local neduc2 = strofreal(`r(N)' * `r(mean)', "%9.0f")
		local shareeduc2 = strofreal(`neduc2' / `ntotal' * 100, "%9.1f")
		disp "`shareeduc2'" 
		
		qui sum numobs if educ==3
		local neduc3 = strofreal(`r(N)' * `r(mean)', "%9.0f")
		local shareeduc3 = strofreal(`neduc3' / `ntotal' * 100, "%9.1f")
		disp "`shareeduc3'" 
		
		// Mean Earnings
		import delimited using "${out}/${date_analysis} Descriptive_Stat - `vsample'/l_cs_labor_yrgender_sum_stats", clear
		keep if year == `year'
		
		qui sum meanlabor if male == 0
		local meanfemale = strofreal(`r(mean)', "%9.0fc")
		disp "`meanfemale'"
		
		qui sum meanlabor if male == 1
		local meanmale = strofreal(`r(mean)', "%9.0fc")
		disp "`meanmale'"
		
		// Add line to table
		file write table "`year' & `ntotalnice' & `meanmale' & `meanfemale' & `sharefemale' & `share25to35' & `share36to45' & `share46to55' & `shareeduc1' & `shareeduc2' & `shareeduc3' \\" _n

	}

	file write table "\bottomrule" _n
	file write table "\end{tabular}" _n

	file close table

end // desc_table


	
/*---------------------------------------------------------*/
/* Run Main Program */
/*---------------------------------------------------------*/
main // run main program

cap log close _all
set trace off
clear
exit

/*========================================= END ==============================================*/
