//==========================================================================================
// This code prepares the raw TPP data
// Moritz Drechsel-Grau & Kai Schmid (based on code by Halvorsen, Ozkan, Salgado)
//==========================================================================================


/*==========================================================================================

	Variablen
		- hrf (Hochrechnungsfaktor)
		- ef8 (Geschlecht)
		- ef11, ef12 (Soziale Gliederung)
		- ef18 (Veranlagung)
		- ef19 (Splitting/Grund-Tabelle)
		- ef43, ef44 (Geburtsjahr)
		- c65163, c65164 (Bruttoarbeitslohn)
		- c65121, c65122 (Einkünfte aus Gewerbebetrieb)
		- c65141, c65142 (Einkünfte aus selbständiger Arbeit)
		- c65161, c65162 (Einkünfte aus nichtselbständiger Arbeit)
		- c65221, c65222 (Einkünfte aus Kapital und Vermögen)
		- c65241, c65242 (Einkünfte aus Vermietung und Verpachtung)
		- c65101, c65102 (Einkünfte aus Land und Forstwirtschaft)
		- c65261, c65262 (Sonstige Einkünfte)
		-	c47116 c48116	 (Arbeitslohn als Grenzgänger (in Euro oder Landeswährung) ab 2005)
		-	c47016 c48016  (Arbeitslohn als Grenzpendler (in Landeswährung) bis 2004)
		
		
	/* NOTES Geschäftsstatistik vs Bundesstatistik
		- ef1 = 18 --> Lohnsteuerfälle (mit Wohnitz in D)
		- ef1 = 10 --> Lohnsteuerfälle (mit Wohnitz im Ausland)
		- ef18 (Split) is missing for ef1 == 18
	*/
	
	Output: Personen-Jahr Panel
	--> BASEDATA.dta

==========================================================================================*/

clear all
set more off

// Create log-file
local timestamp: disp %tcCCYY-NN-DD,_HH-MM tc(`c(current_date)' `c(current_time)') // example (2021-02-14, 14:54)
capture log close
capture noisily log using "${log_folder}/$date 101_DataPrep (`timestamp').log", replace

timer clear 1
timer on 1


// Initialize Settings
qui do "${do_folder}/001_Initialize.do"


// 	Variablenauswahl
//==========================================================================================

	* time invariant variables
	global constants hrf

	* identifiers
	global ids 		ida idb

	* demographic variables
	global demo 	ef1 ef8 ef11 ef12 ef18 ef19 ef43 ef44

	* income variables
	global inc 		c65163 c65164 /// Bruttoarbeitslohn
								c65121 c65122 /// Einkünfte aus Gewerbebetrieb
								c65141 c65142 /// Einkünfte aus selbständiger Arbeit
								c65221 c65222 /// Einkünfte aus Kapital und Vermögen
								c65241 c65242 /// Einkünfte aus Vermietung und Verpachtung
								c65101 c65102 /// Einkünfte aus Land und Forstwirtschaft
								c65261 c65262 /// Sonstige Einkünfte
								c47116 c48116	/// Arbeitslohn als Grenzgänger (in Euro oder Landeswährung) ab 2005
								c47016 c48016 //  Arbeitslohn als Grenzpendler (in Landeswährung) bis 2004


	* variable list for wide data
	global vars_wide = ""
	foreach var in $constants $ids $demo $inc {
		global vars_wide = "$vars_wide *`var'"
	}

	* stub list for reshape
	global vars_stubs = ""
	foreach var in $ids $demo $inc {
		global vars_stubs = "$vars_stubs @`var'"
	}	

// 	Load data
//==========================================================================================

	if $vollmaterial == 1 & "$user" == "kdfv" {
		use "${datafile_vollmaterial}", clear
	}
	else {
		use $vars_wide using "${datafile}", clear
	}

	describe
	count

	if $vollmaterial == 1 & $vollmaterial_subsample < 1 {
		generate random = runiform()
		keep if random <= $vollmaterial_subsample
		note: This is a $vollmaterial_subsample subsample of the population data.
		drop random
	}

	count
	
// 	Taxunit-Year-Panel
//==========================================================================================

	// add weights to full population data
	if $vollmaterial == 1 {
		cap generate hrf = 1
	}

	// create tax unit ID
	gen unitid = _n
	order unitid, before(hrf)
	label variable unitid "ID of tax unit"

	// reshape
	greshape long $vars_stubs , i(unitid) j(help_year) string
	generate year = substr(help_year, 4, 1) if substr(help_year, 3, 1) == "0"
	replace year = substr(help_year, 3, 2) if substr(help_year, 3, 1) == "1"
	destring year, replace
	replace year = 2000 + year
	drop help_year
	label variable year "Year"

	// create panel
	xtset unitid year
	order unitid year id* ef* c*

	// numeric vars
	destring $demo, replace

	// drop missing years
	egen n_nonmissing = rownonmiss($demo)
	drop if n_nonmissing == 0
	drop n_nonmissing

	describe
	tab year


// Data Cleaning
//==========================================================================================


	// Move info for (single) women in column B (as is standard from 2012 on)
	replace ef12 		= ef11 		if ef8 == 1 & year <= 2011
	replace ef11		= .				if ef8 == 1 & year <= 2011
	replace ef44 		= ef43 		if ef8 == 1 & year <= 2011
	replace ef43		= .				if ef8 == 1 & year <= 2011
	replace c65164 	= c65163 	if ef8 == 1 & year <= 2011
	replace c65163	= .				if ef8 == 1 & year <= 2011
	replace c65122 	= c65121 	if ef8 == 1 & year <= 2011
	replace c65121	= .				if ef8 == 1 & year <= 2011
	replace c65142 	= c65141 	if ef8 == 1 & year <= 2011
	replace c65141	= .				if ef8 == 1 & year <= 2011
	replace c65262 	= c65261 	if ef8 == 1 & year <= 2011
	replace c65261	= .				if ef8 == 1 & year <= 2011
	replace c48016 	= c47016 	if ef8 == 1 & year <= 2011
	replace c47016	= .				if ef8 == 1 & year <= 2011
	replace c48116 	= c47116 	if ef8 == 1 & year <= 2011
	replace c47116	= .				if ef8 == 1 & year <= 2011
	replace idb 		= ida			if ef8 == 1 & year <= 2011
	replace ida			= ""			if ef8 == 1 & year <= 2011

	// Birthyear
	replace ef43 = . if ef43 < 1800
	replace ef44 = . if ef44 < 1800

	// Soziale Gliederung (kein Arbeitslohn --> missing)
	replace ef11 = . if ef11 == 0
	replace ef11 = . if c65163 == 0
	replace ef12 = . if ef12 == 0
	replace ef12 = . if c65164 == 0

	// Negative "Bruttoarbeitslohn"
	replace c65163 = 0 if c65163 < 0
	replace c65164 = 0 if c65164 < 0



// 	Individual-Year-Panel
//==========================================================================================

// Split tax units
//´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´


	/* split units if either of the following:
			- both birthyears available
			- ef18 == 2
	
	// number of available birth years per row
	egen n_birthyears = rownonmiss(ef43 ef44)

	tab ef18 ef19, missing
	tab ef18 n_birthyears, missing
	tab ef19 n_birthyears, missing
	tab ef8 if ef18 != 2 & n_birthyears == 2 // only men
	*/
	
	// split tax units
	generate _split = (ef18 == 2)
	label variable _split "splitting counterpart exists"

	expand 2 if _split == 1, generate(_expanded)
	label variable _expanded "expanded (unit splitting)"

	order unitid year _split _expanded
	sort unitid year _expanded

	tab year


// Individual-level characteristics
//´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´´

	// Gender (male)
	generate male = .
	label variable male "Male Dummy"
	replace male = 1 if ef8 == 0 & _split == 0					/* allein veranlagter Mann */
	replace male = 0 if ef8 == 1 & _split == 0					/* allein veranlagte Frau */
	replace male = 1 if _split == 1 & _expanded == 0		/* gemeinsam veranlagt, Person A -> husband */
	replace male = 0 if _split == 1 & _expanded == 1		/* gemeinsam veranlagt, Person B -> wife */
	tab year male [aw=hrf]

	// Year of birth (yob)
	generate yob = .
	label variable yob "Year of Birth"
	replace yob = ef43 if male == 1
	replace yob = ef44 if male == 0


	// Labor income / Earnings (labor)
	generate labinc = .
	label variable labinc "Annual labor income (earnings)"
	replace labinc = c65163 if male == 1
	replace labinc = c65164 if male == 0
	replace labinc = 0 if missing(labinc)

	// Foreign Labor income / Earnings
	generate foreignlabinc = .
	label variable foreignlabinc "Foregin annual labor income (currency unclear --> see c47117,c48117)"
	replace foreignlabinc = c47016 if male == 1 & year < 2005
	replace foreignlabinc = c48016 if male == 0 & year < 2005
	replace foreignlabinc = c47116 if male == 1 & year >= 2005
	replace foreignlabinc = c48116 if male == 0 & year >= 2005
	replace foreignlabinc = 0 if missing(foreignlabinc)	

	// Selbsständige Arbeit
	generate inc_selfempl = .
	label variable inc_selfempl "Einkünfte aus selbständiger Arbeit"
	replace inc_selfempl = c65141 if male == 1
	replace inc_selfempl = c65142 if male == 0
	replace inc_selfempl = 0 if missing(inc_selfempl)

	// Gewerbebetrieb
	gen inc_gb = .
	label variable inc_gb "Einkünfte aus Gewerbebetrieb"
	replace inc_gb = c65121 if male == 1
	replace inc_gb = c65122 if male == 0
	replace inc_gb = 0 if missing(inc_gb)

	// Einkünfte aus Kapitalvermögen
	generate inc_kv = .
	label variable inc_kv "Einkünfte aus Kapitalvermögen"
	replace inc_kv = c65221 if male == 1
	replace inc_kv = c65222 if male == 0
	replace inc_kv = 0 if missing(inc_kv)

	// TODO? rescale between 2002 and 2008 (Halbeinkünfteverfahren)

	//Einkünfte aus Vermietung und Verpachtung
	generate inc_vv = .
	label variable inc_vv "Einkünfte aus Vermietung und Verpachtung"
	replace inc_vv = c65241 if male == 1
	replace inc_vv = c65242 if male == 0
	replace inc_vv = 0 if missing(inc_vv)

	// Einkünfte aus Land und Forstwirtschaft
	generate inc_lf = .
	label variable inc_lf "Einkünfte aus Land und Forstwirtschaft"
	replace inc_lf = c65101 if male == 1
	replace inc_lf = c65102 if male == 0
	replace inc_lf = 0 if missing(inc_lf)

	// Sonstige Einkünfte
	generate inc_sonst = .
	label variable inc_sonst "Sonstige Einkünfte"
	replace inc_sonst = c65261 if male == 1
	replace inc_sonst = c65262 if male == 0
	replace inc_sonst = 0 if missing(inc_sonst)

	// Einkunfts-Gruppen
	generate businc = inc_gb + inc_lf + inc_selfempl
	label variable businc "Business income (GB + LF + SE)"

	generate businc_noSE = inc_gb + inc_lf
	label variable businc_noSE "Business income without self-employment income (GB + LF)"
	
	generate capinc = inc_vv + inc_sonst + inc_kv /*[break due to Abgeltungssteuer in 2009]*/
	label variable capinc "Capital income (KV + VV + Sonstige)"
	
	generate capinc_noKV = inc_vv + inc_sonst 
	label variable capinc_noKV "Capital income without KV (VV + Sonstige)"
	
	generate totinc = labinc + businc + capinc
	label variable totinc "Total income (using Bruttoarbeitslohn)"

	generate totinc_noKV = labinc + businc + capinc_noKV
	label variable totinc_noKV "Total income without KV (using Bruttoarbeitslohn)"
	
	generate nonlabinc = businc + capinc
	label variable nonlabinc "Non-labor income (businc + capinc)"

	generate nonlabinc_noKV = businc + capinc_noKV
	label variable nonlabinc_noKV "Non-labor income (businc + capinc without KV)"

	// Main income source (labor, nonlabor)
	generate mainsrc2 = .
	replace mainsrc2 = 1 if abs(labinc) > 0 & abs(abs(labinc) - max(abs(labinc), abs(nonlabinc_noKV))) < 1e-8
	replace mainsrc2 = 2 if abs(nonlabinc_noKV) > 0 & abs(abs(nonlabinc_noKV) - max(abs(labinc), abs(nonlabinc_noKV))) < 1e-8
	label variable mainsrc2 "Main income source (1=labinc, 2=nonlabinc_noKV)"
	tab mainsrc2
	
	// Main income source (labor, business, capital)
	generate mainsrc3 = .
	replace mainsrc3 = 1 if abs(labinc) > 0 & abs(abs(labinc) - max(abs(labinc), abs(businc), abs(capinc_noKV))) < 1e-8
	replace mainsrc3 = 2 if abs(businc) > 0 & abs(abs(businc) - max(abs(labinc), abs(businc), abs(capinc_noKV))) < 1e-8
	replace mainsrc3 = 3 if abs(capinc_noKV) > 0 & abs(abs(capinc_noKV) - max(abs(labinc), abs(businc), abs(capinc_noKV))) < 1e-8
	label variable mainsrc3 "Main income source (1=labinc, 2=businc, 3=capinc_noKV)"
	tab mainsrc3
	
	// Fine main source (labor, self-empl, business w/o selfempl, capital)
	generate mainsrc4 = .
	replace mainsrc4 = 1 if abs(labinc) > 0 & abs(abs(labinc) - max(abs(labinc), abs(inc_selfempl), abs(businc_noSE), abs(capinc_noKV))) < 1e-8
	replace mainsrc4 = 2 if abs(inc_selfempl) > 0 & abs(abs(inc_selfempl) - max(abs(labinc), abs(inc_selfempl), abs(businc_noSE), abs(capinc_noKV))) < 1e-8
	replace mainsrc4 = 3 if abs(businc_noSE) > 0 & abs(abs(businc_noSE) - max(abs(labinc), abs(inc_selfempl), abs(businc_noSE), abs(capinc_noKV))) < 1e-8
	replace mainsrc4 = 4 if abs(capinc_noKV) > 0 & abs(abs(capinc_noKV) - max(abs(labinc), abs(inc_selfempl), abs(businc_noSE), abs(capinc_noKV))) < 1e-8
	label variable mainsrc4 "Fine Main income source (1=labinc, 2=selfempl, 3=businc_noSE, 3=capinc_noKV)"
	tab mainsrc4	

	// Soziale Gliederung
	gen sozglied = .
	label variable sozglied "soziale Gliederung"
	replace sozglied = ef11 if male == 1 									/* allein veranlagte Männer */
	replace sozglied = ef12 if male == 0 									/* allein veranlagte Männer */
	tab year sozglied

	/* Beschreibung von ef11 und ef12

		bis 2004:		0 = kein Bruttolohn vorhanden
								1 = nicht rentenversicherungspflichtige AN (gekürzte VSP)
								2 = rentenversicherungspflichtige AN (ungekürzte VSP)
								3 = ungekürzte VSP (no observations)

		2004-09:		0 = kein Bruttolohn vorhanden
								1 = nicht rentenversicherungspflichtige AN (gekürzte VSP)
								2 = rentenversicherungspflichtige AN (ungekürzte VSP)
								3 = gekürzte VSP (almost no observations ~ 0.1%)

		ab 2010:		0 = kein Bruttolohn vorhanden
								1 = Beamte
								2 = sonsitge nicht rentenversicherungspflichtige AN
								3 = rentenversicherungspflichtige AN

		Achtung: 	von 2009 auf 2010 springt die Anzahl rentenversicherungspflichtiger AN von 57%
							auf 64% (Umkodierung).

							Erklärung der Fachabteilung am FDZ (Mail von Maurice Brandt, 25.02.2021):
							"Ich habe nochmals etwas recherchiert und bin zu der Erkenntnis gekommen, dass die Veränderungen von 2009 zu 2010 bei dem EF11 (EF12) darin begründet sind, dass durch das Gesetz zur verbesserten steuerlichen Berücksichtigung von Vorsorgeaufwendungen (Bürgerentlastungsgesetz Krankenversicherung) vom 16. Juli 2009 der Abzug von Vorsorgeaufwendungen ab 1. Januar 2010 in wesentlichen Bereichen geändert wurde.
							Dies betrifft neben dem Abzug sonstiger Vorsorgeaufwendungen als Sonderausgaben im Rahmen der Veranlagung zur Einkommensteuer auch die Berücksichtigung von Vorsorgeaufwendungen im Lohnsteuerabzugsverfahren über die Vorsorgepauschale.
							Der pauschale Ansatz von Vorsorgeaufwendungen im Veranlagungsverfahren mittels Vorsorgepauschale (§ 10c Absatz 2 bis 5) wurde mit Wirkung ab 2010 abgeschafft. Eine Vorsorgepauschale wird ab 2010 ausschließlich im Lohnsteuerabzugsverfahren berücksichtigt (§ 39b Absatz 2 Satz 5 Nummer 3 und Absatz 4 EStG).
							Über die Vorsorgepauschale hinaus werden im Lohnsteuerabzugsverfahren keine weiteren Vorsorgeaufwendungen berücksichtigt; die Eintragung eines Freibetrags auf der Lohnsteuerkarte (§ 39a EStG) ist wie bisher nicht möglich.
							Durch diese gesetzlichen Änderungen wurden die EF11/EF12 ab 2010 neu signiert und sind meiner Meinung nach mit der Signierung von 2009 nicht mehr vergleichbar."
		Anmerkung
	*/

	// Rentenversicherungspflichtig (Beamte haben gekürzte VSP)
	generate 	rentenvers = .
	label variable rentenvers "RV-pflichtige Arbeitnehmer"

		// all years
		replace 	rentenvers = . if sozglied == .		// kein Bruttolohn vorhanden oder missing

		// until 2004
		replace 	rentenvers = 0 if sozglied == 1 & year <= 2004		// nicht rentenversicherungspflichtigen AN
		replace 	rentenvers = 1 if sozglied == 2 & year <= 2004		// rentenversicherungspflichtigen AN

		// 2005 - 2009
		replace 	rentenvers = 0 if sozglied == 1 & year >= 2005 & year <= 2009		// nicht rentenversicherungspflichtigen AN
		replace 	rentenvers = 0 if sozglied == 3 & year >= 2005 & year <= 2009		// rentenversicherungspflichtigen AN
		replace 	rentenvers = 1 if sozglied == 2 & year >= 2005 & year <= 2009		// rentenversicherungspflichtigen AN

		// from 2010
		replace 	rentenvers = 0 if sozglied == 1 & year >= 2010		// Beamte (nicht rentenversicherungspflichtig)
		replace 	rentenvers = 0 if sozglied == 2 & year >= 2010		// sonstige nicht rentenversicherungspflichtigen AN
		replace 	rentenvers = 1 if sozglied == 3 & year >= 2010		// rentenversicherungspflichtigen AN

	// Drop if positive labor income and rentenvers missing
	drop if missing(rentenvers) & labinc > 0 & !missing(labinc)

	tab year rentenvers


	// Veranlagungsart
	//-------------------------------------------
	generate veranlagungsart = ef1
	label variable veranlagungsart "Veranlagungsart"
	tab year ef1
	
	/* Antragsveranlagung für
		- Ehepaare mit nur einer erwerbstätigen Person (auch Steuerklasse 3) --> don't use split info for mustfiler
		- Ehepaare mit zwei erwerbstätigen Personen in Steuerklassen 4/4
		- Singles mit nur einem Arbeitsverhältnis und Steuerklasse 1
		- keine Nebeneinkünfte
	*/

  // Zwangsveranlagung
  generate mustfiler = ef1 == 2 | ef1 == 5 | (nonlabinc_noKV > 500 & !missing(nonlabinc_noKV))
  label variable mustfiler "Zwangsveranlagung (ef1 = 2,5) or non-labor income > 500"
	tab year mustfiler
	
	tab ef1 mustfiler
	generate nebeineinkunfte = (nonlabinc_noKV > 500 & !missing(nonlabinc_noKV))
	tab ef1 nebeineinkunfte

	// Lohnsteuerfälle
	generate nonfiler = ef1 == 18 | ef1 == 10
	label variable nonfiler "Non-filing tax payer (Lohnsteuerfall, ef1=10,18)"
	tab year nonfiler
	
	tab ef1 nonfiler
	tab nonfiler nebeineinkunfte

	drop nebeineinkunfte


	// Person ID 
	//-------------------------------------------

	// ID (not available before 2010)
	generate id = ""
	replace id = ida if male == 1
	replace id = idb if male == 0

	// New Person-ID (personid)
	bysort unitid (year _expanded): generate d = 2 if _n == 1
	generate unitid2 = sum(d)
	generate personid = unitid2
	replace personid = personid + 1 if male == 0
	bysort personid (year): generate byperson = _n == 1
	drop unitid2 d
	label variable personid "ID of individual"
	
	
	drop if foreignlabinc > 0 & !missing(foreignlabinc)
	drop foreignlabinc

	tab year


//==========================================================================================
// CHECKS 
//==========================================================================================

// Delete if multiple id per personid and vice versa
//==========================================================================================

	// how many different id per personid
	destring id, generate(numid)
	bysort personid: egen maxid = max(numid) if !missing(numid)
	bysort personid: egen minid = min(numid) if !missing(numid)
	generate multiple_id = maxid != minid & !missing(maxid, minid)
	drop minid maxid numid

	// drop entire unit if one personid with multiple id
	bysort unitid: egen aux = max(multiple_id)
	drop if aux >= 1 & !missing(aux)
	drop aux multiple_id

	tab year

	// how many different personid per id ?
	bysort id personid: generate newid = _n == 1 & !missing(id)
	bysort id: egen n_personid_per_id = total(newid)
	drop newid

	// drop entire unit if it shares an id with another unit
	bysort unitid: egen aux = max(n_personid_per_id)
	drop if aux >= 2 & !missing(aux)
	drop aux n_personid_per_id

	tab year


// Delete if multiple birth years
//==========================================================================================

	bysort personid: egen sdyob = sd(yob)
	count if sdyob > 0 & byperson == 1

	// drop entire unit
	bysort unitid: egen aux = max(sdyob)
	drop if aux > 0 & aux != .
	drop aux sdyob

	tab year
	
	// drop if birthyear missing
	drop if yob == .

	tab year


// Rescale weights (sum to N in each year)
//==========================================================================================

	bysort year: egen SUMhrf_t = total(hrf)
	bysort year: egen N_t = count(hrf)
	replace hrf = hrf / SUMhrf_t * N_t
	drop SUMhrf_t N_t
	label variable hrf "Sample weight (sums to N in each year)"


// CS Criteria
//==========================================================================================

/* 	CS equals 0 if either of the following is true
		(a) all income sources are zero: labinc == 0 & capinc == 0 & businc == 0
		(b) low labor income and no other income: labinc < mininc & capinc == 0 & businc == 0
		(c) average total income in absolute value is below the average mininc & not (b)

		CSc always equals 1
*/

	// nominal minimum income threshold
	generate threshold = .
	foreach y of numlist $yrfirst/$yrlast {
		local iy = `y' - $yrfirst + 1
		replace threshold = nmininc[`iy',1] if year == `y'
	}

	// b) labor income above threshold
	generate CS_labinc = 1
	replace CS_labinc = 0 if labinc < threshold
	label variable CS_labinc "labor income above threshold"

	tab CS_labinc
	tab male _split if CS_labinc == 0, cell

	// bb) labor income above 1/3 of threshold
	generate CSc_labinc = 1
	replace CSc_labinc = 0 if labinc < (1/3) * threshold
	label variable CSc_labinc "labor income above 1/3 of threshold"

	tab CSc_labinc
	tab male _split if CSc_labinc == 0, cell

foreach suf in "" "_noKV" {
	
	// a) NOT all income sources are zero: labinc == 0 & capinc == 0 & businc == 0
	generate CS_anyinc = 1
	replace CS_anyinc = 0 if labinc == 0 & capinc`suf' == 0 & businc == 0
	label variable CS_anyinc "some income source has non-zero entry"

	tab CS_anyinc [aw=hrf]
	tab male _split if CS_anyinc == 0 [aw=hrf], cell

	// c) avg totinc above avg threshold in absolute values (by CS_labinc)
	generate CS_avgtotinc = 1
	label variable CS_avgtotinc "avg totinc above avg threshold in absolute values (by CS_labinc)"

	bysort personid CS_labinc (year): egen mean_totinc = mean(abs(totinc`suf'))
	bysort personid CS_labinc (year): egen mean_threshold = mean(threshold)

	replace CS_avgtotinc = 0 if mean_totinc < mean_threshold

	tab CS_labinc CS_avgtotinc [aw=hrf]
	tab male _split if CS_avgtotinc == 0 [aw=hrf], cell


	// cc) avg totinc above avg threshold in absolute values (by CS_labinc)
	generate CSc_avgtotinc = 1
	label variable CSc_avgtotinc "avg totinc above 1/3 of avg threshold in absolute values (by CS_labinc)"

	replace CSc_avgtotinc = 0 if mean_totinc < (1/3) * mean_threshold

	tab CSc_labinc CSc_avgtotinc [aw=hrf], cell
	tab male _split if CSc_avgtotinc == 0 [aw=hrf], cell


	// COMBINE: CS selection variables
	generate CS`suf' = 1
	label variable CS`suf' "include in CS`suf' sample"
	replace CS`suf' = 0 if CS_anyinc == 0
	tab CS`suf' [aw=hrf]
	replace CS`suf' = 0 if CS_labinc == 0 & nonlabinc == 0
	tab CS`suf' [aw=hrf]
	replace CS`suf' = 0 if CS_labinc == 0 & CS_avgtotinc == 0
	tab CS`suf' [aw=hrf]

	generate CSc`suf' = 1
	label variable CSc`suf' "include in CSc`suf' sample"
	replace CSc`suf' = 0 if CSc_labinc == 0 & nonlabinc == 0
	tab CSc`suf' [aw=hrf]
	replace CSc`suf' = 0 if CSc_labinc == 0 & CSc_avgtotinc == 0
	tab CSc`suf' [aw=hrf]

	

	drop mean_totinc mean_threshold CS*anyinc CS*avgtotinc

}

drop threshold

// 	Save
//==========================================================================================

	sort unitid personid year
	drop $demo $inc $ids id
	drop byperson
	drop sozglied 
	*drop nyears_unit yrfirst_unit yrlast_unit byunitmarker n_birthyears

	xtset personid year

	order personid year unitid mustfiler nonfiler hrf male yob rentenvers ///
				totinc totinc_noKV labinc businc businc_noSE capinc capinc_noKV nonlabinc nonlabinc_noKV ///
				mainsrc2 mainsrc3 mainsrc4 inc_* CS*

	compress
	save "${dta_folder}/${BASEDATA}", replace


// END
//==========================================================================================

timer off 1
timer list 1

cap log close
