/*==============================================================================

  This do-file imputes the variable rentenvers, which is mis-coded in the years
  before 2010.

==============================================================================*/

do "${do_folder}/001_Initialize.do"
do "${do_folder}/myprogs.do"

// Output folder
global outfolder="$date 102_ImputeCivilServants"
capture mkdir "${out_folder}/$outfolder"

// Log file
local timestamp: disp %tcCCYY-NN-DD,_HH-MM tc(`c(current_date)' `c(current_time)') // example (2021-02-14, 14:54)
capture log close
capture noisily log using "${log_folder}/$outfolder (`timestamp').log", replace

timer clear 1
timer on 1


//------------------------------------------------------------
// Load data
//------------------------------------------------------------

local usevars personid year yob male hrf rentenvers labinc nonfiler mustfiler

use `usevars' using "$dta_folder/${BASEDATA}.dta", clear

cap drop rentenvers_imp

// civil servant indicator
generate C = rentenvers == 0 if !missing(rentenvers)

// save original C
generate Cold = C

// Normalize earnings with CPI
generate rlabinc = .
local k = 1
foreach y of numlist $yrfirst/$yrlast {
	replace rlabinc = labinc / cpimat[`k',1] if year == `y'
	local ++k
}


//------------------------------------------------------------
// Group variable
//------------------------------------------------------------

// age groups
generate age = year - yob + 1
create_pctclass age if !missing(age) [aw=hrf], generate(bin_age) pctiles(20(20)80) lbvar(lb_age) ubvar(ub_age)
tab bin_age [aw=hrf]
drop age

// earnings groups by male and age group
bysort male bin_age: create_pctclass rlabinc if rlabinc > 0 [aw=hrf], generate(bin_rlabinc) pctiles(2 5(5)95) lbvar(lb_rlabinc) ubvar(ub_rlabinc)
tab bin_rlabinc [aw=hrf]

// combined group
global Gvars male bin_age lb_age ub_age bin_rlabinc lb_rlabinc ub_rlabinc
gegen G = group(${Gvars}) if !missing(male, bin_age, bin_rlabinc), missing
tab G, missing

sort personid year

//------------------------------------------------------------
// Predict civil servant shares and transitions by G 
//-------------------------------------------------------------


// estimate (true) share of civil servants by group and year
//-------------------------------------------------------------
preserve
	// use consistent sample: exclude non-filers who are added in 2012
	keep if nonfiler == 0

  tempfile cshares
  gcollapse (count) N=C (rawsum) Nw=hrf (mean) c=C [aw=hrf] if !missing(C) & !missing(G), by(G ${Gvars} year)
  save "`cshares'"

  // mean and standard dev. from 2010 onward
  gcollapse (mean) mu_c=c (sd) sigma_c=c (min) min_c=c (max) max_c=c if year >= 2010, by(G) 
  
  merge 1:m G using "`cshares'", keep(match using) nogenerate
  order G year c mu_c sigma_c
  
  // impute true shares for t < 2010
  set seed 9342984
  generate c_star = min(max(min_c, rnormal(mu_c, sigma_c)), max_c) if year < 2010 
	generate c_imp = .
  keep G ${Gvars} year c c_star c_imp mu_c sigma_c min_c max_c N Nw
  
  save "${out_folder}/$outfolder/civilservant_shares_transitions_predicted", replace
  
restore


// Estimate backward-transition probabilities by G
//-------------------------------------------------
preserve
	// use consistent sample: exclude non-filers who are added in 2012
	keep if nonfiler == 0

  // keep only years from 2010 onward
  keep if year >= 2010
  keep if !missing(C, F.C)
  
  // transition indicators (from t+1 to t)
  generate pi_0to1 = F.C == 0 & C == 1 if F.C == 0
  generate pi_1to0 = F.C == 1 & C == 0 if F.C == 1
	sum pi* [aw=hrf]
  
  gcollapse pi_0to1 pi_1to0 [aw=hrf] if !missing(G) , by(G)
  
  // merge to shares
  merge 1:m G using "${out_folder}/$outfolder/civilservant_shares_transitions_predicted", nogenerate
  sort G year
  order G ${Gvars} year c c_star mu_c sigma_c pi*
  foreach var of varlist * {
    label variable `var' "`var'"
  }
  save "${out_folder}/$outfolder/civilservant_shares_transitions_predicted", replace
  
restore


//------------------------------------------------------------------------------
// Backward-Impute civil servant indicator
//------------------------------------------------------------------------------


// positive labor income --> can be imputed
generate positive = rlabinc > 0


/*  Assumption:
    If a person (or her civil servant status) is missing in some years and observed before and after, 
    we simply skip these years. 
    Assuming that no change happens during a year when no tax return was filed makes sense.
*/

// Create counter within personid
cap drop counter
bysort personid (year): generate counter = _n
xtset personid counter
sort personid counter


quietly foreach tau of numlist 2009(-1)$yrfirst {
global tau = `tau'

  noi disp as text _n "Imputation with tau = $tau"

  // Backward-impute C for workers with a non-missing value of C in t > tau
  //----------------------------------------------------------------------------
  noi disp as text _skip(4) "backward-impute C_t for (t = $tau to $yrfirst) if a non-missing value of C in t > $tau exists"  

  // starting value: find first non-missing C after tau
  generate C_nonmissing = !missing(C)
  bysort personid C_nonmissing (counter): gegen temp = first(C) if year > $tau & C_nonmissing == 1
  bysort personid (counter): gegen C_start = min(temp) if year > $tau
  drop temp C_nonmissing
  
  // reset Cnew for backward-imputation and set starting value
  generate Cnew = C_start if year > $tau
  replace Cnew = . if year <= $tau
  drop C_start

  // mark people who are in the sample in tau+1
  generate _imputable = !missing(F.Cnew) if year == $tau
  bysort personid (counter): gegen imputable = max(_imputable)
  drop _imputable 

  // merge aggregate data
  merge m:1 year G using "${out_folder}/$outfolder/civilservant_shares_transitions_predicted", ///
        nogenerate keepusing(pi_0to1 pi_1to0)
  sort personid counter
  
  // simulate according to transitions even when labor income is not positive (in which case C should be missing)
  generate rswitch = runiform()
  foreach yr of numlist $tau(-1)$yrfirst {
		noi disp as text _skip(8) "... `yr'"  
    replace Cnew = cond(rswitch < pi_1to0 & !missing(pi_1to0), 0, 1) if imputable == 1 & year == `yr' & F.Cnew == 1
    replace Cnew = cond(rswitch < pi_0to1 & !missing(pi_0to1), 1, 0) if imputable == 1 & year == `yr' & F.Cnew == 0
  }
  drop rswitch pi_1to0 pi_0to1
  
  // update C using Cnew only whenever labor income is positive
  replace C = Cnew if year <= $tau & imputable == 1 & positive == 1 & !missing(G)
  drop Cnew imputable

  // Update c and recode C in year tau to match the target
  //----------------------------------------------------------------------------
  noi disp as text _skip(4) "update the share of mis-coded obs. and recode randomly selected workers to match the target in $tau"  
  
  // update share c in year tau and recompute the share of mis-coded observations
  preserve 
    tempfile cshares
    gcollapse c${tau}_A = C [aw=hrf] if !missing(C) & year < 2010 & !missing(G), by(G year)
    save "`cshares'", replace
    merge 1:1 G year using "${out_folder}/$outfolder/civilservant_shares_transitions_predicted", nogenerate
    generate mis_c${tau}_A = c${tau}_A - c_star
		noi sum mis_c${tau}_A if year == ${tau}
    save "${out_folder}/$outfolder/civilservant_shares_transitions_predicted", replace
  restore
  merge m:1 G year using "${out_folder}/$outfolder/civilservant_shares_transitions_predicted", ///
				nogenerate keepusing(G year N Nw c_star c${tau}_A)
  sort personid year

  // compute weighted number of workers to be recoded and impute to match c_star in year tau
  generate mass_recode = abs(c${tau}_A - c_star) * Nw if year == $tau
  generate correct = C == cond(c${tau}_A > c_star, 0, 1) if year == $tau
  generate rfill = runiform() if year == $tau & positive == 1 & correct == 0
	
	// cumulative weight
	bysort year G (rfill): generate mass = sum(hrf)
	
  bysort year G (rfill): replace C = cond(c${tau}_A > c_star, 0, 1) if mass <= mass_recode & year == $tau & rfill < . 
  drop mass_recode mass correct rfill c${tau}_A c_star N Nw
  
  // update c in year $tau and save
  preserve 
    tempfile cshares
    gcollapse c${tau}_B = C [aw=hrf] if !missing(C) & year < 2010 & !missing(G), by(G year)
    save "`cshares'", replace
    merge 1:1 G year using "${out_folder}/$outfolder/civilservant_shares_transitions_predicted", nogenerate
    generate mis_c${tau}_B = c${tau}_B - c_star
		noi sum mis_c${tau}_B if year == ${tau}
		replace c_imp = c${tau}_B if year <= ${tau}
    save "${out_folder}/$outfolder/civilservant_shares_transitions_predicted", replace
		export delimited using "$out_folder/$outfolder/civilservant_shares_transitions_predicted.csv", ///
				replace nolabel delim("$csvdelim")
  restore 

  
} // tau



// Rentenvers
//------------------------------------------------------------------------------
generate rentenvers_imp = C == 0 if !missing(C, rentenvers)
label variable rentenvers_imp "rentenvers (imputed)"

// clean up
//------------------------------------------------------------------------------
sort personid year 
xtset personid year

save "$dta_folder/csimputation.dta", replace

// Save
//------------------------------------------------------------------------------
use "$dta_folder/${BASEDATA}.dta", clear

cap drop rentenvers_imp
merge 1:1 personid year using "$dta_folder/csimputation.dta", nogenerate ///
  keep(master match) keepusing(personid year rentenvers_imp)

save "$dta_folder/${BASEDATA}.dta", replace

tabstat rentenvers_imp rentenvers [aw=hrf], by(year) stat(mean) col(var)



//==========================================================================================
//  E N D 
//==========================================================================================
timer off 1
timer list 1

log close