//==========================================================================================
// This program computes weights for observations before 2012
//==========================================================================================


clear
set more off

// Initialize Settings
do "${do_folder}/001_Initialize.do"
qui do "${do_folder}/myprogs.do"
qui do "${do_folder}/data_privacy_restrictions.do"

// Output folder
global outfolder="$date 103_CorrectSampleSelection"
capture mkdir "${out_folder}/${outfolder}"

// Log file
local timestamp: disp %tcCCYY-NN-DD,_HH-MM tc(`c(current_date)' `c(current_time)') // example (2021-02-14, 14:54)
capture log close
capture noisily log using "${log_folder}/$outfolder (`timestamp').log", replace

timer clear 1
timer on 1


// Some settings
global rvcutoff = 60000
global minlabinc_weights = 1
global minage_weights = 16
global maxage_weights = 70

cap program drop generate_agegp
program define generate_agegp
	capture recode age (0/19 = 19) (66/70 = 70), generate(agegp)
	capture recode agegp (0/19 = 19) (66/70 = 70), replace
end


//==========================================================================================
//  R E G U L A R   W O R K E R S
//==========================================================================================

import delimited using "${dta_folder}/iab_weights_new.csv", clear
	rename niab n_target
	replace lb = min(lb, $rvcutoff)
	levelsof lb, local(levels_lb)
	generate_agegp
	gcollapse (sum) n_target , by(year male agegp lb)
save "${dta_folder}/iab_weights_work", replace


// Group TPP data
//==========================================================================================

// Load raw TPP data
use personid yob year male labinc mustfiler nonfiler rentenvers_imp hrf using "${dta_folder}/${BASEDATA}.dta", clear 

// Social security workers
keep if rentenvers_imp == 1

// Age 
generate age = year - yob + 1
generate_agegp

// Normalize earnings with CPI
generate labor = .
local k = 1
foreach y of numlist $yrfirst/$yrlast {
	replace labor = labinc / cpimat[`k',1] if year == `y'
	local ++k
}

// Earnings bins
create_bins labor, lbs(`levels_lb') binvar(bin) lbvar(lb) ubvar(ub)

// Sample restriction
keep if labinc >= $minlabinc_weights & labinc < . 

// Age restriction
keep if age >= $minage_weights & age <= $maxage_weights


keep if !missing(bin, age, male)

// Save (temporarily)
save "${dta_folder}/weights_rv.dta", replace 
save "${dta_folder}/weights_toprv.dta", replace 


//==========================================================================================
// COMPUTE WEIGHTS FOR ALL RV WORKERS
//==========================================================================================

use "${dta_folder}/weights_rv.dta", clear 

// (weighted) headcount in each group
gcollapse (rawsum) n=hrf (count) nraw=labor  [aw=hrf], by(year male agegp bin lb ub mustfiler)

// reshape
greshape wide n nraw, i(year male agegp bin lb ub) j(mustfiler)
rename n0 n_voluntary
rename n1 n_mustfiler
rename nraw0 nraw_voluntary
rename nraw1 nraw_mustfiler
foreach var in n_voluntary n_mustfiler nraw_voluntary nraw_mustfiler {
	replace `var' = 0 if missing(`var')
}
generate n = n_voluntary + n_mustfiler
generate nraw = nraw_voluntary + nraw_mustfiler
order nraw*, last

// merge with target headcounts
merge m:1 year male agegp lb using "${dta_folder}/iab_weights_work", keep(master match) nogenerate
replace n_target = n_target * $samplesize 

// initiate weights
generate weight_rv_voluntary = 1
generate weight_rv_mustfiler = 1

// Correct D2 for 2001-2011
//--------------------------------

// average (time-invariant) devation between 2012-2016 by group (where only D1 present)
generate adj_D1 = n_target / n if year >= 2012
bysort male agegp lb: gegen mean_adj_D1 = mean(adj_D1)

// number of missing nonfilers by year, male, agegp and bin
generate n_target_noD1 = n_target / mean_adj_D1
generate n_missing_noD1 = n_target_noD1 - n

// if n_missing_noD1 >= 0
replace weight_rv_voluntary = weight_rv_voluntary * (n_voluntary + n_missing_noD1) / n_voluntary if n_missing_noD1 >= 0 & year < 2012 & n_voluntary > 0
replace weight_rv_mustfiler = weight_rv_mustfiler * (n_mustfiler + n_missing_noD1) / n_mustfiler if n_missing_noD1 >= 0 & year < 2012 & n_voluntary == 0 & n_mustfiler > 0

// if n_missing_noD1 < 0 (not possible in Vollmaterial)
replace weight_rv_voluntary = weight_rv_voluntary * max(n_target_noD1 / n, 0) if n_missing_noD1 < 0 & year < 2012
replace weight_rv_mustfiler = weight_rv_mustfiler * max(n_target_noD1 / n, 0) if n_missing_noD1 < 0 & year < 2012

// Correct D1
//--------------------------------

// 2001-2011
replace weight_rv_voluntary = weight_rv_voluntary * mean_adj_D1 if year < 2012
replace weight_rv_mustfiler = weight_rv_mustfiler * mean_adj_D1 if year < 2012

// 2012-2016
replace weight_rv_voluntary = weight_rv_voluntary * n_target / n if year >= 2012
replace weight_rv_mustfiler = weight_rv_mustfiler * n_target / n if year >= 2012

// save
save "${out_folder}/$outfolder/summary_weights_rv", replace
export delimited using "${out_folder}/${outfolder}/summary_weights_rv.csv", nolabel replace delim("$csvdelim")


// Weights for RV workers
//==========================================================================================

// Merge weights to individual-year panel
use "${dta_folder}/weights_rv.dta", clear 

merge m:1 year male agegp lb using "${out_folder}/$outfolder/summary_weights_rv", ///
      nogenerate keep(master match) keepusing(year male agegp lb weight_rv_voluntary weight_rv_mustfiler)
			
			
// Insert weights 
generate weight_rv = 1
replace weight_rv = weight_rv_voluntary if mustfiler == 0
replace weight_rv = weight_rv_mustfiler if mustfiler == 1
			
// save weights
keep personid year hrf weight_rv male agegp lb mustfiler
save "${dta_folder}/weights_rv.dta", replace 



//==========================================================================================
// COMPUTE WEIGHTS FOR RV WORKERS ABOVE 
//==========================================================================================

/* 
	Sources of deviation:
		D1: for the years 2001-2016, the TPP only includes units with at least 2 waves of data
		D2: for the years 2001-2011, the TPP additionally does not include non-filers

	Procedure
		1. estimate average share of non-filers by G based on data from 2012-2016
		2. reweight voluntary filers in 2001-2011 to correct D2
				- Nhat_nonfiler = N_filer / (1-share_nonfiler) - N_filer
		3. reweight all observations in 2001-2016 to correct D1
*/

use "${dta_folder}/weights_toprv.dta", clear

// only top bracket (no detailed IAB info due to top coding at RV contribution limit)
keep if lb >= $rvcutoff
drop lb

// earnings groups by male and agegp within top bracket (time-invariant s)
bysort male agegp: create_pctclass labor [aw=hrf], generate(bintop) pctiles(5(5)95) lbvar(lbtop) ubvar(ubtop)

// save
save "${dta_folder}/weights_toprv.dta", replace


// Estimate share of non-filers by group
//==========================================================================================

// Average share of nonfilers (2012-2016, time-invariant)
//------------------------------------------------------------------------------
preserve
	gcollapse (count) N=nonfiler (mean) share_nonfiler=nonfiler [aw=hrf] if year >= 2012, ///
			by(male agegp bintop lbtop ubtop)
	save "${out_folder}/$outfolder/share_toprv_nonfiler_by_group", replace
restore


// Distribution of top RV workers across groups, by mustfiler
//-----------------------------------------------------------

// Number of top workers
gcollapse (count) nraw=personid (rawsum) n=hrf [aw=hrf], by(year male agegp bintop lbtop ubtop mustfiler)

// reshape
greshape wide nraw n, i(year male agegp bintop lbtop ubtop) j(mustfiler)
rename n0 n_voluntary
rename n1 n_mustfiler
rename nraw0 nraw_voluntary
rename nraw1 nraw_mustfiler
foreach var in n_voluntary n_mustfiler nraw_voluntary nraw_mustfiler {
	replace `var' = 0 if missing(`var')
}
generate n = n_voluntary + n_mustfiler
generate nraw = nraw_voluntary + nraw_mustfiler

// total by year, male and agegp (not by bintop)
bysort year male agegp: gegen n_voluntary_above60 = total(n_voluntary)
bysort year male agegp: gegen n_mustfiler_above60 = total(n_mustfiler)
generate n_above60 = n_voluntary_above60 + n_mustfiler_above60


// Merge targets
//----------------------------------------------------------

// IAB: total number of workers above 
preserve
	use "$dta_folder/iab_weights_work", clear
	keep if lb >= $rvcutoff
	tempfile toprvheadcount
	save "`toprvheadcount'", replace
restore
merge m:1 year male agegp using "`toprvheadcount'", nogenerate keepusing(year male agegp n_target)
rename n_target n_target_above60
replace n_target_above60 = n_target_above60 * $samplesize 

// Target share of nonfilers (merge on agegp here)
merge m:1 male agegp bintop using "${out_folder}/$outfolder/share_toprv_nonfiler_by_group", ///
					nogenerate keep(master match) keepusing(share_nonfiler)

					
// Compute weights
//--------------------------------------------------------

// initiate weights
generate weight_toprv_voluntary = 1
generate weight_toprv_mustfiler = 1

// Correct D2 for 2001-2011
//--------------------------------

// number of missing nonfilers by group (abstracting from D1)
generate n_missing_G = n / (1-share_nonfiler) - n // this is always >= 0

replace weight_toprv_voluntary = weight_toprv_voluntary * (n_voluntary + n_missing_G) / n_voluntary if year < 2012 & n_voluntary > 0
replace weight_toprv_mustfiler = weight_toprv_mustfiler * (n_mustfiler + n_missing_G) / n_mustfiler if year < 2012 & n_voluntary == 0 & n_mustfiler > 0 /* this should almost never be relevant in the Vollmaterial */

// Correct D1 for 2001-2016
//--------------------------------
bysort year male agegp: gegen n_above60_noD1 = total(n_voluntary * weight_toprv_voluntary + n_mustfiler * weight_toprv_mustfiler)

replace weight_toprv_voluntary = weight_toprv_voluntary * (n_target_above60 / n_above60_noD1)
replace weight_toprv_mustfiler = weight_toprv_mustfiler * (n_target_above60 / n_above60_noD1)
	
// Save
save "${out_folder}/$outfolder/summary_weights_toprv", replace
export delimited using "${out_folder}/${outfolder}/summary_weights_toprv.csv", nolabel replace delim("$csvdelim")


// Weights for top RV workers
//==========================================================================================

// Merge weights to individual-year panel
use "${dta_folder}/weights_toprv.dta", clear 

merge m:1 year male agegp bintop using "${out_folder}/$outfolder/summary_weights_toprv", ///
      nogenerate keep(master match) keepusing(year male agegp bintop weight_toprv_voluntary weight_toprv_mustfiler)

// Insert weights 
cap drop weight_toprv
generate weight_toprv = 1
replace weight_toprv = weight_toprv_voluntary if mustfiler == 0
replace weight_toprv = weight_toprv_mustfiler if mustfiler == 1

// save weights
keep personid year hrf weight_toprv male agegp bintop mustfiler
save "${dta_folder}/weights_toprv.dta", replace 


//==========================================================================================
//  C I V I L   S E R V A N T S
//==========================================================================================

/* 
	Sources of deviation:
		D1: for the years 2001-2016, the TPP only includes units with at least 2 waves of data (cannot do this properly due to lack of aggregates)
		D2: for the years 2001-2011, the TPP additionally does not include non-filers 

	Procedure
		1. estimate average share of non-filers by G based on data from 2012-2016
		2. reweight voluntary filers in 2001-2011 to correct D2
				- Nhat_nonfiler = N_filer / (1-share_nonfiler) - N_filer
*/

// Load raw TPP data
use personid yob year male labinc mustfiler nonfiler rentenvers_imp hrf using "${dta_folder}/${BASEDATA}.dta", clear 

// Age and age group
cap generate age = year - yob + 1
generate_agegp

// Normalize earnings with CPI
generate labor = .
local k = 1
foreach y of numlist $yrfirst/$yrlast {
	replace labor = labinc / cpimat[`k',1] if year == `y'
	local ++k
}

// Sample restriction
keep if labinc > 0 & labinc < . 
keep if !missing(mustfiler)
keep if rentenvers_imp == 0


// Group TPP data
//==========================================================================================

// earnings groups by male and agegp (time-invariant)
generate select_pct = year >= 2012 if !missing(year) // use only 2012-2016 to form time-invariant labor bins 
bysort male agegp: create_pctclass labor [aw=hrf], generate(bin_labor) pctiles(5 10(10)90 95) lbvar(lb_labor) ubvar(ub_labor) ifpct(select_pct)
drop select_pct

// combined group
global Gvars male agegp bin_labor lb_labor ub_labor
gegen G = group(male agegp bin_labor) if !missing(male, agegp, bin_labor), missing

drop if missing(G)

// Save (temporarily)
sort personid year
save "${dta_folder}/weights_cs.dta", replace 


// Estimate share of non-filers by group
//==========================================================================================

// Average share of nonfilers (2012-2016, time-invariant)
//------------------------------------------------------------------------------
preserve
	gcollapse (count) N=nonfiler (mean) share_nonfiler=nonfiler [aw=hrf] if year >= 2012, ///
			by(G ${Gvars})
	save "${out_folder}/$outfolder/share_cs_nonfiler_by_group", replace
restore


// Distribution of CS across groups, by mustfiler
//-----------------------------------------------------------

// Number of CS by G and mustfiler filing (N^0_gt and N^1_gt)
gcollapse (count) nraw=personid (rawsum) n=hrf [aw=hrf], by(year G ${Gvars} mustfiler)

// balance!
greshape wide nraw n, i(year G ${Gvars}) j(mustfiler)

rename n0 n_voluntary
rename n1 n_mustfiler
rename nraw0 nraw_voluntary
rename nraw1 nraw_mustfiler

foreach var in n_voluntary n_mustfiler nraw_voluntary nraw_mustfiler {
	replace `var' = 0 if missing(`var')
}

generate n = n_voluntary + n_mustfiler
generate nraw = nraw_voluntary + nraw_mustfiler

// total by year and gender (aggregation level of Destatis data)
bysort year male: gegen n_voluntary_gender = total(n_voluntary)
bysort year male: gegen n_mustfiler_gender = total(n_mustfiler)
generate n_gender = n_voluntary_gender + n_mustfiler_gender


// Merge targets
//----------------------------------------------------------

// Target share of nonfilers
merge m:1 G using "${out_folder}/$outfolder/share_cs_nonfiler_by_group", ///
					nogenerate keep(master match) keepusing(share_nonfiler)

					
// Compute weights
//--------------------------------------------------------

// initiate weights
generate weight_cs_voluntary = 1
generate weight_cs_mustfiler = 1

// Correct D2 for 2001-2011
//--------------------------------

// number of missing nonfilers by group (abstracting from D1)
generate n_missing_G = n / (1-share_nonfiler) - n // this is always >= 0

replace weight_cs_voluntary = weight_cs_voluntary * (n_voluntary + n_missing_G) / n_voluntary if year < 2012 & n_voluntary > 0
replace weight_cs_mustfiler = weight_cs_mustfiler * (n_mustfiler + n_missing_G) / n_mustfiler if year < 2012 & n_voluntary == 0 & n_mustfiler > 0 /* this should almost never be relevant in the Vollmaterial */


// Save
save "${out_folder}/$outfolder/summary_weights_cs", replace
export delimited using "${out_folder}/${outfolder}/summary_weights_cs.csv", nolabel replace delim("$csvdelim")


// Merge to individual-year panel
//==========================================================================================

use "${dta_folder}/weights_cs.dta", clear 
merge m:1 year G using "${out_folder}/$outfolder/summary_weights_cs", ///
			nogenerate keep(master match) keepusing(year G weight_cs_voluntary weight_cs_mustfiler)

// Insert weights 
cap drop weight_cs
generate weight_cs = 1
replace weight_cs = weight_cs_voluntary if mustfiler == 0
replace weight_cs = weight_cs_mustfiler if mustfiler == 1

// Save
sort personid year
keep personid year hrf weight_cs mustfiler G ${Gvars}
save "${dta_folder}/weights_cs.dta", replace 



//==========================================================================================
//  M E R G E   T O   B A S E D A T A
//==========================================================================================

// load ${BASEDATA}
use "${dta_folder}/${BASEDATA}.dta", clear 

// weights for RV workers
cap drop weight_rv
merge 1:1 personid year using "${dta_folder}/weights_rv.dta", nogenerate keep(master match) keepusing(personid year weight_rv)

// weights for above  RV workers
cap drop weight_toprv
merge 1:1 personid year using "${dta_folder}/weights_toprv.dta", nogenerate keep(master match) keepusing(personid year weight_toprv)

// weights for CS workers
cap drop weight_cs
merge 1:1 personid year using "${dta_folder}/weights_cs.dta", nogenerate keep(master match) keepusing(personid year weight_cs)

// Reweighting flag
cap drop _reweight
generate _reweight = !missing(weight_rv) | !missing(weight_cs)
						
// combined weight
// --------------------------------------------------------
cap drop weight
generate weight = hrf
replace weight = hrf * weight_rv if !missing(weight_rv)
replace weight = hrf * weight_toprv if !missing(weight_toprv)
replace weight = hrf * weight_cs if !missing(weight_cs)
label variable weight "hrf adjusted by weight_rv, weight_toprv and weight_cs"


// semi-scaled combined weight 
// (sum to N in each year, but rescale only non-reweighted observations)
// ----------------------------------------------------------------------

bysort year: gegen N_t = count(weight)

bysort year _reweight: gegen SUMweight_t_re = total(weight)
generate _aux = SUMweight_t_re if _reweight == 1
bysort year: gegen SUMweight_t_re1 = min(_aux)
drop _aux

cap drop sweight
generate sweight = weight
replace sweight = weight / SUMweight_t_re * (N_t - SUMweight_t_re1) if _reweight == 0
label variable sweight "scaled weight (sum of weights = N), rescaling only if _reweight = 0"

drop N_t SUMweight_t_re SUMweight_t_re1

// save
compress
save "${dta_folder}/${BASEDATA}.dta", replace 


// analyze weights
preserve 
	gcollapse (count) N=hrf 	(sum) sum_weight=weight (sum) sum_sweight=sweight (sum) sum_hrf=hrf ///
													(mean) mean_weight=weight (mean) mean_sweight=sweight (mean) mean_hrf=hrf, ///
													by(year _reweight)
	export 	delimited using "${out_folder}/${outfolder}/comparison_weight_sweight_hrf.csv", ///
					replace nolabel delim("$csvdelim")
restore

//==========================================================================================
//  C H E C K S
//==========================================================================================

// number of civil servants by gender
preserve
	use "${dta_folder}/weights_cs.dta", clear

	generate weight = hrf * weight_cs
	gcollapse (rawsum) n_cs_weight = weight (rawsum) n_cs_hrf = hrf, by(year male)
	
	merge m:1 year male using "$dta_folder/target_csheadcount_male", nogenerate keep(master match)
	
	replace n_cs_true = n_cs_true * $samplesize
  
  generate rdiff_weight = (n_cs_weight - n_cs_true) / n_cs_true
  generate rdiff_hrf = (n_cs_hrf - n_cs_true) / n_cs_true
  
  sum rdiff*
  
	export delimited using "$out_folder/$outfolder/check_n_cs.csv", replace nolabel delim("$csvdelim")
		
	list
restore

// RV workers by earnings bins
preserve 
	use "${dta_folder}/weights_rv.dta", clear
	generate weight = hrf * weight_rv
	merge 1:1 personid year using "${dta_folder}/weights_toprv.dta"
	replace weight = hrf * weight_toprv if !missing(weight_toprv)

	gcollapse (count) N=personid (rawsum) n_weight=weight, by(year male agegp lb)
	
	merge 1:1 year male agegp lb using "$dta_folder/iab_weights_work", ///
		nogenerate keep(master match)
	replace n_target = n_target * $samplesize
	
	generate rdiff = abs(n_weight - n_target) / n_target
	sum rdiff, d 
		
	export delimited using "$out_folder/$outfolder/check_n_rv.csv", replace nolabel delim("$csvdelim")

restore


// delete temporary files
*cap erase "${dta_folder}/weights_rv.dta"
*cap erase "${dta_folder}/weights_toprv.dta"
*cap erase "${dta_folder}/weights_cs.dta"

//==========================================================================================
//  E N D 
//==========================================================================================
timer off 1
timer list 1

cap log close


