/*==============================================================================

  This do-file gives a first overview of the TPP-Vollmaterial

==============================================================================*/

do "${do_folder}/001_Initialize.do"
do "${do_folder}/myprogs.do"

// Output folder
global outfolder="$date 105_OverviewData"
capture mkdir "${out_folder}/$outfolder"

// Log file
local timestamp: disp %tcCCYY-NN-DD,_HH-MM tc(`c(current_date)' `c(current_time)') // example (2021-02-14, 14:54)
capture log close
capture noisily log using "${log_folder}/$outfolder (`timestamp').log", replace

timer clear 1
timer on 1


global panel_dimension = 0
global labor_bins = 0

//------------------------------------------------------------
// Load data
//------------------------------------------------------------

use "$dta_folder/${BASEDATA}.dta", clear

// Draw sample for testing only
if $debug == 1 {
  generate _aux = runiform()
  bysort personid: gegen aux = first(_aux)
  if "$user" == "kdfv" keep if aux < 0.00001
  else keep if aux < 0.1
}

// Age restrictions
generate age = year - yob + 1
keep if age >= 25 & age <= 55

// Age group
create_bins age, lbs(25(5)50) binvar(bin_age) lbvar(lb_age) ubvar(ub_age)

// either RV or CS
generate rvORcs = !missing(rentenvers_imp)

// Real labor income
generate labor = .
local k = 1
foreach y of numlist $yrfirst/$yrlast {
	replace labor = labinc / cpimat[`k',1] if year == `y'
	local ++k
}

// Sample selection (weak)
keep if CS_noKV == 1

cap program drop data_privacy_overview
program data_privacy_overview
  syntax varlist, nvar(string)
  foreach var of varlist `varlist' {
    replace `var' = . if `nvar' < 30
  }
end


//------------------------------------------------------------
// Panel Dimension, Sampling
//------------------------------------------------------------
if $panel_dimension == 1 {

preserve
	bysort personid (year): gegen year_entry = min(year)
	bysort personid (year): gegen year_exit = max(year)
	bysort personid (year): gegen nyears = count(year)
	
	bysort personid (year): keep if _n == 1
	
	gcollapse (count) N=personid (rawsum) N_weight=$weight_var (rawsum) N_hrf=hrf ///
						(mean) yob [aw=$weight_var], by(male year_entry year_exit nyears)
						
  // DATA PRIVACY
	if $protect == 1 data_privacy_overview N_weight N_hrf male yob N, nvar(N)
  // Save
  export delimited using "${out_folder}/${outfolder}/panelstats_entry_exit_nyears.csv", delim("$csvdelim") replace nolabel

restore

}

//------------------------------------------------------------
// Distribution of labor income --> Bins
//------------------------------------------------------------

if $labor_bins == 1 {
			
// Earnings bins
create_bins labor,  lbs(0(1000)100000 105000(5000)120000 125000(25000)200000 300000 400000 500000 750000 1000000) ///
                    binvar(bin_labor) lbvar(lb_labor) ubvar(ub_labor)

// Split by RV / CS
preserve
  // Collapse data
  gcollapse  (count) N=labor (rawsum) N_weight=$weight_var (rawsum) N_hrf=hrf ///
            (mean) rentenvers mustfiler nonfiler _split male age labor [aw=$weight_var], ///
            by(rentenvers_imp year bin_labor lb_labor ub_labor) 
  // DATA PRIVACY
 	if $protect == 1 data_privacy_overview N_weight N_hrf rentenvers mustfiler nonfiler _split male age labor N, nvar(N)	
  // Save
  export delimited using "${out_folder}/${outfolder}/headcounts_labor_rvcs.csv", delim("$csvdelim") replace nolabel
restore

// Split by RV / CS by gender
preserve
  // Collapse data
  gcollapse  (count) N=labor (rawsum) N_weight=$weight_var (rawsum) N_hrf=hrf ///
            (mean) rentenvers mustfiler nonfiler _split age labor [aw=$weight_var], ///
            by(rentenvers_imp year male bin_labor lb_labor ub_labor) 

  // DATA PRIVACY
 	if $protect == 1 data_privacy_overview N_weight N_hrf rentenvers mustfiler nonfiler _split age labor N, nvar(N)	
  // Save
  export delimited using "${out_folder}/${outfolder}/headcounts_male_labor_rvcs.csv", delim("$csvdelim") replace nolabel
restore
										
// Split by RV / CS by gender and agegp
preserve
  // Collapse data
  gcollapse  (count) N=labor (rawsum) N_weight=$weight_var (rawsum) N_hrf=hrf ///
            (mean) rentenvers mustfiler nonfiler _split age labor [aw=$weight_var], ///
            by(rentenvers_imp year male bin_age lb_age ub_age bin_labor lb_labor ub_labor) 

  // DATA PRIVACY
 	if $protect == 1 data_privacy_overview N_weight N_hrf rentenvers mustfiler nonfiler _split age labor N, nvar(N)	
  // Save
  export delimited using "${out_folder}/${outfolder}/headcounts_maleagegp_labor_rvcs.csv", delim("$csvdelim") replace nolabel
restore


// Aggregate
preserve
  // Collapse data
  gcollapse  (count) N=labor (rawsum) N_weight=$weight_var (rawsum) N_hrf=hrf ///
            (mean) rentenvers_imp rentenvers mustfiler nonfiler _split age labor [aw=$weight_var], ///
            by(year bin_labor lb_labor ub_labor) 

  // DATA PRIVACY
 	if $protect == 1 data_privacy_overview N_weight N_hrf rentenvers_imp rentenvers mustfiler nonfiler _split age labor N, nvar(N)	
  // Save
  export delimited using "${out_folder}/${outfolder}/headcounts_labor.csv", delim("$csvdelim") replace nolabel
restore

// Aggregate by gender
preserve
  // Collapse data
  gcollapse  (count) N=labor (rawsum) N_weight=$weight_var (rawsum) N_hrf=hrf ///
            (mean) rentenvers_imp rentenvers mustfiler nonfiler _split age labor [aw=$weight_var], ///
            by(year male bin_labor lb_labor ub_labor) 

  // DATA PRIVACY
 	if $protect == 1 data_privacy_overview N_weight N_hrf rentenvers_imp rentenvers mustfiler nonfiler _split age labor N, nvar(N)	
  // Save
  export delimited using "${out_folder}/${outfolder}/headcounts_male_labor.csv", delim("$csvdelim") replace nolabel
restore

// Aggregate by gender and age group
preserve
  // Collapse data
  gcollapse  (count) N=labor (rawsum) N_weight=$weight_var (rawsum) N_hrf=hrf ///
            (mean) rentenvers_imp rentenvers mustfiler nonfiler _split age labor [aw=$weight_var], ///
            by(year male bin_age lb_age ub_age bin_labor lb_labor ub_labor) 

  // DATA PRIVACY
 	if $protect == 1 data_privacy_overview N_weight N_hrf rentenvers_imp rentenvers mustfiler nonfiler _split age labor N, nvar(N)	
  // Save
  export delimited using "${out_folder}/${outfolder}/headcounts_maleagegp_labor.csv", delim("$csvdelim") replace nolabel
restore

} // labor_bins


//------------------------------------------------------------
// Distribution of different income sources
//------------------------------------------------------------

local incvars labinc totinc totinc_noKV nonlabinc nonlabinc_noKV businc businc_noSE capinc capinc_noKV inc_kv inc_selfempl

local nz_incvars = ""
local neg_incvars = ""
foreach var in `incvars' {
  generate nonzero_`var' = `var' != 0
  generate neg_`var' = `var' != 0
  local nz_incvars `nz_incvars' nonzero_`var'
  local neg_incvars `neg_incvars' neg_`var'
}

// deflate by CPI
foreach incvar in `incvars' {
local k = 1
foreach y of numlist $yrfirst/$yrlast {
  replace `incvar' = `incvar' / cpimat[`k',1] if year == `y'
  local ++k
}
}

// main income type
tab mainsrc2, generate(mainsrc2)
tab mainsrc4, generate(mainsrc4)

// Analysis
foreach sample in "_logsample" "" {
foreach incvar in `incvars' {

  // Conditional statistics
  /*
  preserve

    if "`sample'" == "_logsample" keep if totinc_noKV >= 2300

    // create bins (defined by time-varying percentiles)
    bysort male:  create_pctclass `incvar' [aw=$weight_var] if `incvar' != 0, pctiles(0.01 0.1 1/99 99.9 99.99 99.999) ///
                  generate(bin_`incvar') lbvar(lb_`incvar') ubvar(ub_`incvar')
    replace bin_`incvar' = 0 if `incvar' == 0
    replace lb_`incvar'  = 0 if `incvar' == 0
    replace ub_`incvar'  = 0 if `incvar' == 0

    // gcollapse
    local byvars = "year male bin_`incvar' lb_`incvar' ub_`incvar'"
    local meanvars = "CS_labinc CS CS_noKV rentenvers_imp rvORcs mustfiler nonfiler age"
    gcollapse  (count) N=labor (rawsum) N_weight=$weight_var (rawsum) N_hrf=hrf ///
              (mean) `meanvars' `incvars' `nz_incvars' `neg_incvars' mainsrc2? mainsrc4? [aw=$weight_var], ///
              by(`byvars') 

    // DATA PRIVACY
    if $protect == 1 data_privacy_overview N_weight N_hrf `meanvars' `incvars' `nz_incvars' `neg_incvars' mainsrc2? mainsrc4? lb_`incvar' ub_`incvar' N, nvar(N)	
    // Save
    export delimited using "${out_folder}/${outfolder}/distribution_stats`sample'_`incvar'_male.csv", delim("$csvdelim") replace nolabel

  restore
  */

  // Summary statistics
  if "`sample'" == "_logsample" local ifcond  = "if totinc_noKV >= 2300"
  else local ifcond = ""
  bymydist_detail `incvar' `ifcond' [aw=$weight_var], by(year male) percentiles($pctiles_selected) ///
        saving("${out_folder}/${outfolder}/sumstats`sample'_`incvar'_male") 
  preserve
    use "${out_folder}/${outfolder}/sumstats`sample'_`incvar'_male", clear
    if $protect == 1 data_privacy_sumstats `incvar'
    export delimited using "${out_folder}/${outfolder}/sumstats`sample'_`incvar'_male.csv", delimiter("$csvdelim") replace
  restore

} // incvar
} // sample


// END
//==========================================================================================

timer off 1
timer list 1

cap log close
