/*==============================================================================

  This do-file gives a first overview of the TPP-Vollmaterial

==============================================================================*/

// Arguments
args arg_analysisyears arg_analysis arg_collect

// Clear
clear all
set more off

// Create folder for output and log-file
global outfolder="$date 106_Descriptives"
capture mkdir "${out_folder}/$outfolder"

// Log file
local timestamp: disp %tcCCYY-NN-DD,_HH-MM tc(`c(current_date)' `c(current_time)') // example (2021-02-14, 14:54)
capture log close
capture noisily log using "${log_folder}/$outfolder (`timestamp').log", replace

// Initialize Settings
qui do "${do_folder}/001_Initialize.do"

// Cd to the output file, create the program for moments, and load base sample.
cd "${out_folder}/$outfolder"
qui do "${do_folder}/myprogs.do"
qui do "${do_folder}/data_privacy_restrictions.do"

// Analysis
if "`arg_analysis'" == ""   global ANALYSIS = 1
else                        global ANALYSIS = "`arg_analysis'"
disp "arg_analysis = `arg_analysis', ANALYSIS = $ANALYSIS"

// Collect
if "`arg_collect'" == ""  global COLLECT = 1
else                      global COLLECT = "`arg_collect'"
disp "arg_collect = `arg_collect', COLLECT = $COLLECT"

// Analysis Years
if "`arg_analysisyears'" == ""  global ANALYSISYEARS = "$yrfirst/$yrlast"
else                            global ANALYSISYEARS = "`arg_analysisyears'"
disp "arg_analysisyears = `arg_analysisyears', ANALYSISYEARS = $ANALYSISYEARS"


// Settings
global incvars labinc totinc totinc_noKV nonlabinc_noKV businc_noSE capinc_noKV inc_kv inc_selfempl

timer clear 1
timer on 1


quietly {

if $ANALYSIS == 1 {

//==========================================================================================
//  D A T A
//==========================================================================================

foreach yr of numlist $ANALYSISYEARS { 

noi disp as text _n "Working in year `yr'"

// Load data for year yr
use personid year male hrf $weight_var yob CS_noKV mainsrc* $incvars ///
    using "$dta_folder/${BASEDATA}.dta", clear
keep if year == `yr'

// Draw sample for testing only
if $debug == 1 {
	sample 1
}

// Sample selection (weak)
keep if CS_noKV == 1
drop CS_noKV

// Age restrictions
generate age = year - yob + 1
keep if age >= 25 & age <= 55
drop yob

// deflate by CPI
foreach incvar in $incvars {
local k = 1
foreach y of numlist $yrfirst/$yrlast {
  replace `incvar' = `incvar' / cpimat[`k',1] if year == `y'
  local ++k
}
}

// Non-zero and negative incomes
local nz_incvars = ""
local neg_incvars = ""
foreach var in $incvars {
  generate nonzero_`var' = `var' != 0 if !missing(`var')
  generate neg_`var' = `var' < 0 if !missing(`var')
  local nz_incvars `nz_incvars' nonzero_`var'
  local neg_incvars `neg_incvars' neg_`var'
}

// Loop over weight variable
foreach w in $weight_var hrf {
	if "`w'" == "hrf" local wsuf = "_hrf"
	else local wsuf = ""

  // Loop over samples
  local samples = cond("`w'" == "hrf", `""""', `""" "_logsample""')
  foreach sample in `samples' {
    if "`sample'" == "_logsample" local ifcond  = "totinc_noKV >= 2300"
    else                          local ifcond = "1 == 1"

    noi disp as text "--> w = `w', sample = `sample'"

    // Basic Statistics
    //-----------------------------------------------------
    noi disp as text "... basic statistics"
    foreach ms in 2 4 {
      preserve 
        gcollapse (count) N=totinc_noKV (rawsum) sum_weights=$weight_var  ///
                  (mean) $incvars `nz_incvars' `neg_incvars' if `ifcond' [aw=`w'], /// 
                  by(year male age mainsrc`ms')
        save "basicstats`sample'`wsuf'_maleagemainsrc`ms'`yr'", replace
      restore
    }


    // Percentiles of Income Components (if non-zero)
    //-----------------------------------------------------
    foreach incvar in $incvars {
      noi disp as text "... percentiles of `incvar'"
      bymydist_detail `incvar' if `ifcond' & `incvar' != 0 [aw=`w'], by(year) ///
            percentiles(0.001 0.01 0.1 0.5(0.5)2.5 3/97 97.5(0.5)99.5 99.9 99.99 99.999) ///
            saving("sumstats`sample'`wsuf'_`incvar'_`yr'") 
      bymydist_detail `incvar' if `ifcond' & `incvar' != 0 [aw=`w'], by(year male) ///
            percentiles(0.001 0.01 0.1 0.5(0.5)2.5 3/97 97.5(0.5)99.5 99.9 99.99 99.999) ///
            saving("sumstats`sample'`wsuf'_`incvar'_male`yr'") 
    } // incvar


    // Correlations of income sources
    //-----------------------------------------------------
    noi disp as text "... correlations of components"
    capture noisily { // There was a bug that occurs only at FDZ, not in test run

    foreach m in 0 1 {
      // Bad code because of bug
      noi correlate labinc totinc totinc_noKV nonlabinc_noKV businc_noSE capinc_noKV inc_kv inc_selfempl if male == `m' & `ifcond' [aw=`w']
      matrix C = r(C)
      local numobs = r(N)

      // write matrix to dta
      preserve
        clear
        svmat C, names(col)
        generate N = `numobs'
        generate year = `yr'
        generate male = `m'
        generate incvar = ""
        local k = 1
        foreach incvar in $incvars {
          replace incvar = "`incvar'" if _n == `k'
          local ++k
        }
        order year male N incvar
        save "correlations_incomesources_male`m'`yr'", replace
      restore
    } // m 

    preserve
      clear 
      foreach m in 0 1 {
        append using "correlations_incomesources_male`m'`yr'"
        erase "correlations_incomesources_male`m'`yr'.dta"
      }
      save "correlations`sample'`wsuf'_incomesources_male`yr'", replace
    restore

    } // capture

  } // w
} // sample


} // Loop over years


} // ANALYSIS

//------------------------------------------------------------------------------
} // quietly
//------------------------------------------------------------------------------


//======================================================================================================
// COLLECT DATA ACROSS YEARS
//======================================================================================================

if $COLLECT == 1 {


// DATA PRIVACY PROGRAM
cap program drop data_privacy_descriptives
program data_privacy_descriptives
  syntax , varnames(varlist) nvar(string)
  foreach var of varlist `varnames' {
    replace `var' = . if `nvar' < 30
  }
end


noi disp as text _n "Collecting data across years"

// Loop over weight variable
foreach w in $weight_var hrf {
	if "`w'" == "hrf" local wsuf = "_hrf"
	else local wsuf = ""

  // Loop over samples
  local samples = cond("`w'" == "hrf", `""""', `""" "_logsample""')
  foreach sample in `samples' {
    if "`sample'" == "_logsample" local ifcond  = "totinc_noKV >= 2300"
    else                          local ifcond = "1 == 1"

    // Basic Statistics
    //-------------------------------------------
    noi disp as text "... basic statistics"
    foreach ms in 2 4 {
      capture append_and_export , filestub("basicstats`sample'`wsuf'_maleagemainsrc`ms'") years($yrfirst/$yrlast) dataprivacy(data_privacy_descriptives) varnames(sum_weights $incvars `nz_incvars' `neg_incvars' N) nvar(N)
    }  


    // Percentiles of Income Components (if non-zero)
    //-----------------------------------------------------
    foreach incvar in $incvars {
      noi disp as text "... percentiles of components"
      capture append_and_export , filestub("sumstats`sample'`wsuf'_`incvar'_") years($yrfirst/$yrlast) dataprivacy(data_privacy_sumstats) var(`incvar')
      capture append_and_export , filestub("sumstats`sample'`wsuf'_`incvar'_male") years($yrfirst/$yrlast) dataprivacy(data_privacy_sumstats) var(`incvar')
    } // incvar

    // Correlations of income sources
    //-----------------------------------------------------
    noi disp as text "... correlations of components"
    capture append_and_export , filestub("correlations`sample'`wsuf'_incomesources_male") years($yrfirst/$yrlast) dataprivacy(data_privacy_descriptives) varnames($incvars N) nvar(N)
    

  } // w
} // sample


} // COLLECT



// END
//==========================================================================================

timer off 1
timer list 1

cap log close
