//==========================================================================================
// This program generates the base sample
// Moritz Drechsel-Grau & Kai Schmid (based on code by Halvorsen, Ozkan, Salgado)
//==========================================================================================


/*==========================================================================================

This do file renames the variables, does very basic sample selection, creates new variables
	(e.g., log and residual earnings), and generates the master_sample.dta, which is a wide
	form dataset which will be used in the rest of the do files.

The main output of this do file (master_sample.dta) is saved in the folder ${dta_folder}
	and contains the following variables:
	(a) personid: id of the individual used throughout the do files
	(b) male: indicator variable equal to 1 if male and 0 if female
	(c) yob: year of birth of the individual
	(d) labor: real labor earnings in levels
	(e) logearn: real labor earnings in log levels
	(f) permearn: permanent income defined as avg. income from t-3 to t-1
	(g) permearnalt: alternative measure of permanent income
	(h) researn: residual of log earnings (from regressions on age dummies)
	(i) researn1F: 1-year forward residualized log earnings change
	(j) researn5F: 5-year forward residualized log earnings change
	(k) arcearn1F: 1-year forward netarc earnings change
	(l) arcearn5F: 5-year forward netarc earnings change

==========================================================================================*/


// Start
// -----------------------------------------------------------------------------------------

// Start log file
local timestamp: disp %tcCCYY-NN-DD,_HH-MM tc(`c(current_date)' `c(current_time)') // example (2021-02-14, 14:54)
capture log close
capture noisily log using "${log_folder}/${date} 201_Gen_Base_Sample (`timestamp').log", replace

// Clear workspace
clear all
set more off

// Initialize Settings
qui do "${do_folder}/001_Initialize.do"

// Option RV-only
if "$rvonly" == "yes" local rvsuffix = "_rv"
else 									local rvsuffix = ""

// CD to data folder
cap mkdir "${dta_folder}/labor`rvsuffix'"
cd "${dta_folder}/labor`rvsuffix'"


quietly {

noi disp as text _n "START ..."


// Load Data and Reshape
//==================================================================================================

noi disp as text "... load data and reshape to wide format"

// Reshape data in long format to wide. Faster than RESHAPE command.

// Load data
use $weight_var $rawweight_var $personid_var $male_var $yob_var $year_var $labor_var $rv_var  ///
  if ${year_var} >= ${yrfirst} & ${year_var} <= ${yrlast} using "${basedata}"
  
// Option RV-only !!!
if "$rvonly" == "yes" {
  keep if $rv_var == 1
}

// Rename variables
rename $personid_var personid
rename $year_var year
rename $male_var male
rename $yob_var yob
rename $labor_var labor
rename $weight_var weight
rename $rawweight_var hrf

noi describe
noi tab year


// Extract annual cross sections
sort $personid_var $year_var
forvalues yr = $yrfirst/$yrlast{
  preserve
  keep if $year_var ==`yr'

  keep personid male yob labor weight hrf
    // Need to keep all variables. If only keep for first year, we
    // miss the data for observations that enter in the sample after
    // the first year.

  rename labor labor`yr'
  rename weight weight`yr'
  label var labor`yr' "Real labor earnings in `yr'"
  label var weight`yr' "Sample weight in `yr'"

  sort personid
  save "temp`yr'.dta",replace
  restore
}

// Merge cross sections
local first=$yrfirst+1
use "temp$yrfirst.dta", clear
erase "temp${yrfirst}.dta"
forvalues yr = `first'/$yrlast{
  merge 1:1 personid using "temp`yr'.dta", nogen update replace
      // update replace makes sure the variables such as year of birth
      // are replaced by non-missing values for observations that enter
      // in the sample after the first year.
  erase "temp`yr'.dta"
  sort personid
}

order personid male yob labor* weight* hrf


// Create Base Sample
//==================================================================================================

noi disp as text "... create base sample --> base_sample.dta"


// Drop anybody who is too old or too young or too dead.
// Criteria 1 (Age) in CS Sample
drop if yob == .
drop if $yrfirst-yob + 1 > $end_age | $yrlast-yob + 1<$begin_age
describe

global base_price_index = ${base_price}-${yrfirst}+1	// The base year nominal values are converted to real.


// Loop over years
forvalues yr = $yrfirst/$yrlast{

	// Covert to real values
	// Criteria d (Inflation) in CS Sample

	local cpi_index = `yr'-${yrfirst}+1
	replace labor`yr' = labor`yr' / cpimat[`cpi_index',1]		//Coverting to real values

	// Winsorization
	gen temp=labor`yr' if `yr'-yob + 1 >= $begin_age & `yr'- yob + 1<= $end_age
	_pctile temp [aw=weight`yr'], p($winsor)
	replace labor`yr'= r(r1) if labor`yr'>=r(r1) & labor`yr'!=.
	drop temp

	// Add a small noise
	gen temp=${noise}*(uniform()-0.5)
	replace labor`yr'=labor`yr'+labor`yr'*temp
	drop temp

	if (${miss_earn}==0) {
	// Any earnings that are missing inside of $begin_age and $end_age are set to zero.
	replace labor`yr'= 0 if labor`yr'== . & `yr'-yob + 1 >= $begin_age & `yr'- yob + 1 <= $end_age
	}

	// Assign missing if outside of $begin_age and $end_age
	replace labor`yr'= . if `yr'-yob + 1 < $begin_age | `yr'- yob + 1 > $end_age

}

// Base sample creation completed.
order personid male yob weight* labor*
compress
save "base_sample.dta", replace



// Create residuals of log-earnings and age profiles (for each year separately, by gender)
//==================================================================================================
noi disp as text "... create age profiles and residuals of log-earnings (by year and gender) --> researn.dta, age_dums.dta"


// Loop over years
forvalues yr = $yrfirst/$yrlast{

	// Load data for year yr
	use personid male yob weight`yr' labor`yr' using ///
	"base_sample.dta" if labor`yr' != . , clear

	// Create year
	generate year = `yr'

	// Create age
	generate age = `yr'-yob + 1
	drop if age<${begin_age} | age> ${end_age}

	// Create log earn if earnings above the min treshold
	// Criteria c (Trimming) in CS Sample
	// Notice we do not drop the observations but log-earnings are generated for those with
	// income below 1/3*min threshold. Variable logearn`yr'c is used for growth rates conditional
	// on permanent income only

	generate logearn`yr' = log(labor`yr') if labor`yr'>=rmininc[`yr'-${yrfirst}+1,1] & labor`yr'!=.
	generate logearnc`yr' = log(labor`yr') if labor`yr'>=(1/3)*rmininc[`yr'-${yrfirst}+1,1] & labor`yr'!=.
	drop if logearnc`yr' == . & logearn`yr' == .
	label var logearn`yr' "Real log-labor earnings of year `yr' above min threshold"
	label var logearnc`yr' "Real log-labor earnings of year `yr' above 1/3*min threshold"

	// Create dummies for age groups
	tab age, gen(agedum)
	drop agedum1

	// Regressions for residual earnings (by gender)
	//--------------------------------------------------------------------------
	// Save age dummies
	statsby _b N=e(N),  by(year) saving(age_yr`yr'_m,replace):  ///
	regress logearn`yr' agedum* [aw=weight`yr'] if male==1

	predict temp_m if e(sample)==1, resid

	statsby _b N=e(N),  by(year) saving(age_yr`yr'_f,replace):  ///
	regress logearn`yr' agedum* [aw=weight`yr'] if male==0

	predict temp_f if e(sample)==1, resid

	// Regressions for residual earnings with income above 1/3*minincome (by gender)
	regress logearnc`yr' agedum* [aw=weight`yr'] if male==1
	predict temp_m_c if e(sample)==1, resid

	regress logearnc`yr' agedum* [aw=weight`yr'] if male==0
	predict temp_f_c if e(sample)==1, resid

	// Generate the residuals by year and save a dataset for later append.
	gen researn`yr'= temp_m
	replace researn`yr'= temp_f if male==0
	label var researn`yr' "Residual of real log-labor earnings of year `yr'"

	gen researnc`yr'= temp_m_c
	replace researnc`yr'= temp_f_c if male==0
	label var researnc`yr' "Residual of real log-labor earnings of year `yr' above 1/3*min threshold"

	// Drop temporary variables
	drop temp_*

	// Save data set for later append
	preserve
		keep personid male yob weight`yr' researn`yr' researnc`yr' logearn`yr' logearnc`yr' labor`yr' 
		sort personid
		save "researn`yr'.dta", replace
	restore

}

// Appending residual of real log-labor earnings
forvalues yr = $yrfirst/$yrlast{
	if (`yr' == $yrfirst) {
		use "researn`yr'.dta", clear
		erase "researn`yr'.dta"
	}
	else{
		merge 1:1 personid using "researn`yr'.dta", nogen
		erase "researn`yr'.dta"
	}
	sort personid
}
save "researn.dta", replace

// Appending coefficients of age profiles for gender groups
clear
forvalues yr = $yrfirst/$yrlast{
	append using "age_yr`yr'_m.dta"
	cap: gen male = 1
	cap: replace male = 1 if male == .
	erase "age_yr`yr'_m.dta"
	append using "age_yr`yr'_f.dta"
	cap: replace male = 0 if male == .
	erase "age_yr`yr'_f.dta"
}
rename _eq2_N N // number of observations of year specific regression
order year male N
save "age_dums.dta", replace



// Calculate growth of (residual) earnings (Section 2.e and 2.f)
//==================================================================================================
noi disp as text "... calculate earnings growth rates (arc and log, k = 1,5) --> researn1F.dta, researn5F.dta"

clear
foreach k in 1 5 {

	// Given the jump k, calculate the growth rate for each worker in each year

	local lastyr=$yrlast-`k'
	forvalues yr = $yrfirst/`lastyr'{

		local yrnext=`yr'+`k'

		use personid male yob weight`yr' researn`yr' researnc`yrnext' labor`yrnext' labor`yr' using "researn.dta", clear

		gen age = `yr'-yob + 1

		bys male age: gegen avelabor`yrnext' = mean(labor`yrnext')
		bys male age: gegen avelabor`yr' = mean(labor`yr')

		gen researn`k'F`yr'= researnc`yrnext'-researn`yr'		// Growth with earnings above mininc in t and 1/3*mininc in t+k
		gen arcearn`k'F`yr'= (labor`yrnext'/avelabor`yrnext' - labor`yr'/avelabor`yr')/(0.5*(labor`yrnext'/avelabor`yrnext' + labor`yr'/avelabor`yr'))

		label var researn`k'F`yr'  "Residual earnings growth between `yrnext' and `yr'"
		label var arcearn`k'F`yr'  "Arc-percent earnings growth between `yrnext' and `yr'"

		keep personid weight`yr' researn`k'F`yr' arcearn`k'F`yr'
		save "researn`k'F`yr'.dta", replace

	}

	// Collect annual data
	forvalues yr = $yrfirst/`lastyr'{

		if (`yr' == $yrfirst) {
		use "researn`k'F`yr'.dta", clear
		erase "researn`k'F`yr'.dta"
		}
		else{
			merge 1:1 personid using "researn`k'F`yr'.dta", nogen
			erase "researn`k'F`yr'.dta"
		}
		sort personid
	}

	compress
	save "researn`k'F.dta", replace

}


// Calculate permanent income
//==================================================================================================
noi disp as text "... calculate permanent income --> permearn.dta"

clear
local firstyr=$yrfirst+2
forvalues yr = `firstyr'/$yrlast{
	local yrL1=`yr'-1
	local yrL2=`yr'-2

	use personid male yob weight`yr' labor`yrL2' labor`yrL1' labor`yr' using ///
	"base_sample.dta" if labor`yr' != . , clear

	// Create year
	gen year=`yr'

	// Create age
	gen age = `yr'-yob + 1
	drop if age<${begin_age} | age> ${end_age}

	// Create average income for those with at least 2 years of income above
	// the treshold income between t-1 and t-3
	gen totearn=0
	gen numobs=0

	*replace numobs = -5 if labor`yr' < rmininc[`yr'-${yrfirst}+1,1]
	// This ensures that permanent income is only constructed for those
	// with income above the threshold in t-1


	forvalues yrp=`yrL2'/`yr'{
		replace totearn=totearn+labor`yrp' if labor`yrp' != .
		replace numobs=numobs+1 if labor`yrp'>=rmininc[`yrp'-${yrfirst}+1,1] & labor`yrp' != .
		// Notice earnings below the min threshold are still used to get totearn
	}

	replace totearn=totearn/numobs if numobs>=2			// Average income
	drop	if numobs<2									// Drop if less than 2 obs

	// Create log of totearn
	generate log_totearn = log(totearn)
	drop if log_totearn==.

	// Gen dummies for regressions
	tab age, gen(agedum)
	drop agedum1

	// Regression to get residuals permanent income (add mean to residuals)
	regress log_totearn agedum* [aw=weight`yr'] if male==1
	predict temp_m if e(sample)==1, resid
	qui sum log_totearn [aw=weight`yr'] if male==1 & e(sample)==1
	replace temp_m = temp_m + r(mean)

	qui regress log_totearn agedum* [aw=weight`yr'] if male==0
	predict temp_f if e(sample)==1, resid
	qui sum log_totearn [aw=weight`yr'] if male==0 & e(sample)==1
	replace temp_f = temp_f + r(mean)

	gen permearn`yr'= temp_m
	replace permearn`yr'= temp_f if male==0

	label var permearn`yr' "Residual permanent income between `yr' and `yrL2'"
	
	// Save average past earnings (residualized)
	generate level_permearn`yr' = exp(permearn`yr')
	label var level_permearn`yr' "Level of permanent income between `yr' and `yrL2'"
	
	// Save average past earnings
	rename totearn avg_pastearn`yr'
	label var avg_pastearn`yr' "Average income between `yr' and `yrL2'"	

	// Save
	keep personid weight`yr' permearn`yr' level_permearn`yr' avg_pastearn`yr'
	compress
	sort personid
	save "permearn`yr'.dta", replace

}

// Collect annual data
clear
local firstyr=$yrfirst+2
forvalues yr = `firstyr'/$yrlast{

	if (`yr' == `firstyr') {
		use "permearn`yr'.dta", clear
		erase "permearn`yr'.dta"
	}
	else{
		merge 1:1 personid using "permearn`yr'.dta", nogen
		erase "permearn`yr'.dta"
	}
	sort personid
}
save "permearn.dta", replace



// Calculate modified permanent income
//==================================================================================================
noi disp as text "... calculate modified permanent income (all individuals) --> permearnalt.dta"
/*
   Relative to the previos version, here we consider all individuals
   even those with low earnimgs. See section "Key Statisitcs 4: Mobilitity"
*/
clear
local firstyr=$yrfirst+2
forvalues yr = `firstyr'/$yrlast{
	local yrL1=`yr'-1
	local yrL2=`yr'-2

	use personid male yob weight`yr'  labor`yrL1' labor`yrL2' labor`yr' using ///
	"base_sample.dta" if labor`yr' != . , clear

	// Create year
	gen year=`yr'

	// Create age
	gen age = `yr'-yob + 1
	drop if age<${begin_age} + 2 | age> ${end_age}			// This makes the min age 26 and max age 54

	// Create average income for those with at least 2 years of income
	gen totearn=0
	gen numobs=0
	gen numobspos=0

	forvalues yrp=`yrL2'/`yr'{
		replace totearn=totearn+labor`yrp' if labor`yrp' != .
		replace numobs=numobs+1 if labor`yrp' != .
		replace numobspos=numobspos+1 if labor`yrp'>= rmininc[`yrp'-${yrfirst}+1,1] & labor`yrp' != .
			// Notice earnings below the min threshold are still used to get totearn
			// This ensure we do not consider income of individuals when they were 24 yrs old or less
	}
	replace totearn=totearn/numobs if numobs==3			// Average income
	drop if numobs<3			// Drop if less than 2 obs
	drop if numobspos < 1 		// Drop if less than 1 obs above min income
	drop if totearn==.

	gen permearnalt`yr' = totearn
	label var permearnalt`yr' "Altenative permanent income: average between `yr' and `yrL2'"
	
	// Gen dummies for regressions
	tab age, gen(agedum)
	drop agedum1

	// residualized modified permanent income
	gen log_totearn = log(totearn)
	
	regress log_totearn agedum* [aw=weight`yr'] if male==1
	predict temp_m if e(sample)==1, resid
	qui sum log_totearn [aw=weight`yr'] if male==1 & e(sample)==1
	replace temp_m = temp_m + r(mean)

	qui regress log_totearn agedum* [aw=weight`yr'] if male==0
	predict temp_f if e(sample)==1, resid
	qui sum log_totearn [aw=weight`yr'] if male==0 & e(sample)==1
	replace temp_f = temp_f + r(mean)

	gen respermearnalt`yr'= temp_m
	replace respermearnalt`yr'= temp_f if male==0

	label var respermearnalt`yr' "Residualized altenative permanent income: average between `yr' and `yrL2'"

	
	// Save
	keep personid weight`yr' permearnalt`yr' respermearnalt`yr'


	compress
	sort personid
	save "permearnalt`yr'.dta", replace

}

// Collect annual data
clear
local firstyr=$yrfirst + 2
forvalues yr = `firstyr'/$yrlast{

	if (`yr' == `firstyr') {
		use "permearnalt`yr'.dta", clear
		erase "permearnalt`yr'.dta"
	}
	else{
		merge 1:1 personid using "permearnalt`yr'.dta", nogen
		erase "permearnalt`yr'.dta"
	}
	sort personid
}
save "permearnalt.dta", replace




// Merge all data sets to a master file
//==================================================================================================
noi disp as text "... merge all data sets to a master file --> master_sample.dta"

use "base_sample.dta", clear

merge 1:1 personid using "permearn.dta", nogen
merge 1:1 personid using "permearnalt.dta", nogen
merge 1:1 personid using "researn.dta", nogen
merge 1:1 personid using "researn1F.dta", nogen
merge 1:1 personid using "researn5F.dta", nogen

compress
order  personid male yob labor* logearn* permearn* researn* respermearnalt* avg_pastearn* level_permearn*
save "master_sample.dta", replace

erase "permearn.dta"
erase "permearnalt.dta"
erase "researn.dta"
erase "researn1F.dta"
erase "researn5F.dta"
erase "base_sample.dta"

} // END of quietly

noi disp as text "... END"

capture log close



// END OF DO-FILE
//////////////////////////////////////////////
