//==========================================================================================
// This program generates the descriptive statistics
// Moritz Drechsel-Grau
//==========================================================================================

/*==========================================================================================

	Dieses do-file erstellt deskriptive Statistiken zur Variable Arbeitseinkommen (labor).
  
==========================================================================================*/

// Arguments
args arg_analysisyears arg_analysis arg_collect

// Clear
clear all
set more off

// Option RV-only
if "$rvonly" == "yes" local dtafolder = "${dta_folder}/labor_rv"
else 									local dtafolder = "${dta_folder}/labor"

// Create folder for output and log-file
global outfolder="$date 206_Distributions${foldersuffix}"
capture mkdir "${out_folder}/$outfolder"
capture mkdir "${out_folder}/$outfolder/inequality"
capture mkdir "${out_folder}/$outfolder/volatility"
capture mkdir "${out_folder}/$outfolder/mobility"

// Log file
local timestamp: disp %tcCCYY-NN-DD,_HH-MM tc(`c(current_date)' `c(current_time)') // example (2021-02-14, 14:54)
capture log close
capture noisily log using "${log_folder}/$outfolder (`timestamp').log", replace

// Initialize Settings
qui do "${do_folder}/001_Initialize.do"

// Cd to the output file, create the program for moments, and load base sample.
cd "${out_folder}/$outfolder"
qui do "${do_folder}/myprogs.do"
qui do "${do_folder}/data_privacy_restrictions.do"

// Analysis
if "`arg_analysis'" == ""   global ANALYSIS = 1
else                        global ANALYSIS = "`arg_analysis'"
disp "arg_analysis = `arg_analysis', ANALYSIS = $ANALYSIS"

// Collect
if "`arg_collect'" == ""  global COLLECT = 1
else                      global COLLECT = "`arg_collect'"
disp "arg_collect = `arg_collect', COLLECT = $COLLECT"

// Analysis Years
if "`arg_analysisyears'" == ""  global ANALYSISYEARS = "$yrfirst/$yrlast"
else                            global ANALYSISYEARS = "`arg_analysisyears'"
disp "arg_analysisyears = `arg_analysisyears', ANALYSISYEARS = $ANALYSISYEARS"


//==========================================================================================
//  S E T T I N G S
//==========================================================================================

// Inequality variables
global ineqvars labor logearn researn permearn

// Growth rate variables
global growthvars researn //arcearn

// Defines the number of points in the Kernel Density Estimator
global kpoints =  400

// Sections
global inequality_sumstats      = 1
global inequality_incomeshares  = 1
global inequality_densities		  = 1
global volatility_sumstats      = 1
global volatility_bycurrent     = 1
global volatility_byperm        = 1
global volatility_densities     = 1
global volatility_autocorr			= 1
global mobility_transitions     = 1
global mobility_ranks           = 1


timer clear 1
timer on 1


quietly {

 
if $ANALYSIS == 1 {

//==========================================================================================
//  D A T A
//==========================================================================================


foreach yr of numlist $ANALYSISYEARS { 

noi disp as text _n "Working in year `yr'"


local yr5F = `yr'+5   // t+5
local yr1F = `yr'+1   // t+1
local yrp  = `yr'-1		// t-1

// Variables to load
local vars = "male yob hrf weight`yr' labor`yr' logearn`yr' researn`yr'"
local flist = ""

if inlist(`yr', $d1yrlistcomma) {
	local vars "`vars' labor`yr1F' researn1F`yr' arcearn1F`yr' weight`yr1F'"
	local flist "`flist' 1"
}

if inlist(`yr',${d5yrlist}) {
	local vars "`vars' labor`yr5F' researn5F`yr' arcearn5F`yr' weight`yr5F'"
	local flist "`flist' 5"
}

if inlist(`yrp',${perm3yrlist}) {
	local vars "`vars' permearn`yrp' avg_pastearn`yrp'"
}

if inlist(`yr', ${perm3yrlist}) {
	local vars "`vars' permearn`yr'"
}

if inlist(`yr', ${perm3yrlist}) & inlist(`yr', ${d1yrlistcomma}) {
	local vars "`vars' permearnalt`yr' respermearnalt`yr' permearnalt`yr1F' respermearnalt`yr1F'"
}

if inlist(`yr', ${perm3yrlist}) & inlist(`yr', ${d5yrlist}) {
	local vars "`vars' permearnalt`yr5F' respermearnalt`yr5F'"
}

foreach k of numlist 1/5 {
	local vars = cond(`yr' <= $yrlast - `k', "`vars' researn`=`yr'+`k''", "`vars'")
}
foreach k of numlist 2/4 {
	local vars = cond(`yr' <= $yrlast - `k', "`vars' labor`=`yr'+`k''", "`vars'")
}

// Load base data
noi disp as text "... load and prepare data"
use `vars' using "`dtafolder'/master_sample.dta" if labor`yr'~=. , clear


// Draw sample for testing only
if $debug == 1 {
	sample 50
}


// Rename variables	
rename weight`yr' weight
cap rename weight`yr1F' weight1F
cap rename weight`yr5F' weight5F

rename labor`yr' labor
rename logearn`yr' logearn
rename researn`yr' researn
cap rename labor`yr1F' labor1F
cap rename labor`yr5F' labor5F
cap generate topcoded1F = labor > 60000 | labor1F > 60000 if !missing(labor, labor1F)
cap generate topcoded5F = labor > 60000 | labor5F > 60000 if !missing(labor, labor5F)

cap rename labor`=`yr'+2' labor2F
cap rename labor`=`yr'+3' labor3F
cap rename labor`=`yr'+4' labor4F

cap rename researn`=`yr'+1' researn_1F
cap rename researn`=`yr'+2' researn_2F
cap rename researn`=`yr'+3' researn_3F
cap rename researn`=`yr'+4' researn_4F
cap rename researn`=`yr'+5' researn_5F

cap rename researn1F`yr' researn1F	
cap rename researn5F`yr' researn5F	
cap rename arcearn1F`yr' arcearn1F	
cap rename arcearn5F`yr' arcearn5F	
cap rename permearn`yrp' permearn1L
cap rename permearn`yr' permearn
cap rename avg_pastearn`yrp' avg_pastearn
cap rename permearnalt`yr' permearnalt
cap rename permearnalt`yr1F' permearnalt1F
cap rename permearnalt`yr5F' permearnalt5F
cap rename respermearnalt`yr' respermearnalt
cap rename respermearnalt`yr1F' respermearnalt1F
cap rename respermearnalt`yr5F' respermearnalt5F

// drop arcearn variables if not used
if strpos("`growthvars'", "arc") == 0 cap drop arcearn*

// Generate year
gen year = `yr'

// Age group
gen age = `yr'-yob+1
gen agegp = .
replace agegp = 1 if age <= 34 & agegp == .
replace agegp = 2 if age <= 44 & agegp == .
replace agegp = 3 if age <= 55 & agegp == .

// Age restriction
drop if age > $end_age | age < $begin_age


// CS sample indicator
generate CSsample = logearn != .


//==========================================================================================
//  I N E Q U A L I T Y 
//==========================================================================================


// Labor Income Percentiles
//------------------------------------------------------------------------------------------

if $inequality_sumstats == 1 {
noi disp as text "... labor income summary statistics"

foreach w in weight hrf {
	if "`w'" == "hrf" local wsuf = "_hrf"
	else local wsuf = ""

foreach var in $ineqvars {
if "`var'" != "permearn" | inlist(`yr',${perm3yrlist}) {

  // loop over (sub)populations
  foreach het in "" $hetgrouplabor_ineq_sumstats {
    local suf=subinstr("`het'"," ","",.)

			local ginitheil = cond("`var'" == "labor", "ginitheil", "")
			if "`var'" == "logearn" 		local pctiles "0.5(0.5)5 6(1)95 95.5(0.5)99.5 99.9 99.95 99.99 99.995 99.999"
			else if "`var'" == "labor" 	local pctiles = "50"
			else 												local pctiles = "$pctiles_selected"

				bymydist_detail `var' [aw=`w'] if CSsample == 1, saving("inequality/sumstats`wsuf'_`var'_`suf'`yr'") by(year `het') ///
					percentiles(`pctiles') `ginitheil'


	} // het
} // var
} // if
} // w

} // inequality_sumstats


// Labor Income Shares
//------------------------------------------------------------------------------------------

if $inequality_incomeshares == 1 {
noi disp as text "... labor income shares"

// loop over (sub)populations
foreach het in "" $hetgrouplabor_ineq_shares {
	local suf=subinstr("`het'"," ","",.)

	// create fractiles
	cap drop ranklabor lblabor ublabor
	bysort year `het': create_pctclass labor [aw=weight] if CSsample == 1, ///
          generate(ranklabor) lbvar(lblabor) ubvar(ublabor) pctiles(10(10)90 95 99 99.9 99.99 99.999)

	// average income within fractiles
  bymymeans labor [aw=weight] if CSsample == 1, by(year `het' ranklabor lblabor ublabor) ///
    saving("inequality/mean_labor_by_ranklabor_`suf'`yr'")
}

} // inequality_incomeshares


// Calculate Empirical Density of Log Income
//------------------------------------------------------------------------------------------
	
if $inequality_densities == 1 & inlist(`yr', 2001, 2005, 2010, 2015) {
noi disp "... empirical densities of log earnings"

	// loop over (sub)populations
	foreach het in "" $hetgrouplabor_ineq_densities {
		local suf=subinstr("`het'"," ","",.)
		noi capture empiricaldensity logearn [aw=weight] if CSsample == 1, npoint($kpoints) by(year `het') ///
				saving("inequality/density_logearn_`suf'`yr'")
		if _rc 	noi disp as error "error in empirical density estimation (yr=`yr', suf=`suf')"
	}

} // inequality_densities


//==========================================================================================
//  V O L A T I L I T Y 
//==========================================================================================


// Growth-Rate Percentiles
//------------------------------------------------------------------------------------------

if $volatility_sumstats == 1 & inlist(`yr', $d1yrlistcomma) {

  noi disp as text "... summary stats for growth rate (LX-sample)"

	// loop over growth rate variable
	foreach gvar in $growthvars {

  // loop over (sub)populations
	foreach w in weight hrf {
		if "`w'" == "hrf" local wsuf = "_hrf"
		else local wsuf = ""

  foreach het in "" $hetgrouplabor_vola {
    local suf=subinstr("`het'"," ","",.)
		
    foreach f in `flist' {
      bymydist_detail `gvar'`f'F [aw=`w'], saving("volatility/sumstats`wsuf'_`gvar'`f'F_`suf'`yr'") by(year `het') ///
        percentiles($pctiles_selected) meanvars(topcoded`f'F)  
    } 

  } // het
	} // w
	} // gvar

} // volatility_sumstats


// Growth-Rate Percentiles by Labor Income Bins
//------------------------------------------------------------------------------------------

if $volatility_bycurrent == 1 & inlist(`yr', $d1yrlistcomma) {
noi disp as text "... summary stats for growth rate by labor income bins"

// loop over growth rate variable
foreach gvar in $growthvars {

// loop over (sub)populations
foreach w in weight hrf {
	if "`w'" == "hrf" local wsuf = "_hrf"
	else local wsuf = ""

foreach het in "" $hetgrouplabor_vola {
	local suf=subinstr("`het'"," ","",.)

	// Labor Income Bins
	if "`het'" == "male age" local lbs $lbfine6
	else if "`het'" == "male agegp" local lbs $lbfine5
	else local lbs $lbfine4
	cap drop bin lb ub
	create_bins labor, binvar(bin) lbvar(lb) ubvar(ub) lbs(`lbs')

	// Analysis
	foreach f in `flist' {
		// Conditional distribution of growth rate
		bymydist_detail `gvar'`f'F [aw=`w'], saving("volatility/sumstats`wsuf'_`gvar'`f'F_by_labor_`suf'`yr'") by(year `het' bin lb ub) ///
				percentiles(0.01 0.1 0.5 1/10 15(5)90 91/99 99.5 99.9) meanvars(labor topcoded`f'F)   
		// Distribution of labor income in analysis sample
		bymydist_detail labor if `gvar'`f'F != . [aw=`w'], saving("volatility/sumstats`wsuf'_`gvar'`f'Fsample_labor_`suf'`yr'") by(year `het') ///
				percentiles(1/99 99.9 99.99)
	}

} // het
} // w
} // gvar

} // volatility_bycurrent


// Growth-Rate Percentiles by Permanent Income Rank
//------------------------------------------------------------------------------------------

if $volatility_byperm == 1 & inlist(`yr', $d5yrlist) & inlist(`yrp', $perm3yrlist) {

noi disp as text "... summary stats for growth rate by permanent income rank (H-sample)"


// loop over growth rate variable
foreach gvar in $growthvars {

	cap drop Hsample
	generate Hsample = 1 if `gvar'1F != . & `gvar'5F != .	& permearn1L != .

// loop over (sub)populations
foreach het in "" $hetgrouplabor_vola_byperm {
	local suf=subinstr("`het'"," ","",.)

	// Rankings of permanent income
	//-----------------------------------
	cap drop rank_permearn lb_permearn ub_permearn
	bysort year `het': create_pctclass permearn1L [aw=weight] if Hsample == 1,   ///
				generate(rank_permearn) lbvar(lb_permearn) ubvar(ub_permearn) pctiles(1/10 15(5)90 91/99 99.9) 

	// Summary statistics by het group and permanent income
	//------------------------------------------------------
	foreach f in `flist' {
		bymydist_detail `gvar'`f'F if Hsample == 1 [aw=weight], by(year `het' rank_permearn) ///
				saving("volatility/sumstats_`gvar'`f'F_by_permearn_`suf'`yr'") ///
				percentiles($pctiles_selected) meanvars(permearn1L avg_pastearn lb_permearn ub_permearn)
	} // f

} // het
} // gvar

} // volatility_byperm


// Calculate Empirical Density of one year and five year changes
//------------------------------------------------------------------------------------------
	
if $volatility_densities == 1 & inlist(`yr', 2001, 2005, 2010, 2015) {
noi disp "... empirical densities of earnings growth"

	// loop over (sub)populations
	foreach het in "" $hetgrouplabor_vola_densities {
		local suf=subinstr("`het'"," ","",.)
		foreach f in `flist' {
			capture empiricaldensity researn`f'F [aw=weight], npoint($kpoints) by(year `het') ///
					saving("volatility/density_researn`f'F_`suf'`yr'")
			if _rc 	noi disp as error "error in empirical density estimation (yr=`yr', suf=`suf', f=`f')"
		}
	}

} // volatility_densities
cap drop researn?F


// Autocorrelations of Residual Log Earnings
//------------------------------------------------------------------------------------------

if $volatility_autocorr == 1 {
noi disp "... autocorrelations of residual log earnings"

	// auxiliary variables (components of autocorrelation)
	foreach k of numlist 1/5 {
		cap generate researn_`k'F = . // keep code running for years where `k'F not available
		cap generate labor`k'F = .
		generate researn_0F`k'F = researn * researn_`k'F // y(t) * y(t+k)
		generate researn_`k'F`k'F = researn_`k'F * researn_`k'F // y(t+k)^2
		generate researn_`k'Fsample = researn if !missing(researn_`k'F) // y(t) if y(t+k) available
		generate topcoded_`k'F = labor > 60000 | labor`k'F > 60000 if !missing(researn_0F`k'F)
	} // k

	// Labor Income Bins
	cap drop bin lb ub
	create_bins labor, binvar(bin) lbvar(lb) ubvar(ub) lbs($lbfine6)


	// loop over (sub)populations
	foreach het in "" $hetgrouplabor_vola_autocorr {
		local suf=subinstr("`het'"," ","",.)

		// Conditional distribution of growth rate
		/* noi sum researn_?Fsample researn_?F researn_?F?F topcoded_?F weight year `het' bin lb ub */
		bymymeans researn_?Fsample researn_?F researn_?F?F topcoded_?F [aw=weight], saving("volatility/autocorr_researn_by_labor_`suf'`yr'") by(year `het' bin lb ub) 
		// Distribution of labor income in analysis sample
		foreach k of numlist 1/5 {
			if `yr' <= $yrlast - `k' {
				bymydist_detail labor if researn != . & researn_`k'F != . [aw=weight], saving("volatility/sumstats_researn0F`k'Fsample_labor_`suf'`yr'") by(year `het') ///
						percentiles(1/99 99.9)
			}
		}

	} // het

} // volatility_autocorr
cap drop researn_?Fsample researn_?F researn_?F?F


//==========================================================================================
//  M O B I L I T Y
//==========================================================================================

/* 

  1. compute distribution of permearn by age which gives a mapping from permearn to age-specific TPP-ranks
  2. combine distribution of permearn from TPP and IAB which gives a mapping from permearn to true age-specific ranks
  3. construct bins of permearn
  4. in each bin and age, compute percentiles of permearn5F
  5. map these percentiles into age-specific ranks of permearn5F
  6. compute the average of these ranks

  Alternative: 
    - regress permearn and permearn5F on age dummies to take out age-specific averages
    - repeat the above steps using age groups instead of age
*/


// Transitions between Income Groups
//------------------------------------------------------------------------------------------


if $mobility_transitions == 1 & inlist(`yr', $d1yrlistcomma) {
noi disp as text "... transitions for labor (focus on top)"

  local pcutoffs = "50 90 99 99.9"

	foreach ranktype in "" $rankgrouplabor_mobility_trans {
		local sufrank = cond("`ranktype'" == "", "_by_total", subinstr("_by_`ranktype'", " ", "", .))

		// rank current earnings variable by cohort and ranktype
		bysort year yob `ranktype': create_pctclass labor [aw=weight] if labor >= 2300, ///
								generate(ranklabor) pctiles(`pcutoffs')
		
		// loop over horizons
		foreach f in `flist' {	
			
			// rank future earnings variable by cohort and ranktype
			bysort year yob `ranktype': create_pctclass labor`f'F [aw=weight`f'F] if labor`f'F >= 2300, ///
									generate(ranklabor`f'F) pctiles(`pcutoffs')

			foreach het in "" $hetgrouplabor_mobility_trans {
				local suf=subinstr("`het'"," ","",.)
				// if gender-specific ranking, use only het groups that include gender
				if strpos("`ranktype'", "male") == 0 | strpos("`het'", "male") != 0 {
					preserve
						// collapse
						local meanvars = cond(strpos("`het'", "male") == 0, "male age", "age")
						gcollapse (count) N=labor (rawsum) n_weight=weight (mean) `meanvars' labor labor`f'F [aw=weight] ///
											if !missing(logearn), by(year `het' ranklabor ranklabor`f'F)
						gcollapse (sum) Ntotal=N (sum) ntotal_weight=n_weight, by(year `het' ranklabor) merge		
						order N Ntotal n_weight ntotal_weight, first

						// save
						save "mobility/transitions_ranklabor`f'F`sample'`sufrank'_`suf'`yr'", replace

					restore
				} // if
			} // het
			
		} // f
		
		drop ranklabor*

	} // ranktype
			
} // mobility_transitions



// Future Permanent Income by Permanent Income
//------------------------------------------------------------------------------------------

if $mobility_ranks == 1  & inlist(`yr', $perm3yrlist) {
noi disp as text "... summary stats for future permanent income by current permanent income"

foreach f in 1 5 {

	if (`f' == 1 & inlist(`yr', $d1yrlistcomma)) | (`f' == 5 & inlist(`yr', $d5yrlist)) {

  // rank permanent earnings within gender and age
  foreach v in "" "`f'F" {
    bysort male age: create_pctclass permearnalt`v' [aw=weight] if !missing(permearnalt, permearnalt`f'F), generate(rankpermearnalt`v') ///
                          lbvar(lbpermearnalt`v') ubvar(ubpermearnalt`v') ///
                          pctiles(2.5(2.5)97.5 99 99.9)
  } 
  
  // rank residualized permanent earnings by gender and age group
  foreach v in "" "`f'F" {
    bysort male agegp: create_pctclass respermearnalt`v' [aw=weight] if !missing(respermearnalt, respermearnalt`f'F), ///
													generate(rankrespermearnalt`v') lbvar(lbrespermearnalt`v') ubvar(ubrespermearnalt`v') ///
                          pctiles(2.5(2.5)97.5 99 99.9)
  }
  
 
  // permearnalt and permearnalt`f'F by age
	bymydist_detail permearnalt [aw=weight] if !missing(permearnalt, permearnalt`f'F), ///
      saving("mobility/sumstats_permearnalt_maleage`yr'") by(year male age) ///
			percentiles(1/99 99.9)
	bymydist_detail permearnalt`f'F [aw=weight] if !missing(permearnalt, permearnalt`f'F), ///
      saving("mobility/sumstats_permearnalt`f'F_maleage`yr'") by(year male age) ///
			percentiles(1/99 99.9) 
      
 	// permearnalt`f'F by rankpermearnalt and age
	bymydist_detail permearnalt`f'F [aw=weight] if !missing(permearnalt, permearnalt`f'F), ///
      saving("mobility/sumstats_permearnalt`f'F_by_rankpermearnalt_maleage`yr'") ///
      by(year male age rankpermearnalt) ///
			percentiles(1/10 15(5)90 91/99) meanvars(permearnalt respermearnalt rankrespermearnalt rankpermearnalt`f'F rankrespermearnalt`f'F)
     
  // respermearnalt by age group
	bymydist_detail respermearnalt [aw=weight] if !missing(respermearnalt, respermearnalt`f'F), ///
      saving("mobility/sumstats_`f'Fsample_respermearnalt_maleagegp`yr'") by(year male agegp) ///
			percentiles(1/99 99.9)
	// respermearnalt`f'F by age group
	bymydist_detail respermearnalt`f'F [aw=weight] if !missing(respermearnalt, respermearnalt`f'F), ///
			saving("mobility/sumstats_respermearnalt`f'F_maleagegp`yr'") by(year male agegp) ///
			percentiles(1/99 99.9)
	// respermearnalt`f'F by rankrespermearnalt and age group
	bymydist_detail respermearnalt`f'F [aw=weight] if !missing(respermearnalt, respermearnalt`f'F), ///
			saving("mobility/sumstats_respermearnalt`f'F_by_rankrespermearnalt_maleagegp`yr'") ///
			by(year male agegp rankrespermearnalt) ///
			percentiles(1/10 15(5)90 91/99) meanvars(respermearnalt permearnalt rankrespermearnalt`f'F)


	} // if
	} // f

} // mobility_ranks





} // Loop over years


} // ANALYSIS



//======================================================================================================
// COLLECT DATA ACROSS YEARS
//======================================================================================================

if $COLLECT == 1 {

noi disp as text _n "Collecting data across years"


// Inequality Data
//==========================================================================================

// Sumstats
//-------------------------------------------

noi disp as text "... inequality data: sumstats"
foreach w in weight hrf {
	if "`w'" == "hrf" local wsuf = "_hrf"
	else local wsuf = ""
	
	foreach var in $ineqvars {
		foreach het in "" $hetgrouplabor_ineq_sumstats {
			local suf=subinstr("`het'"," ","",.)
			local years = cond("`var'" == "permearn", "`=$yrfirst+2'/$yrlast", "$yrfirst/$yrlast")
			capture append_and_export , filestub("inequality/sumstats`wsuf'_`var'_`suf'") years(`years') var(`var')
		}
	}
	
}


// Income Shares
//-------------------------------------------

noi disp as text "... inequality data: shares"
foreach het in "" $hetgrouplabor_ineq_shares {
	local suf=subinstr("`het'"," ","",.)
	capture append_and_export , filestub("inequality/mean_labor_by_ranklabor_`suf'") years($yrfirst/$yrlast) var(labor) dataprivacy(data_privacy_mean)
}


// Log Earnings Kernel Densities
//-------------------------------------------------
noi disp as text "... inequality data: densities"
foreach het in "" $hetgrouplabor_ineq_densities {
	local suf=subinstr("`het'"," ","",.)
	local years = "2001 2005 2010 2015"
	capture append_and_export , filestub("inequality/density_logearn_`suf'") years(`years') var(logearn) dataprivacy(data_privacy_density)
}


// Volatility Data: 
//==========================================================================================


// Growth-Rate Percentiles
//-------------------------------------------------
noi disp as text "... volatility data: sumstats"

foreach w in weight hrf {
	if "`w'" == "hrf" local wsuf = "_hrf"
	else local wsuf = ""
	foreach het in "" $hetgrouplabor_vola {
		local suf=subinstr("`het'"," ","",.)
		foreach f in 1 5 {
			if `f' == 1 local years = "$d1yrlist"
			else local years = subinstr("$d5yrlist", ",", " ", .)
			capture append_and_export , filestub("volatility/sumstats`wsuf'_researn`f'F_`suf'") years(`years') var(researn`f'F)
		}
	}
}

// Growth-Rate Percentiles by Current Income Bins
//-------------------------------------------------
noi disp as text "... volatility data: by labor"

foreach gvar in $growthvars {
	foreach w in weight hrf {
		if "`w'" == "hrf" local wsuf = "_hrf"
		else local wsuf = ""
		foreach het in "" $hetgrouplabor_vola {
			local suf=subinstr("`het'"," ","",.)
			foreach f in 1 5 {
				if `f' == 1 local years = "$d1yrlist"
				else local years = subinstr("$d5yrlist", ",", " ", .)
				capture append_and_export , filestub("volatility/sumstats`wsuf'_`gvar'`f'F_by_labor_`suf'") years(`years') var(`gvar'`f'F)
				capture append_and_export , filestub("volatility/sumstats`wsuf'_`gvar'`f'Fsample_labor_`suf'") years(`years') var(labor)
			} // f
		} // het
	} // w
} // gvar


// Growth-Rate Percentiles by Permanent Income Bins
//-------------------------------------------------
noi disp as text "... volatility data: by permearn"

foreach gvar in $growthvars {
foreach f in 1 5 {
  foreach het in "" $hetgrouplabor_vola_byperm {
    local suf=subinstr("`het'"," ","",.)
    append_and_export , filestub("volatility/sumstats_`gvar'`f'F_by_permearn_`suf'") years($years_Hsample) var(`gvar'`f'F)
  }
}
}


// Growth-Rate Kernel Densities
//-------------------------------------------------

foreach f in 1 5 {
  if `f' == 1 local years 2001 2005 2010 2015 
  else        local years 2001 2005 2010
  foreach het in "" $hetgrouplabor_vola_densities {
    local suf=subinstr("`het'"," ","",.)
    capture append_and_export , filestub("volatility/density_researn`f'F_`suf'") years(`years') var(researn`f'F) dataprivacy(data_privacy_density)
  }
}

// Autocorrelations
//-------------------------------------------------

foreach het in "" $hetgrouplabor_vola_autocorr {
	local suf=subinstr("`het'"," ","",.)
	capture append_and_export , filestub("volatility/autocorr_researn_by_labor_`suf'") years($yrfirst/$yrlast) dataprivacy(data_privacy_mean)
	foreach k of numlist 1/5 {
		capture append_and_export , filestub("volatility/sumstats_researn0F`k'Fsample_labor_`suf'") years($yrfirst/`=$yrlast-`k'') var(labor) dataprivacy(data_privacy_sumstats)
	}
}


// Mobility Data
//==========================================================================================


// Transitions of labor income
//-------------------------------------------------
noi disp as text "... mobility data: transitions"

foreach ranktype in "" $rankgrouplabor_mobility_trans {
	local sufrank = cond("`ranktype'" == "", "_by_total", subinstr("_by_`ranktype'", " ", "", .))

	foreach het in "" $hetgrouplabor_mobility_trans {
	local suf=subinstr("`het'"," ","",.)
		foreach f in 1 5 {
			if `f' == 1 local years $d1yrlist 
			else local years = subinstr("$d5yrlist", ",", " ", .)

			local filestub = "mobility/transitions_ranklabor`f'F`sufrank'_`suf'"
			
			capture {
				clear
				foreach yr of numlist `years' {
					append using "`filestub'`yr'"
				}
				// DATA PRIVACY
				if $protect == 1 {
					foreach var in male age labor labor`f'F n_weight N {
						replace `var' = . if N < 30
					}
					foreach var in ntotal_weight Ntotal {
						replace `var' = . if Ntotal < 30
					}
				}
				
				if substr("`filestub'", -1, .) == "_" local outstub = substr("`filestub'", 1, `= strlen("`filestub'") -1')
				else local outstub `filestub'

				export delimited using "`outstub'.csv", delimiter("$csvdelim") replace
				noi disp "save to CSV: $outfolder/`outstub'.csv"
			} // capture
		}
	}
}


// Average future perm. rank by current perm. rank
//-----------------------------------------------------
noi disp as text "... mobility data: ranks"

foreach var in respermearnalt permearnalt {
foreach f in 1 5 {

  if "`var'" == "respermearnalt" local suf = "maleagegp"
  else local suf = "maleage"

	local years = cond(`f' == 1, "2003/2015", "2003/2011")

	capture append_and_export , filestub("mobility/sumstats_`f'Fsample_`var'_`suf'") years(`years') var(`var')
	capture append_and_export , filestub("mobility/sumstats_`var'`f'F_`suf'") years(`years') var(`var'`f'F)
	capture append_and_export , filestub("mobility/sumstats_`var'`f'F_by_rank`var'_`suf'") years(`years') var(`var'`f'F)
  
} // f
} // var

  
	
} // COLLECT

	
//------------------------------------------------------------------------------
} // quietly
//------------------------------------------------------------------------------


// END
//==========================================================================================

timer off 1
timer list 1

cap log close
