//==========================================================================================
// This program generates the base sample
// Moritz Drechsel-Grau & Kai Schmid (based on code by Halvorsen, Ozkan, Salgado)
//==========================================================================================


/*==========================================================================================

This do file renames the variables, does very basic sample selection, creates new variables
	(e.g., residual income), and generates the master_sample.dta, which is a wide
	form dataset which will be used in the rest of the do files.

The main output of this do file (master_sample.dta) is saved in the folder ${dta_folder}
	and contains the following variables:
	(a) personid: id of the individual used throughout the do files
	(b) male: indicator variable equal to 1 if male and 0 if female
	(c) yob: year of birth of the individual
	(d) income: real income in levels
	(e) loginc: real income in log levels
	(f) perminc: permanent income defined as avg. income from t-3 to t-1
	(g) permincalt: alternative measure of permanent income
	(h) resinc: residual of log earnings (from regressions on age dummies)
	(i) resinc1F: 1-year forward residualized log earnings change
	(j) resinc5F: 5-year forward residualized log earnings change


==========================================================================================*/


// Start
// -----------------------------------------------------------------------------------------

// Start log file
local timestamp: disp %tcCCYY-NN-DD,_HH-MM tc(`c(current_date)' `c(current_time)') // example (2021-02-14, 14:54)
*capture log close
capture noisily log using "${log_folder}/${date} 401_Gen_Base_Sample (`timestamp').log", replace

// Clear workspace
clear all
set more off

// Initialize Settings
qui do "${do_folder}/001_Initialize.do"

// CD to data folder
cap mkdir "${dta_folder}/${income_var}"
cd "${dta_folder}/${income_var}"


quietly {

noi disp as text _n "START ..."


// Load Data and Reshape
//==================================================================================================

noi disp as text "... load data and reshape to wide format"

// Reshape data in long format to wide. Faster than RESHAPE command.


// Load data
use $weight_var $personid_var $male_var $yob_var $year_var $income_var ${CS_var} ${CSc_var} ${stratvars_totinc} ///
		if ${year_var} >= ${yrfirst} & ${year_var} <= ${yrlast} using "${basedata}", clear

// Rename variables
rename $personid_var personid
rename $year_var year
rename $male_var male
rename $yob_var yob
rename $income_var income
rename $weight_var weight
rename $CS_var CS
rename $CSc_var CSc

noi describe
noi tab year

// Extract annual cross sections
sort personid year
forvalues yr = $yrfirst/$yrlast{
	preserve
	keep if year ==`yr'
	
	keep personid male yob weight income CS CSc ${stratvars_totinc}
		// Need to keep all variables. If only keep for first year, we
		// miss the data for observations that enter in the sample after
		// the first year. 
		
	foreach var in weight income CS CSc ${stratvars_totinc} {
		rename `var' `var'`yr'
	}
	label var income`yr' "Real $income_var in `yr'"
	label var weight`yr' "Sample weight in `yr'"

	sort personid
	save "temp`yr'.dta",replace
	restore
}

// Merge cross sections
local first = $yrfirst + 1
use "temp$yrfirst.dta", clear
erase "temp${yrfirst}.dta"
forvalues yr = `first'/$yrlast{
	merge 1:1 personid using "temp`yr'.dta", nogen update replace
			// update replace makes sure the variables such as year of birth
			// are replaced by non-missing values for observations that enter
			// in the sample after the first year.
	erase "temp`yr'.dta"
	sort personid
}

order personid male yob weight* income* 


// Create Base Sample
//==================================================================================================

noi disp as text "... create base sample --> base_sample.dta"

// Drop anybody who is too old or too young or too dead.
// Criteria 1 (Age) in CS Sample
drop if yob == .
drop if $yrfirst - yob + 1 > $end_age | $yrlast - yob + 1 < $begin_age
noi describe

global base_price_index = ${base_price}-${yrfirst} + 1	// The base year nominal values are converted to real.


// Loop over years
forvalues yr = $yrfirst/$yrlast {

	// Covert to real values
	// Criteria d (Inflation) in CS Sample

	local cpi_index = `yr'-${yrfirst} + 1
	replace income`yr' = income`yr' / cpimat[`cpi_index',1]		//Coverting to real values

	// Winsorization
	gen temp = income`yr' if `yr' - yob + 1>= $begin_age & `yr'- yob+ 1<= $end_age 
	_pctile temp [aw=weight`yr'], p($winsor)
	replace income`yr'= r(r1) if income`yr'>=r(r1) & income`yr'!=.
	drop temp

	// Add a small noise
	gen temp = ${noise}*(uniform()-0.5)
	replace income`yr' = income`yr' + income`yr'*temp
	drop temp

	if(${miss_earn}==0){
	// Any incomes that are missing inside of $begin_age and $end_age are set to zero.
	replace income`yr'= 0 if income`yr'== . &  ///
	   `yr' - yob + 1 >= $begin_age & `yr'- yob+ 1<=$end_age
	}

	// Assign missing if outside of $begin_age and $end_age
	replace income`yr'= . if `yr' - yob + 1 < $begin_age | `yr'- yob+ 1>$end_age

}

// Base sample creation completed.
order personid male yob weight* income*
compress
save "base_sample.dta", replace


// Create residuals of log-income and age profiles (for each year separately, by gender)
//==================================================================================================
noi disp as text "... create age profiles and residuals of positive log-income (by year and gender) --> resinc.dta, age_dums.dta"


// Loop over years
forvalues yr = $yrfirst/$yrlast {

	// Load data for year yr
	use personid male yob weight`yr' income`yr' ///
			using "base_sample.dta" if income`yr' != . , clear

	// Create year
	gen year=`yr'

	// Create age
	gen age = `yr' - yob + 1
	drop if age<${begin_age} | age>${end_age}

	// Create log income if income above the min treshold
	// Criteria c (Trimming) in CS Sample
	// Notice we do not drop the observations but log-income are generated for those with
	// income below 1/3*min threshold. Variable loginc`yr'c is used for growth rates conditional
	// on permanent income only

	gen loginc`yr' = log(income`yr') if income`yr'>=rmininc[`yr'-${yrfirst} + 1,1] & income`yr'!=.
	gen logincc`yr' = log(income`yr') if income`yr'>=(1/3)*rmininc[`yr'-${yrfirst} + 1,1] & income`yr'!=.
	drop if logincc`yr' == . & loginc`yr' == .
	label var loginc`yr' "Real log income of year `yr' above min threshold"
	label var logincc`yr' "Real log income of year `yr' above 1/3*min threshold"

	// Create dummies for age groups
	tab age, gen(agedum)
	drop agedum1

	// Regressions for residual income (by gender)
	//--------------------------------------------------------------------------
	// Save age dummies
	statsby _b N=e(N),  by(year) saving(age_yr`yr'_m,replace):  ///
	regress loginc`yr' agedum* [aw=weight`yr'] if male==1

	predict temp_m if e(sample)==1, resid

	statsby _b N=e(N),  by(year) saving(age_yr`yr'_f,replace):  ///
	regress loginc`yr' agedum* [aw=weight`yr'] if male==0

	predict temp_f if e(sample)==1, resid

	// Regressions for residual income with income above 1/3*minincome (by gender)
	regress logincc`yr' agedum* [aw=weight`yr'] if male==1
	predict temp_m_c if e(sample)==1, resid

	regress logincc`yr' agedum* [aw=weight`yr'] if male==0
	predict temp_f_c if e(sample)==1, resid

	// Generate the residuals by year and save a dataset for later append.
	gen resinc`yr'= temp_m
	replace resinc`yr'= temp_f if male==0
	label var resinc`yr' "Residual of real log income of year `yr'"

	gen resincc`yr'= temp_m_c
	replace resincc`yr'= temp_f_c if male==0
	label var resincc`yr' "Residual of real log income of year `yr' above 1/3*min threshold"

	// Drop temporary variables
	drop temp_*

	// Save data set for later append
	preserve
		keep personid male yob weight`yr' resinc`yr' resincc`yr' loginc`yr' logincc`yr' income`yr' 
		sort personid
		save "resinc`yr'.dta", replace
	restore
	
}

// Appending residual of real log-income
forvalues yr = $yrfirst/$yrlast{
	if (`yr' == $yrfirst){
		use "resinc`yr'.dta", clear
		erase "resinc`yr'.dta"
	}
	else{
		merge 1:1 personid using "resinc`yr'.dta", nogen
		erase "resinc`yr'.dta"
	}
	sort personid
}
save "resinc.dta", replace



// Calculate growth of (residual) log income (Section 2.e and 2.f)
//==================================================================================================
noi disp as text "... calculate log income growth rates (k = 1,5) --> resinc1F.dta, resinc5F.dta"

clear
foreach k in 1 5 {

	// Given the jump k, calculate the growth rate for each worker in each year

	local lastyr=$yrlast-`k'
	forvalues yr = $yrfirst/`lastyr'{

		local yrnext=`yr'+`k'

		use personid male yob weight`yr' resinc`yr' resincc`yrnext' using "resinc.dta", clear

		gen age = `yr' - yob + 1

		gen resinc`k'F`yr'= resincc`yrnext'-resinc`yr'		// Growth with income above mininc in t and 1/3*mininc in t+k
		label var resinc`k'F`yr'  "Residual income growth between `yrnext' and `yr'"

		keep personid weight`yr' resinc`k'F`yr' 
		save "resinc`k'F`yr'.dta", replace

	}

	// Collect annual data
	forvalues yr = $yrfirst/`lastyr'{

		if (`yr' == $yrfirst){
			use "resinc`k'F`yr'.dta", clear
			erase "resinc`k'F`yr'.dta"
		}
		else{
			merge 1:1 personid using "resinc`k'F`yr'.dta", nogen
			erase "resinc`k'F`yr'.dta"
		}
		sort personid
	}

	compress
	save "resinc`k'F`rvsuffix'.dta", replace

}


// Calculate log permanent income
//==================================================================================================
noi disp as text "... calculate permanent income with logs --> perminc.dta"

clear
local firstyr=$yrfirst +2
forvalues yr = `firstyr'/$yrlast{
	local yrL1=`yr'-1
	local yrL2=`yr'-2

	use personid male yob weight`yr' income`yrL2' income`yrL1' income`yr' using ///
	"base_sample`rvsuffix'.dta" if income`yr' != . , clear

	// Create year
	gen year=`yr'

	// Create age
	gen age = `yr' - yob + 1
	drop if age<${begin_age} | age>${end_age}

	// Create average income for those with at least 2 years of income above
	// the treshold income between t-1 and t-3
	gen totinc=0
	gen numobs=0

	*replace numobs = -5 if income`yr' < rmininc[`yr'-${yrfirst} + 1,1]
	// This ensures that permanent income is only constructed for those
	// with income above the threshold in t-1


	forvalues yrp=`yrL2'/`yr'{
		replace totinc=totinc+income`yrp' if income`yrp' != .
		replace numobs=numobs+ 1 if income`yrp'>=rmininc[`yrp'-${yrfirst} + 1,1] & income`yrp' != .
		// Notice earnings below the min threshold are still used to get totinc
	}

	replace totinc=totinc/numobs if numobs>=2			// Average income
	drop	if numobs<2									// Drop if less than 2 obs

	// Create log of totinc
	generate log_totinc = log(totinc)
	drop if log_totinc==.

	// Gen dummies for regressions
	tab age, gen(agedum)
	drop agedum1

	// Regression to get residuals permanent income (add mean to residuals)
	regress log_totinc agedum* [aw=weight`yr'] if male==1
	predict temp_m if e(sample)==1, resid
	qui sum log_totinc [aw=weight`yr'] if male==1 & e(sample)==1
	replace temp_m = temp_m + r(mean)

	qui regress log_totinc agedum* [aw=weight`yr'] if male==0
	predict temp_f if e(sample)==1, resid
	qui sum log_totinc [aw=weight`yr'] if male==0 & e(sample)==1
	replace temp_f = temp_f + r(mean)

	gen perminc`yr'= temp_m
	replace perminc`yr'= temp_f if male==0

	label var perminc`yr' "Residual log permanent income between `yr' and `yrL2'"
	
	// Save average past earnings (residualized)
	generate level_perminc`yr' = exp(perminc`yr')
	label var level_perminc`yr' "Level of residual log permanent income between `yr' and `yrL2'"
	
	// Save average past earnings
	rename totinc avg_pastinc`yr'
	label var avg_pastinc`yr' "Average income between `yr' and `yrL2' when using perminc"
		

	// Save
	keep personid weight`yr' perminc`yr' level_perminc`yr' avg_pastinc`yr'
	compress
	sort personid
	save "perminc`yr'.dta", replace

}

// Collect annual data
clear
local firstyr=$yrfirst +2
forvalues yr = `firstyr'/$yrlast{

	if (`yr' == `firstyr'){
		use "perminc`yr'.dta", clear
		erase "perminc`yr'.dta"
	}
	else{
		merge 1:1 personid using "perminc`yr'.dta", nogen
		erase "perminc`yr'.dta"
	}
	sort personid
}
save "perminc`rvsuffix'.dta", replace



// Calculate modified permanent income
//==================================================================================================
noi disp as text "... calculate modified permanent income (all individuals) --> permincalt.dta"
/*
   Relative to the previos version, here we consider all individuals
   even thouse with low earnimgs. See section "Key Statisitcs 4: Mobilitity"
*/
clear
local firstyr=$yrfirst +2
forvalues yr = `firstyr'/$yrlast{
	local yrL1=`yr'-1
	local yrL2=`yr'-2

	use personid male yob weight`yr' income`yrL2' income`yrL1' income`yr' ///
				CS`yrL2' CS`yrL1' CS`yr'  using ///
	"base_sample.dta" if income`yr' != . , clear

	// Create year
	gen year=`yr'

	// Create age
	gen age = `yr' - yob + 1
	drop if age<${begin_age} + 2 | age>${end_age}			// This makes the min age 26 and max age 54

	// Create average income for those with at least 2 years of income
	gen sum_income=0
	gen numobs=0
	gen numobspos=0

	forvalues yrp=`yrL2'/`yr'{
		replace sum_income=sum_income+income`yrp' if income`yrp' != .
		replace numobs=numobs+ 1 if income`yrp' != .
		replace numobspos=numobspos+ 1 if CS`yrp' == 1
			// Notice income below the min threshold are still used to get sum_income
			// This ensure we do not consider income of individuals when they were 24 yrs old or less
	}
	replace sum_income=sum_income/numobs if numobs==3			// Average income
	drop if numobs<3			// Drop if less than 2 obs
	drop if numobspos < 1 		// Drop if less than 1 obs above min income
	drop if sum_income==.
	gen permincalt`yr' = sum_income
	label var permincalt`yr' "Altenative permanent income between `yr' and `yrL2'"

	// Save
	keep personid weight`yr' permincalt`yr'


	compress
	sort personid
	save "permincalt`yr'.dta", replace

}

// Collect annual data
clear
local firstyr=$yrfirst + 2
forvalues yr = `firstyr'/$yrlast{

	if (`yr' == `firstyr'){
		use "permincalt`yr'.dta", clear
		erase "permincalt`yr'.dta"
	}
	else{
		merge 1:1 personid using "permincalt`yr'.dta", nogen
		erase "permincalt`yr'.dta"
	}
	sort personid
}
save "permincalt.dta", replace


*/

// Merge all data sets to a master file
//==================================================================================================
noi disp as text "... merge all data sets to a master file --> master_sample.dta"

use "base_sample.dta", clear

merge 1:1 personid using "resinc.dta", nogen
merge 1:1 personid using "resinc1F.dta", nogen
merge 1:1 personid using "resinc5F.dta", nogen
merge 1:1 personid using "perminc.dta", nogen
merge 1:1 personid using "permincalt.dta", nogen

// drop unnecessary variables
cap drop logincc* 
cap drop resincc* 

compress
order  personid male yob income* loginc* perminc* resinc* avg_pastinc* level_perminc*
save "master_sample.dta", replace


erase "resinc.dta"
erase "resinc1F.dta"
erase "resinc5F.dta"
erase "perminc.dta"
erase "permincalt.dta"
erase "base_sample.dta"




} // END of quietly

noi disp as text "... END"

capture log close



// END OF DO-FILE
//////////////////////////////////////////////
