//==========================================================================================
// This program generates the descriptive statistics
// Moritz Drechsel-Grau
//==========================================================================================

/*==========================================================================================

	Dieses do-file erstellt deskriptive Statistiken zur Variable Gesamteinkommen (income).
  
==========================================================================================*/

/* 2022-03-16: use lead-weights to compute percentiles/ranks of leads. */


// Arguments
args arg_analysisyears arg_analysis arg_collect


// Clear
clear all
set more off

// Create folder for output and log-file
global outfolder="$date 406_Distributions ($income_var)"
capture noisily mkdir "${out_folder}/$outfolder"
capture mkdir "${out_folder}/$outfolder/inequality"
capture mkdir "${out_folder}/$outfolder/volatility"
capture mkdir "${out_folder}/$outfolder/mobility"

// Log file
local timestamp: disp %tcCCYY-NN-DD,_HH-MM tc(`c(current_date)' `c(current_time)') // example (2021-02-14, 14:54)
capture log close
capture noisily log using "${log_folder}/$outfolder (`timestamp').log", replace

// Initialize Settings
qui do "${do_folder}/001_Initialize.do"

// Cd to the output file, create the program for moments, and load base sample.
cd "${out_folder}/$outfolder"
qui do "${do_folder}/myprogs.do"
qui do "${do_folder}/data_privacy_restrictions.do"

// Growth rate variables
global growthvars resinc //arcinc

// Inequality variables
global ineqvars loginc income resinc //perminc

// Analysis
if "`arg_analysis'" == ""   global ANALYSIS = 1
else                        global ANALYSIS = "`arg_analysis'"
disp "arg_analysis = `arg_analysis', ANALYSIS = $ANALYSIS"

// Collect
if "`arg_collect'" == ""  global COLLECT = 1
else                      global COLLECT = "`arg_collect'"
disp "arg_collect = `arg_collect', COLLECT = $COLLECT"

// Analysis Years
if "`arg_analysisyears'" == ""  global ANALYSISYEARS = "$yrfirst/$yrlast"
else                            global ANALYSISYEARS = "`arg_analysisyears'"
disp "arg_analysisyears = `arg_analysisyears', ANALYSISYEARS = $ANALYSISYEARS"

// Defines the number of points in the Kernel Density Estimator
global kpoints =  400

local lastyear = ""
foreach yr of numlist $ANALYSISYEARS {
  local lastyear = `yr'
}


// Sections
global inequality_sumstats      = 1
global inequality_incomeshares  = 1
global inequality_densities     = 1
global volatility_sumstats      = 1
global volatility_byperm        = 1
global volatility_densities     = 1
global mobility_transitions     = 1
global mobility_ranks           = 1


timer clear 1
timer on 1


quietly {


if $ANALYSIS == 1 {

//==========================================================================================
//  D A T A
//==========================================================================================


foreach yr of numlist $ANALYSISYEARS { 

noi disp as text _n "Working in year `yr'"


local yr5F = `yr'+5   // t+5
local yr1F = `yr'+1   // t+1
local yrp  = `yr'-1		// t-1

// Variables to load
local vars = "personid male yob weight`yr' income`yr' loginc`yr' resinc`yr' CS`yr'"
local vars = "`vars' mainsrc2`yr' mainsrc4`yr'"
local flist = ""

if inlist(`yr',${d1yrlistcomma}) {
	local vars = "`vars' loginc`yr1F' income`yr1F' resinc1F`yr' CS`yr1F' weight`yr1F'"
	local flist "`flist' 1"
}

if inlist(`yr',${d5yrlist}) {
	local vars = "`vars' loginc`yr5F' income`yr5F' resinc5F`yr' CS`yr5F' weight`yr5F'"
	local flist "`flist' 5"
}

if inlist(`yrp',${perm3yrlist}) {
	local vars = "`vars' perminc`yrp' avg_pastinc`yrp'"
}

if inlist(`yr', ${perm3yrlist}) & inlist(`yr', ${d5yrlist}) {
	local vars = "`vars' permincalt`yr' permincalt`yr5F'"
}


// Load base data
noi disp as text "... load and prepare data"
use `vars' using "${dta_folder}/${income_var}/master_sample.dta" if income`yr' != . , clear


// Draw sample for testing only
if $debug == 1 {
	if "$user" == "kdfv" sample 0.0001
  else sample 0.1
}


// Rename variables	
rename weight`yr' weight
cap rename weight`yr1F' weight1F
cap rename weight`yr5F' weight5F

rename income`yr' income
rename loginc`yr' loginc
rename resinc`yr' resinc

cap rename loginc`yr1F' loginc1F
cap rename loginc`yr5F' loginc5F
cap rename income`yr1F' income1F
cap rename income`yr5F' income5F

cap rename CS`yr' CS
cap rename CS`yr1F' CS1F
cap rename CS`yr5F' CS5F

cap rename resinc1F`yr' resinc1F	
cap rename resinc5F`yr' resinc5F	
cap rename perminc`yrp' perminc
cap rename avg_pastinc`yrp' avg_pastinc

cap rename permincalt`yr' permincalt
cap rename permincalt`yr5F' permincalt5F

cap rename mainsrc2`yr' mainsrc2
cap rename mainsrc3`yr' mainsrc3
cap rename mainsrc4`yr' mainsrc4

// Age group
gen age = `yr'-yob+1
gen agegp = .
replace agegp = 1 if age <= 34 & agegp == .
replace agegp = 2 if age <= 44 & agegp == .
replace agegp = 3 if age <= 55 & agegp == .

// Age restriction
drop if $yrfirst - yob + 1 > $end_age | $yrlast - yob + 1 < $begin_age

// Generate year
gen year = `yr'

// CS indicator with threshold
generate CSsample = loginc != .
cap generate CSsample_5F = loginc5F != .



//==========================================================================================
//  I N E Q U A L I T Y 
//==========================================================================================


// Cross-Sectional Distribution (Percentiles)
//------------------------------------------------------------------------------------------

if $inequality_sumstats == 1 {
noi disp as text "... summary statistics for income (CS-sample)"

foreach var in $ineqvars {

  // loop over (sub)populations
  foreach het in "" $hetgrouptotinc_ineq_sumstats {
    local suf=subinstr("`het'"," ","",.)
    
    local ginitheil = cond("`var'" == "labor", "ginitheil", "")
    local pctiles = cond("`var'" == "labor", "50", "0.1(0.1)1 1.5(0.5)5 6(1)95 95.5(0.5)99 99.1(0.1)99.9 99.95 99.99 99.995 99.999")

    bymydist_detail `var' if CSsample == 1 [aw=weight], saving("inequality/sumstats_`var'_`suf'`yr'") by(year `het') ///
      percentiles(`pctiles') `ginitheil'
    noi disp as res "    --> inequality/sumstats_`var'_`suf'.dta"
  }
} // var

} // inequality_sumstats


// Income Shares
//------------------------------------------------------------------------------------------

if $inequality_incomeshares == 1 {
noi disp as text "... income shares"

// loop over (sub)populations
local prevrankvars = "9999"
foreach het in "" $hetgrouptotinc_ineq_shares {
	local suf=subinstr("`het'"," ","",.)

	// create fractiles (do not rank within income source)
  local rankvars = "`het'"
  foreach source in mainsrc2 mainsrc4 {
    local rankvars = subinstr("`rankvars'", "`source'", "", .)
  }
  // check if ranking already exists
  if "`rankvars'" != "`prevrankvars'" {
	  cap drop rankincome lbincome ubincome
    bysort year `rankvars': create_pctclass income [aw=weight] if CSsample == 1, ///
            generate(rankincome) lbvar(lbincome) ubvar(ubincome) pctiles(10(10)90 95 99 99.9 99.99 99.999)
  }
	// average income within fractiles
  bymymeans income if CSsample == 1 [aw=weight], 	by(year `het' rankincome lbincome ubincome) ///
																saving("inequality/mean_income_by_rankincome_`suf'`yr'")
  noi disp as res "    --> inequality/mean_income_by_rankincome_`suf'.dta (ranks by [year `rankvars'])"
}

cap drop rankincome lbincome ubincome

} // inequality_incomeshares


// Calculate Empirical Density of Log Income
//------------------------------------------------------------------------------------------
	
if $inequality_densities == 1 & inlist(`yr', 2001, 2005, 2010, 2015) {
noi disp "... empirical densities of log income"

	// loop over (sub)populations
	foreach het in "" $hetgrouptotinc_ineq_densities {
		local suf=subinstr("`het'"," ","",.)
		noi capture empiricaldensity loginc [aw=weight] if CSsample == 1, npoint($kpoints) by(year `het') ///
				saving("inequality/density_loginc_`suf'`yr'")
		if _rc 	noi disp as error "error in empirical density estimation (yr=`yr', suf=`suf')"
	}

} // inequality_densities


//==========================================================================================
//  V O L A T I L I T Y 
//==========================================================================================


// Growth-Rate Percentiles
//------------------------------------------------------------------------------------------

if $volatility_sumstats == 1 & inlist(`yr', $d1yrlistcomma) {

  noi disp as text "... summary stats for growth rate (LX-sample)"

	// loop over growth rate variable
	foreach gvar in $growthvars {

  // loop over (sub)populations
  foreach het in "" $hetgrouptotinc_vola {
    local suf=subinstr("`het'"," ","",.)
		
    foreach f in `flist' {
      bymydist_detail `gvar'`f'F [aw=weight], saving("volatility/sumstats_`gvar'`f'F_`suf'`yr'") by(year `het') ///
        percentiles($pctiles_selected)
    } 

  } // het
	} // gvar

} // volatility_sumstats


// Growth-Rate Percentiles by Permanent Income Rank
//------------------------------------------------------------------------------------------

if $volatility_byperm == 1 & inlist(`yr', $d5yrlist) & inlist(`yrp', $perm3yrlist) {

noi disp as text "... summary stats for growth rate by permanent income rank (H-sample)"


// loop over growth rate variable
foreach gvar in $growthvars {

	cap drop Hsample
	generate Hsample = 1 if `gvar'1F != . & `gvar'5F != .	& perminc != .

  // loop over (sub)populations
  local prevrankvars = "9999"
  foreach het in "" $hetgrouptotinc_vola {
    local suf=subinstr("`het'"," ","",.)

    // Rankings of permanent income
    //-----------------------------------
    // do not rank within income source
    local rankvars = "`het'"
    foreach source in mainsrc2 mainsrc4 {
      local rankvars = subinstr("`rankvars'", "`source'", "", .)
    }
    // check if ranking already exists
    if "`rankvars'" != "`prevrankvars'" {
      cap drop rank_perminc lb_perminc ub_perminc
      bysort year `rankvars': create_pctclass perminc [aw=weight] if Hsample == 1,   ///
              generate(rank_perminc) lbvar(lb_perminc) ubvar(ub_perminc) pctiles(2.5 5(5)90 91/99 99.9) 
    }
    local prevrankvars = "`rankvars'"

	// Summary statistics by het group and permanent income
	//------------------------------------------------------
	foreach f in `flist' {
		bymydist_detail `gvar'`f'F if Hsample == 1 [aw=weight], by(year `het' rank_perminc) ///
				saving("volatility/sumstats_`gvar'`f'F_by_perminc_`suf'`yr'") ///
				percentiles($pctiles_selected) meanvars(perminc avg_pastinc lb_perminc ub_perminc)
	} // f

} // het
} // gvar

} // volatility_byperm



// Calculate Empirical Density of one year and five year changes
//------------------------------------------------------------------------------------------
	
if $volatility_densities == 1 & inlist(`yr', 2001, 2005, 2010, 2015) {
noi disp "... empirical densities of income growth"

	// loop over (sub)populations
	foreach het in "" $hetgrouptotinc_vola_densities {
		local suf=subinstr("`het'"," ","",.)
		foreach f in `flist' {
			capture empiricaldensity resinc`f'F [aw=weight], npoint($kpoints) by(year `het') ///
					saving("volatility/density_resinc`f'F_`suf'`yr'")
			if _rc 	noi disp as error "error in empirical density estimation (yr=`yr', suf=`suf', f=`f')"
		}
	}

} // volatility_densities


//==========================================================================================
//  M O B I L I T Y
//==========================================================================================


// Transitions between Income Groups
//------------------------------------------------------------------------------------------

if $mobility_transitions == 1 & inlist(`yr', $d1yrlistcomma) {
noi disp as text "... transitions for income (focus on top)"

  local pcutoffs = "50 90 99 99.9"

  // with or without minimum income threshold
  foreach sample in "" "_nothreshold" {

    local rankvar = cond("`sample'" == "_nothreshold", "income", "loginc")

    foreach ranktype in "" $rankgrouptotinc_mobility_trans {
      local sufrank = cond("`ranktype'" == "", "_by_total", subinstr("_by_`ranktype'", " ", "", .))
      if "`sample'" == "_nothreshold" | inlist("`ranktype'", "", "male") {
        // rank current income variable by cohort and ranktype
        bysort year yob `ranktype': create_pctclass income [aw=weight] if CS == 1 & `rankvar' != . , ///
                    generate(rankincome) pctiles(`pcutoffs')
        
        // loop over horizons
        foreach f in `flist' {	
          
          // rank future income variable by cohort and ranktype
          bysort year yob `ranktype': create_pctclass income`f'F [aw=weight`f'F] if CS`f'F == 1 & `rankvar'`f'F !=. , ///
                      generate(rankincome`f'F) pctiles(`pcutoffs')

          foreach het in "" $hetgrouptotinc_mobility_trans {
            local suf=subinstr("`het'"," ","",.)
            // if gender-specific ranking, use only het groups that include gender
            if strpos("`ranktype'", "male") == 0 | strpos("`het'", "male") != 0 {
              preserve
                // collapse
                local meanvars = cond(strpos("`het'", "male") == 0, "male age", "age")
                gcollapse (count) N=income (rawsum) n_weight=weight (mean) `meanvars' income income`f'F [aw=weight] ///
                          if !missing(`rankvar'), by(year `het' rankincome rankincome`f'F)
                gcollapse (sum) Ntotal=N (sum) ntotal_weight=n_weight, by(year `het' rankincome) merge		
                order N Ntotal n_weight ntotal_weight, first

                // save
                save "mobility/transitions_rankincome`f'F`sample'`sufrank'_`suf'`yr'", replace

              restore
            } // if
          } // het
          
        } // f
        
        drop rankincome*

      } // if

    } // ranktype

  } // sample
        
} // mobility_transitions


// Future Permanent Income by Permanent Income
//------------------------------------------------------------------------------------------

if $mobility_ranks == 1 & inlist(`yr', $d5yrlist) & inlist(`yr', $perm3yrlist) {
  noi disp as text "... summary stats for future permanent income by current permanent income"

  cap drop sample 
  generate sample = 1 if !missing(permincalt, permincalt5F)

  // rank permanent income within gender and age
  foreach v in "" "5F" {
    bysort year male age: create_pctclass permincalt`v' [aw=weight] if sample == 1, ///
                          generate(rankpermincalt`v') lbvar(lbpermincalt`v') ubvar(ubpermincalt`v') ///
                          pctiles(1 5(5)95 99 99.9)
  }
          
 	// permincalt5F by rankpermincalt
  foreach het in "" $hetgrouptotinc_mobility {
    local suf=subinstr("`het'"," ","",.)
    bymydist_detail rankpermincalt5F if sample == 1 [aw=weight], ///
        saving("mobility/sumstats_rankpermincalt5F_by_rankpermincalt_`suf'`yr'") ///
        by(year `het' rankpermincalt) ///
        percentiles(1 10 25 50 75 90 99) meanvars(permincalt lbpermincalt ubpermincalt)
  }

} // mobility_ranks


} // Loop over years


} // ANALYSIS



//======================================================================================================
// COLLECT DATA ACROSS YEARS
//======================================================================================================

if $COLLECT == 1 {

noi disp as text _n "Collecting data across years"


// Inequality Data
//==========================================================================================

// Sumstats
//-------------------------------------------

noi disp as text "... inequality data: sumstats"

foreach var in $ineqvars {
  foreach het in "" $hetgrouptotinc_ineq_sumstats {
    local suf=subinstr("`het'"," ","",.)
    if "`var'" == "income" | "`het'" != "male age" {
    capture append_and_export , filestub("inequality/sumstats_`var'_`suf'") years($yrfirst/$yrlast) var(`var') dataprivacy(data_privacy_sumstats)
    }
  }
}

// Income Shares
//-------------------------------------------

// mean income by income fractiles
foreach het in "" $hetgrouptotinc_ineq_shares {
	local suf=subinstr("`het'"," ","",.)
	capture append_and_export , filestub("inequality/mean_income_by_rankincome_`suf'") years($yrfirst/$yrlast) var(income) dataprivacy(data_privacy_mean)
}

// Log Earnings Kernel Densities
//-------------------------------------------------
noi disp as text "... inequality data: densities"
foreach het in "" $hetgrouptotinc_ineq_densities {
	local suf=subinstr("`het'"," ","",.)
	local years = "2001 2005 2010 2015"
	capture append_and_export , filestub("inequality/density_loginc_`suf'") years(`years') var(loginc) dataprivacy(data_privacy_density)
}



// Volatility Data 
//==========================================================================================

noi disp as text "... volatility data"

// Growth-Rate Percentiles
//-------------------------------------------------

foreach gvar in $growthvars {
foreach f in 1 5 {
  if `f' == 1 local years = "$d1yrlist"
  else local years = subinstr("$d5yrlist", ",", " ", .)
  foreach het in "" $hetgrouptotinc_vola {
    local suf=subinstr("`het'"," ","",.)
    capture append_and_export , filestub("volatility/sumstats_`gvar'`f'F_`suf'") years(`years') var(`gvar'`f'F) dataprivacy(data_privacy_sumstats)
  }
}
}

// Growth-Rate Kernel Densities
//-------------------------------------------------

foreach f in 1 5 {
  if `f' == 1 local years 2001 2005 2010 2015 
  else        local years 2001 2005 2010
  foreach het in "" $hetgrouptotinc_vola_densities {
    local suf=subinstr("`het'"," ","",.)
    capture append_and_export , filestub("volatility/density_resinc`f'F_`suf'") years(`years') var(resinc`f'F) dataprivacy(data_privacy_density)
  }
}

// Growth-Rate Percentiles by Permanent Income Bins
//-------------------------------------------------

foreach gvar in $growthvars {
foreach f in 1 5 {
  foreach het in "" $hetgrouptotinc_vola {
    local suf=subinstr("`het'"," ","",.)
    capture append_and_export , filestub("volatility/sumstats_`gvar'`f'F_by_perminc_`suf'") years($years_Hsample) var(`gvar'`f'F) dataprivacy(data_privacy_sumstats)
  }
}
}


// Mobility Data
//==========================================================================================


// Transitions of total income
//-------------------------------------------------
noi disp as text "... mobility data: transitions"

foreach sample in "" "_nothreshold" {
  local rankvar = cond("`sample'" == "", "loginc", "income")
  foreach ranktype in "" $rankgrouptotinc_mobility_trans {
    local sufrank = cond("`ranktype'" == "", "_by_total", subinstr("_by_`ranktype'", " ", "", .))

    foreach het in "" $hetgrouptotinc_mobility_trans {
    local suf=subinstr("`het'"," ","",.)
      if strpos("`ranktype'", "male") == 0 | strpos("`het'", "male") != 0 {

        foreach f in 1 5 {
          if `f' == 1 local years $d1yrlist 
          else local years = subinstr("$d5yrlist", ",", " ", .)
          local filestub = "mobility/transitions_rankincome`f'F`sample'`sufrank'_`suf'"
          capture {
            clear
            foreach yr of numlist `years' {
              append using "`filestub'`yr'"
            }
            // DATA PRIVACY
            if $protect == 1 {
              foreach var in male age income income`f'F n_weight N {
                replace `var' = . if N < 30
              }
              foreach var in ntotal_weight Ntotal {
                replace `var' = . if Ntotal < 30
              }
            }
            if substr("`filestub'", -1, .) == "_" local outstub = substr("`filestub'", 1, `= strlen("`filestub'") -1')
            else local outstub `filestub'
            export delimited using "`outstub'.csv", delimiter("$csvdelim") replace
            noi disp "save to CSV: $outfolder/`outstub'.csv"
          } // capture
        }
      }
    }
  }
}


// Future Permanent Income by Permanent Income
//-------------------------------------------------
noi disp as text "... mobility data: ranks"

foreach het in "" $hetgrouptotinc_mobility {
  local suf=subinstr("`het'"," ","",.)
  capture append_and_export , filestub("mobility/sumstats_rankpermincalt5F_by_rankpermincalt_`suf'") years(${years_permincalt}) var(rankpermincalt5F) dataprivacy(data_privacy_sumstats)
}


} // COLLECT

  
//------------------------------------------------------------------------------
} // quietly
//------------------------------------------------------------------------------

// END
//==========================================================================================

timer off 1
timer list 1

cap log close
