*! v20220620 hw
/*---------------------------------------------------------*/
*! Researchproject: Inequality and Inceom Dynamics in Germany (GRID project)
*! Author: Johannes F. Schmieder / Stefanie Wolter / Hannes Walz
/*---------------------------------------------------------*/


// =============== Preamble ================ //

{
//--- Settings ---//

* clearing
clear all
program drop _all
macro drop _all
cap log close _all
clear matrix

* set
version 16.1
set more off, perm
set varabbrev off, perm
set matsize 10000, perm
set maxiter 1000, perm
set iterlog on, perm
set linesize 255
set pagesize 100
set rmsg off, perm
set trace on
set tracedepth 2
set traceexpand off, perm
set tracesep off, perm
set traceindent off, perm
set tracenumber off, perm    

* date
global mydate: di %td_CY-N-D date("$S_DATE", "DMY")
global mydate=strltrim("$mydate")

* seed
set seed 190424


//--- paths ---//

* checks
pwd 
sysdir

* paths
global root     = "...\" 	 				// !!! change main dir path here !!!
	global do     = "${root}`c(dirsep)'do`c(dirsep)'"
	global data     = "${root}`c(dirsep)'dta`c(dirsep)'"
	global log      = "${root}`c(dirsep)'log`c(dirsep)'"
	global orig     = "${root}`c(dirsep)'orig`c(dirsep)'"
	global figs     = "${root}`c(dirsep)'figs`c(dirsep)'"
	global out      = "${root}`c(dirsep)'out`c(dirsep)'"	
foreach dir in "log" "data" "figs" "out" {
	cap mkdir "$`dir'"
}
cd "${root}"

* main log path
global mainlog "${log}${mydate}"
cap mkdir "${log}${mydate}"

* set project ado dir
adopath ++ "${do}/"
adopath ++ "${do}/Cola"

*check ado paths
adopath
}


/*===========================================================================================*/
/*                                      Main Program                                         */
/*===========================================================================================*/
capture program drop main
program define main
	set matsize 4000
	set li 120 // 200
	set trace on
	set tracesep off
	set tracedepth 1 //2
  
	local time = subinstr("`=c(current_time)'",":","",.)
	scalar d = date("`=c(current_date)'","DMY") 	// create log file including time and date in file name
	local b  _`=subinstr("`c(current_date)'"," ","",.)'_`time'
	global logfile ${mainlog}/${mydate}_GRID_Germany_preparation_IEB.log
	log using $logfile , text replace name(preparation_IEB_`b')
	
	// =====================================
	// Generate Yearly Data from Spell Data
	// =====================================
	
	  clean_yearly_data, in(${data}ieb_10perc_clean.dta) out(${data}clean_yearly_data)
	  
	  // fulltime var correction
	  fulltime_imp, ds(${data}clean_yearly_data) 
		
	  //prep vars for reweighting
	  reweight_prep, ds(${data}clean_yearly_data) 
	
	cap log preparation_IEB_`b'

end // main program

/*===========================================================================================*/
/*                                   Main Programs                                           */
/*===========================================================================================*/

/*---------------------------------------------*/
/* Create Person by Year level data*/
/*---------------------------------------------*/
cap program drop clean_yearly_data
program define clean_yearly_data
	syntax, in(str) out(str)

	use persid year erwstat wage te_imp betnr begindate enddate spell spelllength fulltime parttime gering /// Basic spell variables
		censor cpi gering  bbg /// More spell variables
		byear educ edyrs female nonger west  /// demographics
	    using `in' , clear

		// German mimimum wage as lowest income Min wage (2015): 8.50 * 260 in real terms

	gen min_income=2300 		//!hw old formel: ((8.84*260)*cpi)/100
	
	replace cpi=   71.8815351801058   if year == 1993
	replace cpi=   73.8173455045538   if year == 1994
	replace cpi=   75.0767883662429   if year == 1995
	replace cpi=   76.1651957775791   if year == 1996
	replace cpi=   77.6423201215354   if year == 1997
	replace cpi=   78.3497849389040   if year == 1998
	replace cpi=   78.8084709193957   if year == 1999
	replace cpi=   79.9435243626463   if year == 2000
	replace cpi=   81.5294894477363   if year == 2001
	replace cpi=   82.6878659069441   if year == 2002
	replace cpi=   83.5430431587083   if year == 2003
	replace cpi=   84.9346497774882   if year == 2004
	replace cpi=   86.2485130097440   if year == 2005
	replace cpi=   87.6090222739143   if year == 2006
	replace cpi=   89.6225759848864   if year == 2007
	replace cpi=   91.9782005965641   if year == 2008
	replace cpi=   92.2658511267030   if year == 2009
	replace cpi=   93.2842894901676    if year == 2010
	replace cpi=   95.2200998146156    if year == 2011
	replace cpi=   97.1325871231064    if year == 2012
	replace cpi=   98.5941627897579    if year == 2013
	replace cpi=   99.4882117347841    if year == 2014
	replace cpi=   100.0000000000000   if year == 2015
	replace cpi=   100.4917486247710   if year == 2016
	replace cpi=   102.0086681113520   if year == 2017
	replace cpi=   103.7756292715450   if year == 2018
	
	cap destring erwstat, force replace

	g byte employed = erwstat>=101  & erwstat<=109 & wage>0 & wage<.

	keep if employed==1

	g byte regular = !gering

	replace spelllength = enddate - begindate+1

	replace year = year(begindate)

	sort persid spell
	by persid: replace spell = _n

	g invwage = wage * -1 // just used for sorting - highest value first, infinity last

	// Drop Duplicate Notifications
	bys persid year betnr begindate enddate (invwage): g duplicate_notification = _n>1 // label 2 notification from same firm as duplicate, prefer higher paying notification
	drop if duplicate_notification ==1
	drop duplicate_notification invwage

	// Consistency Check
	assert spelllength <= 366 if employed // 	replace spelllength = 365 if spelllength > 365
	replace wage = 0 if wage==.
	
	// Fill in imputed wage with observed wage either if there is no imputed wage or
	// if the observation is not actually censored.
	rename te_imp wage_imputed
	
	replace  wage_imputed = wage if missing(wage_imputed) | censor==0

	// Generate yearly earnings	for all regular jobs
	bys persid year: egen earn_all_regular = sum(wage * spelllength * employed * regular)
	bys persid year: egen earn_full_regular = sum(wage * spelllength * fulltime * employed * regular)

	bys persid year: egen earn_all_regular_imputed = sum(wage_imputed * spelllength * employed * regular)
	bys persid year: egen earn_full_regular_imputed = sum(wage_imputed * spelllength * fulltime * employed * regular)

	// Generate yearly earnings	for all jobs (including Mini Jobs)
	bys persid year: egen earn_all = sum(wage * spelllength * employed)
	bys persid year: egen earn_full = sum(wage * spelllength * fulltime * employed)

	bys persid year: egen earn_all_imputed = sum(wage_imputed * spelllength * employed)
	bys persid year: egen earn_full_imputed = sum(wage_imputed * spelllength * fulltime * employed)

	// To do:
	// It would be helpful to have a variable for earning from main employer
	bys persid year betnr: egen earn_from_employer = sum(wage * spelllength * employed)
	bys persid year betnr: egen days_from_employer = sum(spelllength * employed)
	assert earn_from_employer<.
	bys persid year (earn_from_employer): g earn_main = earn_from_employer[_N]
	bys persid year (earn_from_employer): g days_main = days_from_employer[_N]
	bys persid year (earn_from_employer): g byte main_employer = (_n==_N) * employed
	drop earn_from_employer

	bys persid year betnr: egen earn_from_employer_imputed = sum(wage_imputed * spelllength * employed)
	assert earn_from_employer_imputed<.
	bys persid year (earn_from_employer_imputed): g earn_main_imputed = earn_from_employer_imputed[_N]
	drop earn_from_employer_imputed

	bys persid year betnr: egen earn_from_employer_regular = sum(wage * spelllength * regular * employed)
	assert earn_from_employer_regular<.
	bys persid year (earn_from_employer_regular): g earn_main_reg = earn_from_employer_regular[_N]
	drop earn_from_employer_regular

	bys persid year betnr: egen earn_from_employer_regular_imp = sum(wage_imputed * spelllength * regular * employed)
	assert earn_from_employer_regular_imp<.
	bys persid year (earn_from_employer_regular_imp): g earn_main_reg_imp = earn_from_employer_regular_imp[_N]
	drop earn_from_employer_regular_imp


	// Earnings from secondary employers
	g earn_second_employers = earn_all - earn_main
	g earn_second_employers_imputed = earn_all_imputed - earn_main_imputed

	// Employment Variables
	bys persid year: egen int days_full = sum(spelllength * fulltime *  employed)
	bys persid year: egen int days_work = sum(spelllength * employed)


	// Identify Main Employment Spell in Year
	bys persid year: gen inverse_earnings = -1 * wage * spelllength
	bys persid year (inverse_earnings) : g main_spell = _n==1
	assert main_spell==0 | main_spell==1

	order persid year earn_all inverse_earnings wage spelllength main_spell
	drop inverse_earnings

	bys persid year: egen sum_main_spell = sum(main_spell)
	tab sum_main_spell
	assert sum_main_spell==1
	drop sum_main_spell
	
	// education dummies
	
	gen school = inlist(educ, 1, 3)
	gen apprent	= inlist(educ, 2,4)
	gen university = inlist(educ, 5,6)

	label var school	"School graduation, no further training"
	label var apprent 	"Apprenticeship training degree"
	label var university	"College or university degree"

	// Keep only one observation per person and year (the one corresponding to highest earnings )
	bys persid year (main_spell): keep if _n==_N
	drop main_spell

	replace days_work = 365 if days_work>=365

	label var persid "Person ID"
	label var year "Year"
	label var betnr "Establishment ID"


	label var earn_all    "Earnings"
	label var earn_full   "Earnings from fulltime employment"
	label var earn_all_imputed    "Earnings (imputed)"
	label var earn_full_imputed   "Earnings from fulltime employment (imputed)"

	label var earn_all_regular    "Earnings (excl. Mini-Jobs)"
	label var earn_full_regular   "Earnings from fulltime employment (excl. Mini-Jobs)"
	label var earn_all_regular_imputed    "Earnings (excl. Mini-Jobs, imputed)"
	label var earn_full_regular_imputed   "Earnings from fulltime employment (excl. Mini, imputed)"

	label var earn_main "Earnings from main employer"
	label var earn_main_imputed "Earnings from main employer (imputed)"
	label var earn_main_reg "Earnings from main employer (excl. Mini-Jobs)"
	label var earn_main_reg_imp "Earnings from main employer, regular (excl. Mini, imputed)"

	label var earn_second_employers "Earnings from secondary employers"
	label var earn_second_employers_imputed "Earnings from secondary employers (imputed)"

	g share_main_emp_earnings = earn_main / earn_all
	g share_main_emp_earnings_imputed = earn_main_imputed / earn_all_imputed
	label var share_main_emp_earnings         "Ratio of main emp. to all earnings"
	label var share_main_emp_earnings_imputed "Ratio of main emp. to all earnings (imputed)"

	label var days_full "Days per year working fulltime"
	label var days_work "Days per year working"

	label var west "West Germany"
	label var byear "Birthyear"
	label var educ "Education"
	label var nonger "Non-German"
	label var regular "Regular employment (not mini)"
    // GID code asks for "year of death" obviously we don't have this, so we set it to retirement age.
	g dyear = byear + 65
  
	// set up earnings var as in GID code
	cap drop rearn* learn* lrearn*

	foreach var of varlist earn* {

		// Covert to real values
		// Criteria d (Inflation) in CS Sample

		
		// Winsorization
		bysort year: gen temp= `var' if year-byear+1>= 25 & year- byear+1<= 55 & year< dyear  // yod=. id very big number  
		_pctile temp, p(99.999999)
		bysort year: replace `var'= r(r1) if `var'>=r(r1) & `var'!=. 
		drop temp
		
		
		// Any earnings that are missing inside of $begin_age and $end_age are set to zero.
		bysort year: replace `var'= 0 if `var'== . &  ///
		   year-byear+1 >= 25& year- byear+1<=55 & year< dyear

			
		// Assing missing if outside of $begin_age and $end_age
		replace `var'= . if year-byear+1 < 25 | year- byear+1>55 
		replace `var'= . if year>= dyear & dyear~=.  // (yod=. is very big number)
		
		// real earnings
		cap drop cpi_base2018
		gen cpi_base2018=(cpi/103.7756292715450)
			g r`var'=`var'/cpi_base2018	

			local lbl : variable label `var'
		label var r`var'  "Real `lbl'"
	}

	g learn_all = log(earn_all)
	g learn_all_imputed = log(earn_all_imputed)
	g learn_all_regular = log(earn_all_regular)
	g learn_all_regular_imputed = log(earn_all_regular_imputed)
	g learn_main_reg = log(earn_main_reg)
	g learn_main_reg_imp = log(earn_main_reg_imp)

	label var learn_all "Log(Earnings)"
	label var learn_all_imputed "Log(Earnings, imp)"
	label var learn_all_regular "Log(Earnings, regular)"
	label var learn_all_regular_imputed "Log(Earnings, regular, imp)"
	label var learn_main_reg "Log(Earnings, main, regular)"
	label var learn_main_reg_imp "Log(Earnings, main, regular, imp)"

	g lrearn_all = log(rearn_all)
	g lrearn_all_imputed = log(rearn_all_imputed)
	g lrearn_all_regular = log(rearn_all_regular)
	g lrearn_all_regular_imputed = log(rearn_all_regular_imputed)
	g lrearn_main_reg = log(rearn_main_reg)
	g lrearn_main_reg_imp = log(rearn_main_reg_imp)

	label var lrearn_all "Log(Real Earnings)"
	label var lrearn_all_imputed "Log(Real Earnings, imp)"
	label var lrearn_all_regular "Log(Real Earnings, regular)"
	label var lrearn_all_regular_imputed "Log(Real Earnings, regular, imp)"
	label var lrearn_main_reg "Log(Real Earnings, main, regular)"
	label var lrearn_main_reg_imp "Log(Real Earnings, main, regular, imp)"

	rename persnr persid
	
	tsset persid year
	sort persid year
	isid persid year

	g male = (1-female)
	label var male "Male"
	label define male 1 "Male" 0 "Female"
	label values male male

	cap {
		label define west 0 "East Germany" 1 "West Germany"
		label values west west
	}

	tab male
	// get 2018-cpi as cpi
	gen cpi_OLD=cpi
	replace cpi=cpi_base2018

	// Add legal cutoffs
	cutoffs
  
	keep persid year betnr ///
		cpi* regular spelllength ///
		erwstat fulltime parttime gering ///
	 	byear dyear male nonger educ school apprent university edyrs   west  ///
		rearn* earn* days_full days_work days_* wage wage_imputed  ///
    share*  ///
    minwage rminwage minicutoff rminicutoff bbg* ///


	// Shortlist of variables used for Global Income Dynamics Project:
	// keep persid year male byear dyear educ earn_all west
	order persid year betnr byear male educ west earn* wage*
  
	// Earnings at least: Min wage (2015): 8.50 * 260 in real terms -- edit (hw): 2018: 8.84*260=2298.4 -> 2300
	drop if rearn_all<2300
  
	save `"`out'"', replace

	// Shortlist of variables used for Global Income Dynamics Project:
	// keep persid year male byear dyear educ earn_all west
	
end // clean_yearly_data

/*---------------------------------------------*/
/* Legal Cutoffs  */
/*---------------------------------------------*/
capture program drop cutoffs
program define cutoffs

	// Bezugsgroesse (from Rentenversicherung) https://de.wikipedia.org/wiki/Bezugsgr%C3%B6%C3%9Fe
	g bezugsgroesse = .
	replace bezugsgroesse = 3115 if year == 2019 & west==1
	replace bezugsgroesse = 3045 if year == 2018 & west==1
	replace bezugsgroesse = 2975 if year == 2017 & west==1
	replace bezugsgroesse = 2905 if year == 2016 & west==1
	replace bezugsgroesse = 2835 if year == 2015 & west==1
	replace bezugsgroesse = 2765 if year == 2014 & west==1
	replace bezugsgroesse = 2695 if year == 2013 & west==1
	replace bezugsgroesse = 2625 if year == 2012 & west==1
	replace bezugsgroesse = 2555 if year == 2011 & west==1
	replace bezugsgroesse = 2555 if year == 2010 & west==1
	replace bezugsgroesse = 2520 if year == 2009 & west==1
	replace bezugsgroesse = 2485 if year == 2008 & west==1
	replace bezugsgroesse = 2450 if year == 2007 & west==1
	replace bezugsgroesse = 2450 if year == 2006 & west==1
	replace bezugsgroesse = 2415 if year == 2005 & west==1
	replace bezugsgroesse = 2415 if year == 2004 & west==1
	replace bezugsgroesse = 2380 if year == 2003 & west==1
	replace bezugsgroesse = 2345 if year == 2002 & west==1
	replace bezugsgroesse = 4480 if year == 2001 & west==1
	replace bezugsgroesse = 4480 if year == 2000 & west==1
	replace bezugsgroesse = 4410 if year == 1999 & west==1
	replace bezugsgroesse = 4340 if year == 1998 & west==1
	replace bezugsgroesse = 4270 if year == 1997 & west==1
	replace bezugsgroesse = 4130 if year == 1996 & west==1
	replace bezugsgroesse = 4060 if year == 1995 & west==1
	replace bezugsgroesse = 3920 if year == 1994 & west==1
	replace bezugsgroesse = 3710 if year == 1993 & west==1
	replace bezugsgroesse = 3500 if year == 1992 & west==1
	replace bezugsgroesse = 3360 if year == 1991 & west==1
	replace bezugsgroesse = 3290 if year == 1990 & west==1

	replace bezugsgroesse = 2870 if year == 2019 & west==0
	replace bezugsgroesse = 2695 if year == 2018 & west==0
	replace bezugsgroesse = 2660 if year == 2017 & west==0
	replace bezugsgroesse = 2520 if year == 2016 & west==0
	replace bezugsgroesse = 2415 if year == 2015 & west==0
	replace bezugsgroesse = 2345 if year == 2014 & west==0
	replace bezugsgroesse = 2275 if year == 2013 & west==0
	replace bezugsgroesse = 2240 if year == 2012 & west==0
	replace bezugsgroesse = 2240 if year == 2011 & west==0
	replace bezugsgroesse = 2170 if year == 2010 & west==0
	replace bezugsgroesse = 2135 if year == 2009 & west==0
	replace bezugsgroesse = 2100 if year == 2008 & west==0
	replace bezugsgroesse = 2100 if year == 2007 & west==0
	replace bezugsgroesse = 2065 if year == 2006 & west==0
	replace bezugsgroesse = 2030 if year == 2005 & west==0
	replace bezugsgroesse = 2030 if year == 2004 & west==0
	replace bezugsgroesse = 1995 if year == 2003 & west==0
	replace bezugsgroesse = 1960 if year == 2002 & west==0
	replace bezugsgroesse = 3780 if year == 2001 & west==0
	replace bezugsgroesse = 3640 if year == 2000 & west==0
	replace bezugsgroesse = 3710 if year == 1999 & west==0
	replace bezugsgroesse = 3640 if year == 1998 & west==0
	replace bezugsgroesse = 3640 if year == 1997 & west==0
	replace bezugsgroesse = 3500 if year == 1996 & west==0
	replace bezugsgroesse = 3290 if year == 1995 & west==0
	replace bezugsgroesse = 3080 if year == 1994 & west==0
	replace bezugsgroesse = 2730 if year == 1993 & west==0
	replace bezugsgroesse = 2100 if year == 1992 & west==0
	replace bezugsgroesse = 1540 if year == 1991 & west==0
	replace bezugsgroesse = 1400 if year == 1990 & west==0

	replace bezugsgroesse = bezugsgroesse / 1.95583 if year <= 2001

	// Mini Jobs Cutoff
	g minicutoff = 390 / 1.95583 if inrange(year,1982,1995)
	replace minicutoff = 1/7 * bezugsgroesse if inrange(year,1996,1998) // Technically until April 1st, 1999
	replace minicutoff = 325 if inrange(year,1999,2002)
	replace minicutoff = 400 if inrange(year,2003,2012)
	replace minicutoff = 450 if year>=2013

	// Minimum Wage
	g float minwage = 0
	replace minwage = 8.5 if inrange(year,2015,2016)
	replace minwage = 8.84 if inrange(year,2017,2018)
	replace minwage = 9.19 if inrange(year,2019,2019)
	replace minwage = 9.35 if inrange(year,2020,2020)

	g rminicutoff = minicutoff / cpi * 100
	g rminwage    = minwage    / cpi * 100

	label var bezugsgroesse "Bezugsgroesse (RV)"
	label var minicutoff    "Mini Job Cutoff"
	label var rminicutoff    "Mini Job Cutoff (real)"
	label var minwage  "Minimum Wage"
	label var rminwage "Minimum Wage (real)"

	gen float bbg_daily_west=.
	gen float bbg_daily_east=.
	replace bbg_daily_west = 47.06  if year == 1975
	replace bbg_daily_west = 51.97  if year == 1976
	replace bbg_daily_west = 57.15  if year == 1977
	replace bbg_daily_west = 62.19  if year == 1978
	replace bbg_daily_west = 67.24  if year == 1979
	replace bbg_daily_west = 70.40  if year == 1980
	replace bbg_daily_west = 73.96  if year == 1981
	replace bbg_daily_west = 79.00  if year == 1982
	replace bbg_daily_west = 84.05  if year == 1983
	replace bbg_daily_west = 87.17  if year == 1984
	replace bbg_daily_west = 90.77  if year == 1985
	replace bbg_daily_west = 94.13  if year == 1986
	replace bbg_daily_west = 95.82  if year == 1987
	replace bbg_daily_west = 100.58 if year == 1988
	replace bbg_daily_west = 102.54 if year == 1989
	replace bbg_daily_west = 105.90 if year == 1990
	replace bbg_daily_west = 109.26 if year == 1991
	replace bbg_daily_west = 113.99 if year == 1992
	replace bbg_daily_west = 121.03	if year == 1993
	replace bbg_daily_west = 127.75	if year == 1994
	replace bbg_daily_west = 131.12	if year == 1995
	replace bbg_daily_west = 134.11	if year == 1996
	replace bbg_daily_west = 137.84	if year == 1997
	replace bbg_daily_west = 141.20	if year == 1998
	replace bbg_daily_west = 142.88	if year == 1999
	replace bbg_daily_west = 144.17	if year == 2000
	replace bbg_daily_west = 146.24	if year == 2001
	replace bbg_daily_west = 147.95	if year == 2002
	replace bbg_daily_west = 167.67	if year == 2003
	replace bbg_daily_west = 168.85	if year == 2004
	replace bbg_daily_west = 170.96	if year == 2005
	replace bbg_daily_west = 172.6  if year == 2006
	replace bbg_daily_west = 172.6	if year == 2007
	replace bbg_daily_west = 173.77	if year == 2008
	replace bbg_daily_west = 177.53	if year == 2009
	replace bbg_daily_west = 180.82	if year == 2010
	replace bbg_daily_west = 180.82	if year == 2011
	replace bbg_daily_west = 183.61	if year == 2012
	replace bbg_daily_west = 190.68	if year == 2013
	replace bbg_daily_west = 195.62	if year == 2014
	replace bbg_daily_west = 198.90	if year == 2015
	replace bbg_daily_west = 203.28	if year == 2016
	replace bbg_daily_west = 208.77	if year == 2017
	replace bbg_daily_west = 213.69	if year == 2018

	*east
	replace bbg_daily_east =	45.39  if year == 1990
	replace bbg_daily_east =	50.43  if year == 1991
	replace bbg_daily_east =	80.47  if year == 1992
	replace bbg_daily_east =	89.09  if year == 1993
	replace bbg_daily_east =	99.18  if year == 1994
	replace bbg_daily_east =	107.58 if year ==	1995
	replace bbg_daily_east =	113.99 if year ==	1996
	replace bbg_daily_east =	119.35 if year ==	1997
	replace bbg_daily_east =	117.67 if year ==	1998
	replace bbg_daily_east =	121.03 if year ==	1999
	replace bbg_daily_east =	119.02 if year ==	2000
	replace bbg_daily_east =	122.71 if year ==	2001
	replace bbg_daily_east =	123.29 if year ==	2002
	replace bbg_daily_east =	139.73 if year ==	2003
	replace bbg_daily_east =	142.62 if year ==	2004
	replace bbg_daily_east =	144.66 if year ==	2005
	replace bbg_daily_east =	144.66 if year ==	2006
	replace bbg_daily_east =	149.59 if year ==	2007
	replace bbg_daily_east =	147.54 if year ==	2008
	replace bbg_daily_east =	149.59 if year ==	2009
	replace bbg_daily_east =	152.88 if year ==	2010
	replace bbg_daily_east =	157.80 if year ==	2011
	replace bbg_daily_east =	157.38 if year ==	2012
	replace bbg_daily_east =	161.10 if year ==	2013
	replace bbg_daily_east =	164.38 if year ==	2014
	replace bbg_daily_east =	170.96 if year ==	2015
	replace bbg_daily_east =	177.05 if year ==	2016
	replace bbg_daily_east =	187.40 if year ==	2017
	replace bbg_daily_east =	190.68 if year ==	2018
	label var bbg_daily_west "Top Coding Threshold - West Germany (Daily Wage)"
	label var bbg_daily_east "Top Coding Threshold - West Germany (Daily Wage)"

	gen cpi_2015=.
	replace cpi_2015=   39.9418892256144   if year == 1975
	replace cpi_2015=   41.6380739865225   if year == 1976
	replace cpi_2015=   43.1929100173549   if year == 1977
	replace cpi_2015=   44.3671917749067   if year == 1978
	replace cpi_2015=   46.1612333489441   if year == 1979
	replace cpi_2015=   48.6728915525964   if year == 1980
	replace cpi_2015=   51.7608176557881   if year == 1981
	replace cpi_2015=   54.4736259753174   if year == 1982
	replace cpi_2015=   56.2676675493548   if year == 1983
	replace cpi_2015=   57.6213534643103   if year == 1984
	replace cpi_2015=   58.8119446907169   if year == 1985
	replace cpi_2015=   58.7358338360608   if year == 1986
	replace cpi_2015=   58.8826190557548   if year == 1987
	replace cpi_2015=   59.6328546230795   if year == 1988
	replace cpi_2015=   61.2909839566595   if year == 1989
	replace cpi_2015=   62.9436768006212   if year == 1990
	replace cpi_2015=   65.4910288078316   if year == 1991
	replace cpi_2015=   68.8028970737547   if year == 1992
	replace cpi_2015=   71.8815351801058   if year == 1993
	replace cpi_2015=   73.8173455045538   if year == 1994
	replace cpi_2015=   75.0767883662429   if year == 1995
	replace cpi_2015=   76.1651957775791   if year == 1996
	replace cpi_2015=   77.6423201215354   if year == 1997
	replace cpi_2015=   78.3497849389040   if year == 1998
	replace cpi_2015=   78.8084709193957   if year == 1999
	replace cpi_2015=   79.9435243626463   if year == 2000
	replace cpi_2015=   81.5294894477363   if year == 2001
	replace cpi_2015=   82.6878659069441   if year == 2002
	replace cpi_2015=   83.5430431587083   if year == 2003
	replace cpi_2015=   84.9346497774882   if year == 2004
	replace cpi_2015=   86.2485130097440   if year == 2005
	replace cpi_2015=   87.6090222739143   if year == 2006
	replace cpi_2015=   89.6225759848864   if year == 2007
	replace cpi_2015=   91.9782005965641   if year == 2008
	replace cpi_2015=   92.2658511267030   if year == 2009
	replace cpi_2015=   93.2842894901676    if year == 2010
	replace cpi_2015=   95.2200998146156    if year == 2011
	replace cpi_2015=   97.1325871231064    if year == 2012
	replace cpi_2015=   98.5941627897579    if year == 2013
	replace cpi_2015=   99.4882117347841    if year == 2014
	replace cpi_2015=   100.0000000000000   if year == 2015
	replace cpi_2015=   100.4917486247710   if year == 2016
	replace cpi_2015=   102.0086681113520   if year == 2017
	replace cpi_2015=   103.7756292715450   if year == 2018

	label var cpi_2015 "CPI (2015=100)"


end // cutoffs

/*-------------------------------------------------------*/
/* Fulltime imputation  */
/*-------------------------------------------------------*/	
cap program drop fulltime_imp
program define fulltime_imp /* options: ds(str)- datatset to load AND save */
syntax, ds(str)

	use "`ds'" , clear
	
	*** step 1: define sample and gen additional vars needed for imputation procedure 
	
	* sample variable
	gen rsample=(year>2000 & !missing(year) & gering==0 & (year-byear+1>=25) & (year-byear+1<=55))
	
	* get variables adjusted if needed
	//age
	gen age_temp=year-byear+1
	recode age_temp (0/19=1) (20/24=2) (56/60=3) (61/max=4)
	replace age_temp= (floor((year-byear+1)/3))*3 if age_temp>=25 & age_temp<=55
	// days not in employment
	gen non_work_temp = 365 - days_work
	recode non_work_temp (1/60=2) (61/182=3) (183/max=4) (0=1)
	
	*gen working days weighting variable (weight annual earnings by days worked)
	gen days_weight=365/days_work //schaltjahr should already be corrected	
	
	* control variables
	local cell_vars "age_temp edyrs west non_work_temp"
	
	//save "${data}/temp1"
	
	*** step 2: percentile thresholds & distance (idea: similar to Fitzenberge & Seidlitz (2020) 
	
	* lthreshold percentiles for 2012
	//female
	_pctile rearn_all_imputed [pw=days_weight] if year==2012 & male==0 & fulltime==1 & rsample==1, p(80)
	gen p80dw_f_2012=r(r1) 
	//male
	_pctile rearn_all_imputed [pw=days_weight] if year==2012 & male==1 & fulltime==1 & rsample==1, p(25)
	gen p25dw_m_2012=r(r1) 
	
   //indicator for below threshold
	gen below_threshold = (rearn_all_imputed<=p80dw_f_2012) if year==2012 & male==0 & fulltime==1 & rsample==1
	replace below_threshold=1 if rearn_all_imputed<=p25dw_m_2012 & year==2012 & male==1 & fulltime==1 & rsample==1

	//get quantile of threshold & probit sample indicator & distance indicator (theta) for 2012
	gen quantile_threshold=.
	gen psample=0
	gen theta=.
	forval g=0/1 {
		cumul rearn_all_imputed [aw=days_weight] if year==2012 & male==`g' & rsample==1, gen(cdf)
		egen temp=max(cdf) if below_threshold==1 & year==2012
		replace quantile_threshold= temp if below_threshold==1 & year==2012 & male==`g'     
		replace psample=1 if year==2012 & male==`g' & cdf<=quantile_threshold & rsample==1
		replace theta = (quantile_threshold - cdf)/quantile_threshold if year==2012 & male==`g' & rsample==1
		drop temp cdf
	}
	//fill var quantile_threshold
	sort male quantile_threshold
	by male: carryforward quantile_threshold, replace
	
	//save "${data}/temp2"
	
	* probit sample & distance indicator for years < 2012
	//loop over years
	forval yy=2001/2011{
		forval g=0/1 {
			cumul rearn_all_imputed [aw=days_weight] if year==`yy' & male==`g' & rsample==1, gen(cdf)    
			replace psample=1 if year==`yy' & male==`g' & cdf<=quantile_threshold & rsample==1
			replace theta = (quantile_threshold - cdf)/quantile_threshold if year==`yy' & male==`g' & rsample==1
			drop cdf
		}	    
	}
	
	* 10-categorical theta
	gen theta_cat=ceil(theta/10)
	
	//save "${data}/temp3"
	
	*** step 3: cell-wise shares & smooth correction factor
	
	* cell id
	egen cell_id=group(male `cell_vars')
	//replace cell_id=. if psample!=1

	//save "${data}/temp3", replace	
	
	* preserve to be able to go to cell-level
	preserve 
			
	* collapse to get cell-wise share and N
	drop if missing(year)
	drop if missing(cell_id)
	keep fulltime cell_id persid year male `cell_vars'
	collapse (mean) share_FT=fulltime (count) NN=persid, by(year cell_id)
	
	tab year, sum(share_FT)
	
	//save "${data}/temp4"
	
	* get share diff
	xtset cell_id year
	gen share_diff=share_FT-L1.share_FT
	
	* calculate 'smooth correction' factor and corrected share difference to previous year for 2011 & 2012 by cell
	//smoothed correction for 2010->2011 & 2011->2012 FT share developments
	gen smooth_corr=0 if year>2012
	replace smooth_corr = ((0.5* L2.share_diff +  L1.share_diff + F2.share_diff + 0.5* F3.share_diff )/3) if year==2011
	replace smooth_corr = ((0.5* L2.share_diff +  F1.share_diff + 0.5* F2.share_diff )/2) if year==2012
	//gap identification
	gen share_diff_corr=0 if year>2012
	replace share_diff_corr=share_diff - smooth_corr if year==2012
	replace share_diff_corr=share_diff - smooth_corr if year==2011
	sum smooth_corr
	sum share_diff
	sum share_diff_corr
	* generate share_FT corrected and potential gap by cell
	gen share_FT_corr=share_FT if year>2011
	replace share_FT_corr = F1.share_FT * (1 - F1.share_diff_corr) if year==2011
	replace share_FT_corr = (F2.share_FT * (1 - F2.share_diff_corr)) * (1 - F1.share_diff_corr) if year==2010
	
	gen p_gap=0 if year>2011
	replace p_gap=share_FT - share_FT_corr if year==2010 | year==2011
	
	
	forval yr=2001/2009 {
		local dist=2010-`yr'
		replace p_gap=F`dist'.p_gap if year==`yr'
		replace share_FT_corr=share_FT-p_gap
	}
	xtset, clear
		
	* save file
	save "${data}/temp_ft_corr", replace
	
	* return to individual level
	restore
	
	
	*** impute fulltime variable
	
	* merge data
	merge m:1 cell_id year using "${data}/temp_ft_corr"
	drop _merge cell_id
	
	//gen random variables for random assignment later in recoding, including probability increase using theta
	gen randomvar1=rnormal()
	recode randomvar1 (min/0=0) (10/max=10)
	gen randomvar2=runiform(0,10)
	gen randomvar=randomvar1*randomvar2
	gen noisy_theta=(10*theta)+randomvar1
	
	* gen imputed var
	//gen var
	gen fulltime_imp=fulltime
	replace fulltime_imp=0 if gering==1	
	//correct semi-randomly in cells
	bysort year male `cell_vars' (fulltime noisy_theta randomvar randomvar1 randomvar2): replace fulltime_imp=0 if (p_gap>0) & fulltime==1 & (_n/_N > (1-p_gap)) & year>=2001 & year<=2011 & psample==1	

	//save "${data}/temp5"
	
	* drop unneeded vars
	drop rsample psample
	drop age_temp non_work_temp theta theta_cat
	drop p80dw_f_2012 p25dw_m_2012 below_threshold quantile_threshold  
	drop share_FT NN share_diff smooth_corr share_diff_corr share_FT_corr p_gap 
	drop randomvar1 randomvar2 randomvar noisy_theta
	
	* labels
	lab var fulltime_imp "Fulltime employment (imputed using cellwise smoothing)"
	lab var days_weight "Weight for days worked (comparable annual wages when employment < 365 days/year)"

	save "`ds'" , replace	
	
end // fulltime_imp 	

/*-------------------------------------------------------*/
/* prep variables for reweighting  */
/*-------------------------------------------------------*/		
capture program drop reweight_prep	
program define reweight_prep/* options: ds(str)- datatset to load AND save */
syntax, ds(str)

	use "`ds'" , clear

	cap mkdir "${figs}/${mydate} reweight prep meta/"
	
	*** weights
	cap drop weight
	gen weight=1

	*** demographics
	
	* edyrs Variable (4 yrs)
	gen educ4 = educ
	recode educ4 (1 3=1) (2=2) (4=3) (5 6=4) (.=9)
	lab var educ4 "education years in 4 categories"
	lab def educ4lbl 1 "Schulabschluss ohne Berufsausbildung" ///
					2 "Volks-/Hauptschuleabschluss o. Mittlere Reife + Berufsausbildung" ///
					3 "Abitur und Berufsausbildung" ///
					4 "(Fach-)Hochschul- oder Universitätsabschluss" ///
					9 "Missing"
	lab val educ4 educ4lbl		
	
	gen educ2 = educ4
	recode educ2 (4=1) (1 2 3 9 .=2)
	lab def educ2lbl 1 "College degree" 2 "Below college degree an missing"
	lab val educ2 educ2lbl
	
	
	* age varibale in 3-y cohorts	
	gen age=year-byear+1
	gen age3y=(floor(age/3))*3
	lab var age3y "Age in 3-year cohorts, value is 'starting' age"
	
	
	*** work details
	
	* days in employment variables
	xtset persid year
	gen days_work_c = 365 - days_work
	gen days_work_p1 = 365 - L.days_work
	gen days_work_p2 = 730 - int(L.days_work + L2.days_work)
	gen days_work_p3 = 1095 - int(L.days_work + L2.days_work + L3.days_work)
	gen days_work_p5 = 1825 - int(L.days_work + L2.days_work + L3.days_work + L4.days_work + L5.days_work)	
	lab var days_work_c "days in employment - current year"
	lab var days_work_p1 "days in employment - last year"
	lab var days_work_p2 "days in employment - last 2 years"
	lab var days_work_p3 "days in employment - last 3 years"
	lab var days_work_p5 "days in employment - last 5 years"

	xtset, clear
	
	* job type variables
	gen jobtype1=.
	gen jobtype2=.
	replace jobtype1=1 if gering==0 & fulltime==1
	replace jobtype1=2 if gering==0 & fulltime==0
	replace jobtype1=3 if gering==1 & fulltime==0
	replace jobtype1=3 if gering==1 & fulltime==1
	replace jobtype2=1 if gering==0 & fulltime_imp==1
	replace jobtype2=2 if gering==0 & fulltime_imp==0
	replace jobtype2=3 if gering==1 & fulltime_imp==0
	replace jobtype2=3 if gering==1 & fulltime_imp==1	
	// check wages of categories:
	tw (histogram lrearn_all_imputed if gering==0 & fulltime==0 & male==0, color(blue%50)) || ///
		(histogram lrearn_all_imputed if gering==0 & fulltime==1 & male==0, color(red%50)) || ///
		(histogram lrearn_all_imputed if gering==1 & fulltime==0 & male==0, color(green%50)) || ///
		(histogram lrearn_all_imputed if gering==1 & fulltime==1 & male==0, color(black%30))
	graph export "${figs}/${mydate} reweight prep meta/logearn_var_by_jobtype1.pdf", replace as(pdf)
	tw (histogram lrearn_all_imputed if gering==0 & fulltime_imp==0 & male==0, color(blue%50)) || ///
		(histogram lrearn_all_imputed if gering==0 & fulltime_imp==1 & male==0, color(red%50)) || ///
		(histogram lrearn_all_imputed if gering==1 & fulltime_imp==0 & male==0, color(green%50)) || ///
		(histogram lrearn_all_imputed if gering==1 & fulltime_imp==1 & male==0, color(black%30))
	graph export "${figs}/${mydate} reweight prep meta/logearn_var_by_jobtype2.pdf", replace as(pdf)
	//-> looks plausible and gering over-writing fulltime should be applied
	label var jobtype1 "FT / PT / MJ by variables fulltime & gering in clean_yearly_data"
	label var jobtype2 "FT / PT / MJ by variables corrected fulltime & gering in clean_yearly_data"
	lab def jobtypelbl 1 "Fulltime" 2 "Parttime" 3 "Minijob"
	lab val jobtype1 jobtype2 jobtypelbl
	forval jt=1/2 {
		preserve
		keep year jobtype`jt' 
		tab jobtype`jt', gen(jt)
		keep year jt*
		collapse jt1 jt2 jt3, by(year)
		tw (line jt1 year) || (line jt2 year) || (line jt3 year) 
		graph export "${figs}/${mydate} reweight prep meta/jobtype`jt'_ts_all.pdf", replace as(pdf)
		restore
		tab year, sum(fulltime)
		tab year, sum(gering)
		forval i=0/1 {
			preserve
			keep year jobtype`jt' male
			keep if male==`i'
			tab jobtype`jt', gen(jt)
			keep year jt*
			collapse jt1 jt2 jt3, by(year)
			tw (line jt1 year) || (line jt2 year) || (line jt3 year) 
			graph export "${figs}/${mydate} reweight prep meta/jobtype`jt'_ts_m`i'.pdf", replace as(pdf)
			restore	
			tab year if male==`i', sum(fulltime)
		}
	}

	save "`ds'" , replace
	
end // reweight_prep	


/*---------------------------------------------------------*/
/* Run Main Program */
/*---------------------------------------------------------*/
main // run main program

cap log close _all
set trace off
clear
exit

/*========================================= END ==============================================*/
