//=======================================================================================
// Distribution: Bins
//=======================================================================================

// Load data
import delimited using "${joint_folder}/inequality/bins_labor_male.csv", clear case(preserve)
preserve
  import delimited using "${joint_folder}/inequality/bins_labor.csv", clear case(preserve)
  generate male = 9
  tempfile popdata
  save "`popdata'", replace
restore
append using "`popdata'"

// Rescale
qui sum lb, meanonly
foreach var of varlist N* {
  replace `var' = `var' / (ub-lb) * 1000 if lb < r(max)
  replace `var' = `var' / 1e6
}
generate midpoint = (ub+lb) / 2 / 1000
keep year male N_imp midpoint
rename N_imp N

// Smoothing
bysort year male (midpoint): generate N_smooth = (N[_n-1] + N + N[_n+1]) / 3
replace N_smooth = N if missing(N_smooth) | midpoint < 10
drop N
rename N_smooth N
     
// Quantiles, CDF & Density
local byvars year male
bysort `byvars': gegen _Ntotal = total(N)
bysort `byvars' (midpoint): generate _cdf = sum(N / _Ntotal)
generate pmf = N / _Ntotal
drop _Ntotal

qui foreach p in 5 10 25 50 75 90 95 99 {
  generate _diff = abs(100*_cdf - `p')
  bysort `byvars' (midpoint): gegen _mindiff = min(_diff)
  generate p`p'_N = N if _diff == _mindiff
  generate p`p'_pmf = pmf if _diff == _mindiff
  drop _diff _mindiff 
}
drop _cdf

foreach subgp in male female all {

  foreach lb in 2 8 {
    preserve
      if "`subgp'" == "male"    keep if male == 1
      if "`subgp'" == "female"  keep if male == 0
      if "`subgp'" == "all"     keep if male == 9
      keep if midpoint >= `lb' & midpoint < 80
      local suf = cond(`lb'==2, "", "_zoom")
        
      foreach yvar in N pmf {
        
        local ytitle = cond("`yvar'" == "N", "Number of Observations (in mill.)", "Share of Observations")
        local colors blue green red black navy pink
        local patterns solid longdash dash shortdash longdash_dot
        local twoway = ""
        local legend = ""
        local k = 1
        foreach year in 2001 2008 2016 {
          local pat: word `k' of `patterns'
          local col: word `k' of `colors'
          local twoway = "`twoway' (line `yvar' midpoint if year == `year', lcolor(`col') lwidth(*1.5...) lpattern(`pat'))"
          local twoway = "`twoway' (scatter p10_`yvar' p25_`yvar' p50_`yvar' p75_`yvar' p90_`yvar' midpoint if year == `year', msymbol(O S D T O) msize(*1.5...) color(`col' `col' `col' `col' `col') mfcolor(%50...))"
          local legend = `"`legend' `=(`k'-1)*6 + 1' "`year'""'
          local ++k
        }
        twoway  `twoway' ///
                , /* Options */ ///
                 legend(order(`legend')) ylabel(#8) xlabel(0(10)80) ///
                 ytitle("`ytitle'") ///
                 xtitle("Annual Earnings (in 1000 Euro)") name(`subgp', replace)
        graph export "$figs_folder/core/`yvar'_bins_labor_selected_years`suf'_`subgp'`suf'.pdf", replace
      }
    restore
  }
}


//=======================================================================================
// Main Distributional Statistics (Moments)
//=======================================================================================

cd "$joint_folder"


foreach var in logearn researn {

// Loop over (Sub-)Populations
qui foreach subgp in all male female {


  // Load data
	if "`subgp'" == "all"   local suffix = ""
	else                    local suffix = "_male"
  load_csvdata, filename("inequality/sumstats_`var'`suffix'.csv") dropsuffix(`var')
  if "`subgp'" == "male"    keep if male == 1
  if "`subgp'" == "female"  keep if male == 0

  // Additional statistics
  add_statistics
 
  // Core percentiles
  local ylabel = cond("`var'" == "logearn", "8(0.5)11.5", "-1.5(0.5)1")
  plot_timeseries p90 p75 p50 p25 p10, ///
                  legend(order(2 "P90" 3 "P75" 4 "P50" 5 "P25" 6 "P10")) ///
                  ytitle("Log Earnings Percentiles") ylabel(`ylabel')
  graph export "$figs_folder/core/`var'_core_percentiles_`subgp'.pdf", replace      
  
  // Top percentiles
  local ylabel = cond("`var'" == "logearn", "10.5(0.5)14", "0.5(0.5)3.5")
  plot_timeseries p99_99 p99_9 p99 p95 p90, ///
                  legend(order(2 "P99.99" 3 "P99.9" 4 "P99" 5 "P95" 6 "P90")) ///
                  ytitle("Log Earnings Percentiles") ylabel(`ylabel')
  graph export "$figs_folder/core/`var'_top_percentiles_`subgp'.pdf", replace

  // Core percentiles indexed
  plot_timeseries p90 p75 p50 p25 p10, index ///
                  legend(order(2 "P90" 3 "P75" 4 "P50" 5 "P25" 6 "P10")) ///
                  ytitle("Log Earnings Percentiles Relative to $yrfirst") ylabel(-0.2(0.05)0.2)
  graph export "$figs_folder/core/`var'_core_percentiles_index_`subgp'.pdf", replace      
  
  // Top percentiles indexed
  plot_timeseries p99_99 p99_9 p99 p95 p90, index ///
                  legend(order(2 "P99.99" 3 "P99.9" 4 "P99" 5 "P95" 6 "P90")) ///
                  ytitle("Log Earnings Percentiles Relative to $yrfirst") ylabel(-0.1(0.1)0.4)
  graph export "$figs_folder/core/`var'_top_percentiles_index_`subgp'.pdf", replace
 
  // Standard deviation and P90-P10
  plot_timeseries sd256 p90p10, ///
                  colors(blue red) msymbol(O S) ///
                  legend(order(2 "2.56*{&sigma}" 3 "P90-P10")) ///
                  ytitle("Dispersion of Log Earnings") ylabel(1.7(0.1)2.3)
  graph export "$figs_folder/core/`var'_sd_p90p10_`subgp'.pdf", replace

  // P50-P10 and P90-P50
  plot_timeseries p90p50 p50p10, ///
                  colors(blue red) msymbol(O S) ///
                  legend(order(2 "P90-P50" 3 "P50-P10")) ///
                  ytitle("Dispersion of Log Earnings") ylabel(0.6(0.2)1.6)
  graph export "$figs_folder/core/`var'_p90p50_p50p10_`subgp'.pdf", replace     

  // P50-P10 and P90-P50 and P75-P25
  plot_timeseries p90p50 p50p10 p75p25, ///
                  colors(blue red green) msymbol(O S D) ///
                  legend(order(2 "P90-P50" 3 "P50-P10" 4 "P75-P25")) ///
                  ytitle("Dispersion of Log Earnings") ylabel(0.6(0.2)1.6)
  graph export "$figs_folder/core/`var'_p90p50_p50p10_p75p25_`subgp'.pdf", replace     


} // subgp   

} // var    

                         
//=======================================================================================
// Within-Cohort Inequality
//=======================================================================================

foreach data in iab iabtpp {

if "`data'" == "iab" cd "$iab_folder/$mydate 206_Distributions"
else if "`data'" == "iabtpp" cd "$joint_folder"

// Loop over (Sub-)Populations
qui foreach subgp in all male female {
  // Load data
  load_csvdata, filename("inequality/sumstats_logearn_maleage.csv") dropsuffix(logearn)
  preserve
    load_csvdata, filename("inequality/sumstats_logearn_age.csv") dropsuffix(logearn)
    generate male = 9
    tempfile aggdata
    save "`aggdata'", replace
  restore
  append using "`aggdata'"   
  
  if "`subgp'" == "male"    keep if male == 1
  if "`subgp'" == "female"  keep if male == 0
  if "`subgp'" == "all"     keep if male == 9
  
  // Entry cohort
  generate cohort = year - age + 25
  
  // Additional statistics
  add_statistics

  // Initial inequality at age 25: standard deviation and P90-P10
  plot_timeseries sd256 p90p10 if age == 25, ///
                  colors(blue red) msymbol(O S) ///
                  legend(order(2 "2.56*{&sigma}" 3 "P90-P10")) ///
                  ytitle("Dispersion of Log Earnings at Age 25") ylabel(2(0.05)2.25) xlabel(2000(3)2018)
  graph export "$figs_folder/core/logearn_age25_sd_p90p10_`subgp'_`data'data.pdf", replace

  // Upper and Lower Initial inequality at age 25
  plot_timeseries p90p50 p50p10 if age == 25, ///
                  colors(blue red) msymbol(O S) ///
                  legend(order(2 "P90-P50" 3 "P50-P10")) ///
                  ytitle("Dispersion of Log Earnings at Age 25") ylabel(0.6(0.2)1.6) xlabel(2000(3)2018)
  graph export "$figs_folder/core/logearn_age25_p90p50_p50p10_`subgp'_`data'data.pdf", replace


  // P50 by age for selected cohorts
  plot_cohortage_timeseries p50 year, ///
                            cohorts(2001 2005 2009 2013) /// what cohorts
                            ages(25 30 35) /// what ages
                            agesarrow(25 30 35) ///
                            legpos(11) ///
                            twowayoptions( ///
                                xlabel(2000(3)2018) /// x-axis
                                ylabel(9.6(0.2)10.6) ///
                                ytitle("P50 of Log Earnings") xtitle("") ///
                            ) 
  graph export "$figs_folder/core/logearn_cohort_p50_`subgp'_`data'data.pdf", replace

  // P90-P10 by age for selected cohorts
  plot_cohortage_timeseries p90p10 year, ///
                            cohorts(2001 2005 2009 2013) /// what cohorts
                            ages(25 30 35) /// what ages
                            agesarrow(25 30 35) ///
                            legpos(7) ///
                            twowayoptions( ///
                                xlabel(2000(3)2018) /// x-axis
                                ylabel(1.7(0.1)2.4) ///
                                ytitle("P90-P10 of Log Earnings") xtitle("") ///
                            )
  graph export "$figs_folder/core/logearn_cohort_p90p10_`subgp'_`data'data.pdf", replace

  // P90-P50 by age for selected cohorts
  plot_cohortage_timeseries p90p50 year, ///
                            cohorts(2001 2005 2009 2013) /// what cohorts
                            ages(25 30 35) /// what ages
                            agesarrow(25 30 35) ///
                            legpos(11) ///
                            twowayoptions( ///
                                xlabel(2000(3)2018) /// x-axis
                                ylabel(0.5(0.1)0.9) ///
                                ytitle("P90-P50 of Log Earnings") xtitle("") ///
                            )
  graph export "$figs_folder/core/logearn_cohort_p90p50_`subgp'_`data'data.pdf", replace

  // P50-P10 by age for selected cohorts
  plot_cohortage_timeseries p50p10 year, ///
                            cohorts(2001 2005 2009 2013) /// what cohorts
                            ages(25 30 35) /// what ages
                            agesarrow(25 30 35) ///
                            legpos(7) ///
                            twowayoptions( ///
                                xlabel(2000(3)2018) /// x-axis
                                ylabel(1(0.1)1.7) ///
                                ytitle("P50-P10 of Log Earnings") xtitle("") ///
                            )
  graph export "$figs_folder/core/logearn_cohort_p50p10_`subgp'_`data'data.pdf", replace

}
}


//=======================================================================================
// Appendix: Within-Cohort Changes in Characteristics
//=======================================================================================

cd "$iab_folder/$mydate Data ddv cohort plot"

// Loop over (Sub-)Populations
qui foreach subgp in male female {
  // Load data
  load_csvdata, filename("data_diff_dep_var_cohort_plot_v2.csv") 
  rename mean_rearn mean_labor
  keep if year >= 2001 & year <= 2018
  if "`subgp'" == "male"    keep if male == 1
  if "`subgp'" == "female"  keep if male == 0
  
  // Entry cohort
  generate cohort = year - age + 25
  
  // Job Types
  foreach educ in "" 1 2 {
  foreach k in 1 2 3 {
  
    if `k' == 1 local ytitle = "Full-Time Share"
    if `k' == 2 local ytitle = "Part-Time Share"
    if `k' == 3 local ytitle = "Mini-Job Share"

    if `k' == 1 local label = "ft"
    if `k' == 2 local label = "pt"
    if `k' == 3 local label = "mj"
 
    if `k' == 1 local legpos = 7
    if `k' == 2 local legpos = 11
    if `k' == 3 local legpos = 7

    if `k' == 1 local ylabel = "0.4(0.1)1"
    if `k' == 2 local ylabel = "0(0.1)0.6"
    if `k' == 3 local ylabel = cond("`educ'" == "2", "0(0.05)0.3", "0.02(0.02)0.13")

    preserve
      if "`educ'" != "" keep if educ2 == `educ'
      local educsuf = cond("`educ'" == "", "", cond("`educ'" == "1", "_nocollege", "_college"))
      generate thistype = jobtype == `k'
      gcollapse (sum) N, by(year cohort age thistype)
      gegen Ntotal = total(N), by(year cohort age)
      generate share = N / Ntotal
      drop if thistype == 0
      auto_legend_position share year if inlist(cohort, 2001, 2005, 2009, 2013), ylabel(`ylabel')
      local legpos = `r(position)'
      plot_cohortage_timeseries share year, ///
                                cohorts(2001 2005 2009 2013) /// what cohorts
                                ages(25 30 35) /// what ages
                                agesarrow(25 30 35) ///
                                legpos(`legpos') ///
                                twowayoptions( ///
                                    xlabel(2000(3)2018) /// x-axis
                                    ylabel(`ylabel') ///
                                    ytitle("`ytitle'") xtitle("") ///
                                ) 
      graph export "$figs_folder/core/`label'_share_cohort_`subgp'`educsuf'.pdf", replace
    restore
  } // k
  } // educ
 
  // Average Earnings
  foreach educ in "" 1 2 {
    local ylabel = cond("`educ'" == "1", "9.7(0.2)10.5", "9.5(0.5)11.2")
    preserve
      if "`educ'" != "" keep if educ2 == `educ'
      local educsuf = cond("`educ'" == "", "", cond("`educ'" == "1", "_nocollege", "_college"))
      gcollapse (mean) mean_labor [aw=N], by(year cohort age)
      generate logmean = log(mean_labor)
      plot_cohortage_timeseries logmean year, ///
                                cohorts(2001 2005 2009 2013) /// what cohorts
                                ages(25 30 35) /// what ages
                                agesarrow(25 30 35) ///
                                legpos(11) ///
                                twowayoptions( ///
                                    xlabel(2000(3)2018) /// x-axis
                                    ylabel(`ylabel') ///
                                    ytitle("Log Average Earnings") xtitle("") ///
                                ) 
      graph export "$figs_folder/core/meanlabor_cohort_`subgp'`educsuf'.pdf", replace
    restore
  } // educ  

  // Number of Observations
  foreach educ in "" 1 2 {
    preserve
      if "`educ'" != "" keep if educ2 == `educ'
      local educsuf = cond("`educ'" == "", "", cond("`educ'" == "1", "_nocollege", "_college"))
      local ylabel = cond("`educ'" == "", "250(50)450", cond("`educ'" == "1", "210(20)310", "20(20)120"))
      gcollapse (sum) N=N, by(year cohort age)
      replace N = N / 1e3 / 0.1
      plot_cohortage_timeseries N year, ///
                                cohorts(2001 2005 2009 2013) /// what cohorts
                                ages(25 30 35) /// what ages
                                agesarrow(25 30 35) ///
                                legpos(11) ///
                                twowayoptions( ///
                                    xlabel(2000(3)2018) /// x-axis
                                    ylabel(`ylabel') ///
                                    ytitle("Number of Observations (in thous.)") xtitle("") ///
                                ) 
      graph export "$figs_folder/core/n_cohort_`subgp'`educsuf'.pdf", replace
    restore
  } // educ  
  
  // Education
  preserve
    *drop if educ2 == 9 (assume missing = no college)
    generate college = educ2 == 2
    gcollapse (sum) N, by(year cohort age college)
    gegen Ntotal = total(N), by(year cohort age)
    generate share = N / Ntotal
    drop if college == 0
    plot_cohortage_timeseries share year, ///
                              cohorts(2001 2005 2009 2013) /// what cohorts
                              ages(25 30 35) /// what ages
                              agesarrow(25 30 35) ///
                              legpos(11) ///
                              twowayoptions( ///
                                  xlabel(2000(3)2018) /// x-axis
                                  ylabel(0.1(0.1)0.4) ///
                                  ytitle("College Share") xtitle("") ///
                              ) 
    graph export "$figs_folder/core/college_share_cohort_`subgp'.pdf", replace
  restore

}

 
 
//=======================================================================================
// Income Shares
//=======================================================================================

cd "$tpp_folder/$mydate 206_Distributions (labor, rv)"


// Loop over (Sub-)Populations
foreach subgp in all male female {

  // Load data
	if "`subgp'" == "all"   local suffix = ""
	else                    local suffix = "_male"
  load_csvdata, filename("inequality/mean_labor_by_ranklabor`suffix'.csv") dropsuffix(labor)
  if "`subgp'" == "male"    keep if male == 1
  if "`subgp'" == "female"  keep if male == 0
    
  // Income Shares
  compute_incshares, by(year)
  
  // Quintile Shares
  preserve
    plot_timeseries share_Q1-share_Q5, index ///
                colors(green black maroon red blue) msymbol(O X D S T) ///
                legend(order(6 "Q5 (top)" 5 "Q4" 4 "Q3" 3 "Q2" 2 "Q1 (bottom)")) ///
                ytitle("Change in Income Shares Relative to $yrfirst") ylabel(-0.02(0.01)0.04)
    graph export "$figs_folder/core/labor_incshares_quintiles_index_`subgp'.pdf", replace     
  restore
  
  // Quintile Shares, indexed using ratios
  preserve
    index_timeseries share*, method(ratio)    
    plot_timeseries share_Q1-share_Q5, index ///
                colors(green black maroon red blue) msymbol(O X D S T) ///
                legend(order(6 "Q5 (top)" 5 "Q4" 4 "Q3" 3 "Q2" 2 "Q1 (bottom)")) ///
                ytitle("Change in Income Shares Relative to $yrfirst") 
    graph export "$figs_folder/core/labor_incshares_quintiles_indexratio_`subgp'.pdf", replace     
  restore
  
  // Selected Shares
  preserve
    plot_timeseries share_bot50 share_top10 share_top5 share_top1 share_top0_1 share_top0_01, index ///
                colors(green black maroon red blue olive) msymbol(O T D S dh X) ///
                legend(order(7 "Top 0.01%" 6 "Top 0.1%" 5 "Top 1%" 4 "Top 5%" 3 "Top 10%" 2 "Bottom 50%")) ///
                ytitle("Abs. Change in Income Shares Relative to $yrfirst") ylabel(-0.03(0.01)0.03)
    graph export "$figs_folder/core/labor_incshares_selected_index_`subgp'.pdf", replace     
  restore

  // Selected Shares, indexed using ratios
  preserve
    index_timeseries share_*, method(ratio)
    plot_timeseries share_bot50 share_top10 share_top5 share_top1 share_top0_1 share_top0_01, ///
                colors(green black maroon red blue olive) msymbol(O T D S dh X) ///
                legend(order(7 "Top 0.01%" 6 "Top 0.1%" 5 "Top 1%" 4 "Top 5%" 3 "Top 10%" 2 "Bottom 50%")) ///
                ytitle("Rel. Change in Income Shares Relative to $yrfirst") ylabel(-0.2(0.1)0.3)
    graph export "$figs_folder/core/labor_incshares_selected_indexratio_`subgp'.pdf", replace     
  restore
  
 // Table
 table_income_shares using "$figs_folder/core/labor_incshares_`subgp'.tex"
  
}
  
 
// Gini
//--------------------------------------------------------

*cd "$tpp_folder/$mydate 206_Distributions (labor, rv)"
cd "$joint_folder"

// Population
load_csvdata, filename("inequality/sumstats_labor.csv") dropsuffix(labor)
plot_timeseries gini , ///
                legend(order(2 "Gini")) ///
                ytitle("Gini of Earnings") ylabel(0.33(0.01)0.41)
graph export "$figs_folder/core/labor_gini_all.pdf", replace      
      
// By gender
load_csvdata, filename("inequality/sumstats_labor_male.csv") dropsuffix(labor)
keep gini male year
greshape wide gini, by(year) keys(male)
plot_timeseries gini0 gini1, ///
                colors(red blue) msymbol(O S) lpattern(solid longdash) ///
                legend(order(2 "Women" 3 "Men")) ///
                ytitle("Gini of Earnings") ylabel(0.33(0.01)0.41)
graph export "$figs_folder/core/labor_gini_gender.pdf", replace      
   


// Pareto Tails
//--------------------------------------------------------

cd "$tpp_folder/$mydate 206_Distributions (labor, rv)"
*cd "$joint_folder"


// Loop over (Sub-)Populations
qui foreach subgp in all male female {
foreach p in 1 5 {

  // Load Percentiles
	if "`subgp'" == "all"   local suffix = ""
	else                    local suffix = "_male"
  load_csvdata, filename("inequality/sumstats_logearn`suffix'.csv") dropsuffix(logearn)
  if "`subgp'" == "male"    keep if male == 1
  if "`subgp'" == "female"  keep if male == 0
  
  keep year p*
  
  greshape long p , by(year) keys(pstr) string

  generate cdf = subinstr(pstr, "_", ".", .)
  destring cdf, replace
  
  // Dependent variable
  keep if cdf >= 100 - `p'
  generate yvar = log(1 - cdf/100)
  
  // Slope
  reg yvar p if year == 2005
  local alpha2005 = strofreal(round(_b[p], 0.01), "%9.2f")
  reg yvar p if year == 2015
  local alpha2015 = strofreal(round(_b[p], 0.01), "%9.2f")
  twoway  (line yvar p if year == 2005, lcolor(blue) lpattern(solid) lwidth(*2)) ///
          (lfit yvar p if year == 2005, lcolor(blue) lpattern(dash)) ///
          (line yvar p if year == 2015, lcolor(red) lpattern(solid) lwidth(*2)) ///
          (lfit yvar p if year == 2015, lcolor(red) lpattern(dash)) ///
          , legend(order(1 "2005 Level (Slope: `alpha2005')" 3 "2015 Level (Slope: `alpha2015')")) ///
          ytitle("log(1-CDF(y{sub:it}))") xtitle("log(y{sub:it})") ylabel(-12(2)-2) xlabel(11/15) name(`subgp'`p', replace)
          
  graph export "$figs_folder/core/labor_paretotail_top`p'_`subgp'.pdf", replace
  
}
}
